SystemOrganization addCategory: #'CodeSight-Tests'!
SystemOrganization addCategory: #'CodeSight-Tracer'!
SystemOrganization addCategory: #'CodeSight-TestTracer'!
SystemOrganization addCategory: #'CodeSight-Browser'!
SystemOrganization addCategory: #'CodeSight-Install'!
SystemOrganization addCategory: #'CodeSight-Analyzing'!
SystemOrganization addCategory: #CodeSight!
SystemOrganization addCategory: #'CodeSight-TestExamples'!
SystemOrganization addCategory: #'CodeSight-View'!
SystemOrganization addCategory: #'CodeSight-Text'!

ListItemWrapper subclass: #CSTestCaseItemWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSTestCaseItemWrapper methodsFor: 'as yet unclassified' stamp: 'mt 12/16/2009 12:07'!
asString

	^ item name! !

!CSTestCaseItemWrapper methodsFor: 'as yet unclassified' stamp: 'mt 12/16/2009 11:56'!
contents

	^ item value collect: [:test |
		CSTestItemWrapper with: test]! !

ListItemWrapper subclass: #CSTestItemWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSTestItemWrapper methodsFor: 'as yet unclassified' stamp: 'mt 12/16/2009 11:57'!
asString

	^ item selector asString! !

!Debugger methodsFor: '*codesight' stamp: 'bs 4/16/2009 12:06'!
debug: label executeBefore: aBlock
	"Open a full DebuggerView."
	Smalltalk isMorphic
		ifTrue:
			[self openFullMorphicLabel: label.
			 CSStepper new executeInNextStep: [aBlock value: self].
			 errorWasInUIProcess := Project spawnNewProcessIfThisIsUI: interruptedProcess.
			 ^self].
! !

!Debugger methodsFor: '*codesight' stamp: 'bs 4/16/2009 11:16'!
restartAfterMethodChanged 
	"The retrieved information has changed and its source must now be  
	updated. In this case, the retrieved information is the method of the  
	selected context."
	| selector classOfMethod category h ctxt |
	
	h := self selectedContext finalBlockHome.
	self resetContext: h.
	classOfMethod := self selectedClass.
	category := self selectedMessageCategoryName.
	selector := self selectedMessageName.
	
	contents := classOfMethod sourceCodeAt: selector.
	ctxt := interruptedProcess popTo: self selectedContext.
	interruptedProcess suspendedContext privRefreshWith: (classOfMethod compiledMethodAt: selector).
	"interruptedProcess
		restartTopWith: (classOfMethod compiledMethodAt: selector)."
	contextVariablesInspector object: nil.
	theMethodNode := Preferences browseWithPrettyPrint
		ifTrue: [ctxt methodNodeFormattedAndDecorated: Preferences colorWhenPrettyPrinting]
		ifFalse: [ctxt methodNode].
	sourceMap := theMethodNode sourceMap.
	tempNames := theMethodNode tempNames.
		
	self resetContext: ctxt.
	self changed.
	^ true! !

!TestResult methodsFor: '*CodeSight' stamp: 'bs 12/22/2009 15:25'!
removeIfPresent: aTestMethod
	|selector class|
	selector := aTestMethod selector.
	class := aTestMethod methodClass.
	{errors. failures. passed} do: [ :aTestSet | 
		aTestSet 
			select: [ :testCase | (testCase selector == selector) and: [testCase class == class] ]
			thenDo: [ :testCase | aTestSet remove: testCase. ] ].! !

!TestResult methodsFor: '*CodeSight' stamp: 'bs 12/22/2009 15:22'!
removeIfPresent: selector inClass: class
	{errors. failures. passed} do: [ :aTestSet | 
		aTestSet 
			select: [ :testCase | (testCase selector = selector) and: [testCase class = class] ]
			thenDo: [ :testCase | aTestSet remove: testCase. ] ].! !

!TestResult methodsFor: '*CodeSight-override' stamp: 'bs 12/21/2009 12:36'!
runCase: aTestCase
	| testCasePassed |
	testCasePassed := true.
	[[aTestCase runCase] 
			on: self class failure
			do: 
				[:signal | 
				failures add: aTestCase.
				testCasePassed := false.
				aTestCase recordRunFailedWith: signal.
				signal return: false]]
					on: self class error
					do:
						[:signal |
						errors add: aTestCase.
						testCasePassed := false.
						aTestCase recordRunFailedWith: signal.
						signal return: false].
	testCasePassed 
		ifTrue: [
			passed add: aTestCase.
			aTestCase recordRunWasSuccessful]! !

!BreakpointManager class methodsFor: '*codesight' stamp: 'bs 4/16/2009 10:27'!
compileWithSelfRemovingBreakpoint: aSymbol in: aClass 
	"Compile and return a new method containing a break statement"

	| source node method |
	source := self sourceWithSelfRemovingBreakpointFor: aSymbol in: aClass.
	node := aClass compilerClass new
		compile: source
		in: aClass 
		notifying: nil 
		ifFail: [self error: '[breakpoint] unable to install breakpoint'].
	node isNil ifTrue: [^nil].
	"dunno what the arguments mean..."
	method := node generate.
	^method! !

!BreakpointManager class methodsFor: '*codesight' stamp: 'bs 4/16/2009 10:26'!
installSelfRemovingBreakpointInClass: aClass selector: aSymbol 
	"Install a new method containing a breakpoint.
	The receiver will remember this for unstalling it later"

	| breakMethod |
	breakMethod _ self compileWithSelfRemovingBreakpoint: aSymbol in: aClass.
	breakMethod isNil
		ifTrue: [^ nil].
	self installed at: breakMethod put: aClass >> aSymbol. "old method"
	aClass basicAddSelector: aSymbol withMethod: breakMethod.! !

!BreakpointManager class methodsFor: '*codesight' stamp: 'bs 4/16/2009 10:28'!
sourceWithSelfRemovingBreakpointFor: aSymbol in: aClass 
	"Compose new source containing a break statement (currently it will be the first,
	later we want to insert it in any place)"

	| oldSource methodNode breakOnlyMethodNode sendBreakMessageNode |
	oldSource := aClass sourceCodeAt: aSymbol.
	methodNode := aClass compilerClass new
		compile: oldSource
		in: aClass 
		notifying: nil 
		ifFail: [self error: '[breakpoint] unable to install breakpoint'].
	breakOnlyMethodNode := aClass compilerClass new
		compile: 'temporaryMethodSelectorForBreakpoint
self debugAndUninstallBreakpointBefore.
^self'
		in: aClass 
		notifying: nil 
		ifFail: [self error: '[breakpoint] unable to install breakpoint'].
	sendBreakMessageNode := breakOnlyMethodNode block statements first.
	methodNode block statements addFirst: sendBreakMessageNode.
	^methodNode printString
	! !

!BreakpointManager class methodsFor: '*codesight' stamp: 'bs 4/15/2009 23:08'!
unInstallInClass: aClass selector: aSymbol
	self unInstall: (aClass compiledMethodAt: aSymbol)! !

!MenuIcons class methodsFor: '*codesight' stamp: 'mt 5/3/2009 20:40'!
csAcceptIcon

	^ Icons
			at: #'csAccept'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self csAcceptIconContents readStream) ].! !

!MenuIcons class methodsFor: '*codesight' stamp: 'mt 5/3/2009 20:39'!
csAcceptIconContents

	^ 'iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAKfSURBVDjLpZPrS1NhHMf9O3bO
dmwDCWREIYKEUHsVJBI7mg3FvCxL09290jZj2EyLMnJexkgpLbPUanNOberU5taUMnHZUULM
velCtWF0sW/n7MVMEiN64AsPD8/n83uucQDi/id/DBT4Dolypw/qsz0pTMbj/WHpiDgsdSUy
UmeiPt2+V7SrIM+bSss8ySGdR4abQQv6lrui6VxsRonrGCS9VEjSQ9E7CtiqdOZ4UuTqnBHO
1X7YXl6Daa4yGq7vWO1D40wVDtj4kWQbn94myPGkCDPdSesczE2sCZShwl8CzcwZ6NiUs6n2
nYX99T1cnKqA2EKui6+TwphA5k4yqMayopU5mANV3lNQTBdCMVUA9VQh3GuDMHiVcLCS3J4j
SLhCGmKCjBEx0xlshjXYhApfMZRP5CyYD+UkG08+xt+4wLVQZA1tzxthm2tEfD3JxARH7Qkb
D1ZuozaggdZbxK5kAIsf5qGaKMTY2lAU/rH5HW3PLsEwUYy+YCcERmIjJpDcpzb6l7th9KtQ
69fi09ePUej9l7cx2DJbD7UrG3r3afQHOyCo+V3QQzE35pvQvnAZukk5zL5qRL59jsKbPzdh
eXoBZc4saFhBS6AO7V4zqCpiawuptwQG+UAa7Ct3UT0hh9p9EnXT5Vh6t4C22QaUDh6HwnEC
OmcO7K+6kW49DKqS2DrEZCtfuI+9GrNHg4fMHVSO5kE7nAPVkAxKBxcOzsajpS4Yh4ohUPPW
KTUh3PaQEptIOr6BiJjcZXCwktaAGfrRIpwblqOV3YKdhfXOIvBLeREWpnd8ynsaSJoyESFp
hwTtfjN6X1jRO2+FxWtCWksqBApeiFIR9K6fiTpPiigDoadqCEag5YUFKl6Yrciw0VOlhOiv
v/Ff8wtn0KzlebrUYwAAAABJRU5ErkJggg=='! !

!MenuIcons class methodsFor: '*codesight' stamp: 'rkrk 9/22/2009 07:08'!
csBlankBarIcon

	^ Form extent: 57@24.! !

!MenuIcons class methodsFor: '*codesight' stamp: 'rkrk 9/22/2009 06:55'!
csGreenBarIcon

	^ Icons
			at: #'csGreenBar'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self csGreenBarIconContents readStream) ].! !

!MenuIcons class methodsFor: '*codesight' stamp: 'rkrk 9/22/2009 06:54'!
csGreenBarIconContents

	^  'iVBORw0KGgoAAAANSUhEUgAAADkAAAAYCAYAAABA6FUWAAAACXBIWXMAAAsTAAALEwEAmpwY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'! !

!MenuIcons class methodsFor: '*codesight' stamp: 'rkrk 9/30/2009 05:17'!
csHourGlassIcon

	^ Icons
			at: #'csHourGlassIcon'
			ifAbsentPut: [((CursorWithMask
	extent: 16@16
	depth: 1
	fromArray: #( 0 2147352576 1073872896 537133056 268959744 242221056 132120576 62914560 62914560 77594624 143654912 281542656 568590336 1207566336 2147352576 0)
	offset: -1@-1) setMaskForm: (Form
	extent: 16@16
	depth: 1
	fromArray: #( 4294901760 4294901760 4294901760 4162781184 2147352576 1073479680 536346624 267386880 267386880 536346624 1073479680 2147352576 4294901760 4294901760 4294901760 4294901760)
	offset: 0@0)) as8BitColorForm].! !

!MenuIcons class methodsFor: '*codesight' stamp: 'rkrk 9/22/2009 06:56'!
csRedBarIcon

	^ Icons
			at: #'csRedBar'
			ifAbsentPut:[ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: self csRedBarIconContents readStream) ].! !

!MenuIcons class methodsFor: '*codesight' stamp: 'rkrk 9/22/2009 06:56'!
csRedBarIconContents

	^   'iVBORw0KGgoAAAANSUhEUgAAADkAAAAYCAYAAABA6FUWAAAACXBIWXMAAAsTAAALEwEAmpwY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'! !

SimpleHierarchicalListMorph subclass: #CSTestTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSTestTree class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 23:27'!
on: someTestCases

	^ self
			on: [Array with:  (MorphWithSubmorphsWrapper with: World)]
			list: #value
			selected: nil
			changeSelected: nil
			menu: nil
			keystroke: nil! !

!CSTestTree methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 23:40'!
initialize

	super initialize.
	self
		hResizing: #spaceFill;
		vResizing: #rigid;
		borderWidth: 0;
		extent: (CSActivityMorph defaultWidth - 10 ) @ 50;
		color: (Color white alpha: 0.7).! !

!CSTestTree methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 23:18'!
scrollBarThickness

	^ 5! !

!CSTestTree methodsFor: 'as yet unclassified' stamp: 'mt 12/16/2009 12:11'!
setTests: someTestCases

	| g groups |
	g := IdentityDictionary new.
	someTestCases do: [:test |
		(g includesKey: test class)
			ifFalse: [g at: test class put: OrderedCollection new].
		(g at: test class) add: test].
	
	groups := OrderedCollection new.
		g associationsDo: [:group | groups add: (CSTestCaseItemWrapper with: group)].
		
	self model: [groups].
	getListSelector := #value.
	self list: self getList.! !

Morph subclass: #CSActivityMorph
	instanceVariableNames: 'label bgColor isFinished'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSActivityMorph class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:59'!
defaultWidth

	^ CSActivityMorphList defaultWidth - (CSActivityMorphList defaultInset * 2)! !

!CSActivityMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 22:57'!
activate
	"Moves this activity in the owning list to visualize its state."

	self owner ifNotNilDo: [:activityMorphList |
		activityMorphList activateActivity: self].! !

!CSActivityMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 22:12'!
bgColor

	^ bgColor ifNil: [^ Color transparent]! !

!CSActivityMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 22:12'!
bgColor: aColor

	bgColor := aColor.
	self color: self bgColor.! !

!CSActivityMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 22:55'!
contextMenu

	| menu |
	menu := MenuMorph entitled: 'Activity'.
	menu
		add: 'Activate'
		target: self
		selector: #activate
		enabled: true
		icon: nil.
	menu
		add: 'Finish'
		target: self
		selector: #finish
		enabled: true
		icon: nil.
	menu addLine.
	menu
		add: 'Delete'
		target: self
		selector: #delete
		enabled: true
		icon: nil.
	^ menu! !

!CSActivityMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 22:40'!
delete

	super delete.
	self changed: #activityDeleted.! !

!CSActivityMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 22:30'!
finish
	"Moves this activity in the owning list to visualize its state."

	self owner ifNotNilDo: [:activityMorphList |
		activityMorphList finishActivity: self].! !

!CSActivityMorph methodsFor: 'event handling' stamp: 'mt 12/15/2009 22:53'!
handleMouseEnter: evt
	"Hightlight the morph."

	super handleMouseEnter: evt.
	self isFinished
		ifFalse: [self color: Color blue muchLighter].! !

!CSActivityMorph methodsFor: 'event handling' stamp: 'mt 12/15/2009 22:12'!
handleMouseLeave: evt

	super handleMouseLeave: evt.
	self color: self bgColor.! !

!CSActivityMorph methodsFor: 'event handling' stamp: 'mt 12/15/2009 22:42'!
handlesMouseDown: evt

	^ true! !

!CSActivityMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 23:03'!
highlightWithColor: aColor
	"Reset the color then wait some time and restore the old one. Can be used for flashing effects."
	
	| old |
	old := self bgColor.
	self bgColor: aColor.
	[(Delay forMilliseconds: 500) wait. self bgColor: old] fork.! !

!CSActivityMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/16/2009 11:08'!
initialize

	super initialize.
	self
		changeTableLayout;
		hResizing: #rigid;
		vResizing: #shrinkWrap;
		listDirection: #topToBottom;
		layoutInset: 3;
		extent: self class defaultWidth @ 50;
		bgColor: Color red lighter;
		addMorph: self label.! !

!CSActivityMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 22:51'!
isFinished

	^ isFinished ifNil: [false]! !

!CSActivityMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 22:51'!
isFinished: aBoolean

	isFinished := aBoolean.! !

!CSActivityMorph methodsFor: 'accessing' stamp: 'mt 12/16/2009 11:08'!
label

	^ label ifNil: [label := TextMorph new lock; hResizing: #spaceFill]! !

!CSActivityMorph methodsFor: 'event handling' stamp: 'mt 12/15/2009 22:52'!
mouseDown: evt
	"Do nothing upon mouse-down except inform the hand to watch for a 
	double-click; wait until an ensuing click:, doubleClick:, or drag: 
	message gets dispatched"
	
	(self isFinished not and: [evt yellowButtonPressed] )
		ifTrue: [
			self contextMenu popUpInWorld.
			evt wasHandled: true].! !

!CSActivityMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 21:29'!
setLabelText: aString

	self label contentsWrapped: aString! !

CSActivityMorph subclass: #CSAddingMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

CSActivityMorph subclass: #CSRefactoringMorph
	instanceVariableNames: 'method tests tree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

CSRefactoringMorph class
	instanceVariableNames: 'someTestCases'!

CSRefactoringMorph class
	instanceVariableNames: 'someTestCases'!

!CSRefactoringMorph class methodsFor: 'as yet unclassified' stamp: 'bs 12/14/2009 10:32'!
on: aCompiledMethod
	^ self new 
		method: aCompiledMethod;
		addMorph: (StringMorph contents: aCompiledMethod qualifiedName);
		extent: 200 @ 50; 
		yourself.! !

!CSRefactoringMorph methodsFor: 'as yet unclassified' stamp: 'mt 12/16/2009 11:38'!
initialize

	super initialize.
	self addMorphBack: self tree.
	self invalidRect: self fullBounds.! !

!CSRefactoringMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 21:39'!
method
	"^ <CompiledMethod>"

	^ method! !

!CSRefactoringMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 21:50'!
method: aCompiledMethod

	method := aCompiledMethod.
	self updateLabel.! !

!CSRefactoringMorph methodsFor: 'accessing' stamp: 'mt 12/15/2009 21:39'!
tests
	"^ <Collection with TestCases>"
	
	^ tests! !

!CSRefactoringMorph methodsFor: 'accessing' stamp: 'mt 12/16/2009 12:11'!
tests: someTestCases

	tests := someTestCases.
	self tree
		setTests: someTestCases;
		expandRoots.! !

!CSRefactoringMorph methodsFor: 'accessing' stamp: 'mt 12/16/2009 11:02'!
tree

	^ tree ifNil: [tree := CSTestTree new]! !

!CSRefactoringMorph methodsFor: 'accessing' stamp: 'mt 12/16/2009 11:10'!
tree: anObject

	tree := anObject.! !

!CSRefactoringMorph methodsFor: 'updating' stamp: 'mt 12/15/2009 21:50'!
updateLabel

	self method
		ifNil: [self setLabelText: 'n/a']
		ifNotNil: [self setLabelText: ('{1} >> {2}' format: {self method methodClass name. self method selector})]! !

Morph subclass: #CSActivityMorphList
	instanceVariableNames: 'separator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSActivityMorphList class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 22:06'!
defaultInset

	^ 0! !

!CSActivityMorphList class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:53'!
defaultWidth
	"Calculate the correct width out of default values of the environment."

	^ CSTddMonitor defaultWidth
	 - SwaGeneralScrollPane defaultScrollbarThickness
	 - (CSTddMonitor defaultInset * 2)! !

!CSActivityMorphList class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 22:25'!
numberOfFinishedTasksToKeep

	^ 4! !

!CSActivityMorphList methodsFor: 'activity handling' stamp: 'mt 12/15/2009 23:00'!
activateActivity: anActivityMorph
	"Changes the position of the given activity morph to visualize its state."

	self
		removeMorph: anActivityMorph;
		addMorph: anActivityMorph after: self separator.
		
	anActivityMorph highlightWithColor: Color yellow.! !

!CSActivityMorphList methodsFor: 'activity handling' stamp: 'mt 12/16/2009 11:20'!
addActivity: anActivityMorph

	self addMorphBack: anActivityMorph.
	anActivityMorph addDependent: self.! !

!CSActivityMorphList methodsFor: 'private' stamp: 'mt 12/15/2009 22:29'!
discardOldActivities
	"Only keep a specific number of finished activities in the list."
	
	| behind frontMorphs |
	behind := false.
	frontMorphs := OrderedCollection new.
	
	self submorphs do: [:morph |
		morph == self separator
			ifTrue: [behind := true]
			ifFalse: [behind ifFalse: [frontMorphs add: morph]]].
		
	[frontMorphs size > self class numberOfFinishedTasksToKeep]
		whileTrue: [self removeMorph: frontMorphs first. frontMorphs remove: frontMorphs first]! !

!CSActivityMorphList methodsFor: 'activity handling' stamp: 'mt 12/15/2009 23:04'!
finishActivity: anActivityMorph
	"Changes the position of the given activity morph to visualize its state."

	self
		removeMorph: anActivityMorph;
		addMorph: anActivityMorph inFrontOf: self separator;
		discardOldActivities.
		
	anActivityMorph
		bgColor: Color gray lighter;
		isFinished: true;
		highlightWithColor: Color yellow.! !

!CSActivityMorphList methodsFor: 'initialization' stamp: 'mt 12/15/2009 22:08'!
initialize

	super initialize.
	self
		changeTableLayout;
		hResizing: #rigid;
		vResizing: #shrinkWrap;
		listDirection: #topToBottom;
		layoutInset: self class defaultInset;
		cellInset: 5;
		extent: self class defaultWidth @ 10;
		color: Color transparent;
		addMorph: self separator.! !

!CSActivityMorphList methodsFor: 'activity handling' stamp: 'mt 12/16/2009 11:23'!
privateAddMorph: aMorph atIndex: index

	super privateAddMorph: aMorph atIndex: index.
	self invalidRect: self fullBounds.
	self changed: #resized.! !

!CSActivityMorphList methodsFor: 'activity handling' stamp: 'mt 12/15/2009 21:54'!
refactoringActivityOn: aCompiledMethod
	"Return the submorph based of the given method."

	^ self submorphs detect: [:s | s method = aCompiledMethod]! !

!CSActivityMorphList methodsFor: 'accessing' stamp: 'mt 12/15/2009 22:06'!
separator
	"Separator to seperate open and closed activities."

	^ separator ifNil: [separator :=
		Morph new
			extent: CSActivityMorph defaultWidth @ 2;
			color: (Color black alpha: 0.5)]! !

!CSActivityMorphList methodsFor: 'updating' stamp: 'mt 12/16/2009 11:39'!
update: aSymbol

	aSymbol = #activityDeleted
		ifTrue: [
			self invalidRect: self fullBounds.
			self changed: #resized]! !

Morph subclass: #CSTddMonitor
	instanceVariableNames: 'list'
	classVariableNames: 'Instance'
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSTddMonitor class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:48'!
defaultHeight

	^ 500! !

!CSTddMonitor class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:53'!
defaultInset

	^ 7! !

!CSTddMonitor class methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:48'!
defaultWidth

	^ 200! !

!CSTddMonitor class methodsFor: 'instance creation' stamp: 'bs 12/22/2009 13:24'!
instance
	^ Instance
		ifNil: [Instance := super new]! !

!CSTddMonitor class methodsFor: 'instance creation' stamp: 'bs 12/22/2009 13:24'!
open

	^ self instance open.! !

!CSTddMonitor methodsFor: 'activity handling' stamp: 'mt 12/15/2009 22:03'!
addRefactoringActivityOn: aCompiledMethod withTests: someTestCases

	self list addActivity: (CSRefactoringMorph new
		method: aCompiledMethod;
		tests: someTestCases).! !

!CSTddMonitor methodsFor: 'building' stamp: 'bs 12/14/2009 17:42'!
buildWith: aBuilder
	| window |
	window := aBuilder pluggableWindowSpec new
		model: self; "label: self label; extent: self extent;"
		"children: (OrderedCollection new 
			add: ((self buildItemListWith: aBuilder)
				frame: (0.00 @ 0.00 corner: 1.0 @ 1.0 );
				yourself);
			yourself);"
		yourself.
	^ aBuilder build: window.! !

!CSTddMonitor methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:53'!
initialize
	
	super initialize.
	self
		borderWidth: 0;
		color: (Color gray alpha: 0.5);
		changeTableLayout;
		layoutInset: self class defaultInset;
		listDirection: #topToBottom;
		initializeList.! !

!CSTddMonitor methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 22:03'!
initializeList
	
	| scroller |
	scroller := CSActivityScroller new.
	self addMorph: scroller.
	self list: scroller scrollTarget.! !

!CSTddMonitor methodsFor: 'accessing' stamp: 'mt 12/15/2009 21:58'!
list
	^ list! !

!CSTddMonitor methodsFor: 'accessing' stamp: 'mt 12/15/2009 21:58'!
list: anObject
	list := anObject! !

!CSTddMonitor methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:57'!
open

	self
		openInWorld;
		extent: self class defaultWidth @ self class defaultHeight.! !

OBEnhancementMetagraphBuilder subclass: #CSMetagraphBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!CSMetagraphBuilder methodsFor: 'as yet unclassified' stamp: 'bs 5/2/2009 23:41'!
createMethodMetaNode
	| methodMetaNode testMetaNode |
	testMetaNode := OBEnhancementMetaNode named: #CoveringTestCases.
	methodMetaNode := OBEnhancementMetaNode named: #Method.
	methodMetaNode childAt: #coveringTestMethods put: testMetaNode.
	^ methodMetaNode! !

!CSMetagraphBuilder methodsFor: 'as yet unclassified' stamp: 'bs 5/3/2009 09:57'!
methodMetaNode
   ^(self metaNodes
		at: #Method
		ifAbsentPut: [OBEnhancementMetaNode named: #Method])! !

!CSMetagraphBuilder methodsFor: 'as yet unclassified' stamp: 'bs 5/3/2009 10:00'!
populateMethodNode
	| testMetaNode |
	super populateMethodNode.
	testMetaNode := OBEnhancementMetaNode named: #CoveringTestCases.
	self methodMetaNode
		childAt: #coveringTestMethods put: testMetaNode.
! !

OBPackageBrowser subclass: #CSBrowser
	instanceVariableNames: 'isRunning'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!CSBrowser class methodsFor: 'as yet unclassified' stamp: 'bs 5/2/2009 23:32'!
buildMetagraphOn: root class: classSel comment: commentSel metaclass: metaclassSel 
     | metaGraph |
     metaGraph := CSMetagraphBuilder
		               on: root class: classSel comment: commentSel metaclass: metaclassSel.
	metaGraph classMetaNode doubleClickCommandClass: OBEnhancementCmdBrowseHierarchy.
	metaGraph metaclassMetaNode doubleClickCommandClass: OBEnhancementCmdBrowseHierarchy.
	"metaGraph methodMetaNode doubleClickCommandClass: OBCmdSelectMethodCategory."
	^metaGraph execute ! !

!CSBrowser class methodsFor: 'as yet unclassified' stamp: 'bs 5/2/2009 20:52'!
defaultMetaNode
     | env smartGroup package cats classes builder |
	env := OBMetaNode named: 'Environment'.
	package := OBMetaNode named: 'Package'.
	package
	      displaySelector: #indentedName;
	      ancestrySelector: #isDescendantOfPackage:.
	cats := OBMetaNode named: 'Cats'.
	cats
	      displaySelector: #nameWithoutPrefix;	
	      ancestrySelector: #isDescendantOfClassCat:.	
	(package childAt: #classCategories put: cats) navigateTree.
	
	self buildMetagraphOn: package.
	"self buildMetagraphOn: cats."
     (package edges reject: [:edge | edge selector = #classCategories]) do: [:edge | cats addEdge: edge].
	package filters do: [:filter | cats addFilter: filter].

	classes := OBMetaNode named: 'Classes'.
	classes
		displaySelector: #indentedName;
		ancestrySelector: #isDescendantOfClass:.
	builder := CSMetagraphBuilder
		         on: classes 
	              class: #packagesForClassClass 
	              comment: #packagesForClassComment 
	              metaclass: #packagesForClassMetaclass.
	builder execute.
	builder classMetaNode removeFilter: OBClassInheritanceFilter new; displaySelector: #indentedName.		
	builder metaclassMetaNode removeFilter: OBClassInheritanceFilter new; displaySelector: #indentedName.		
	builder classCommentMetaNode removeFilter: OBClassInheritanceFilter new; displaySelector: #indentedName.							
	classes removeFilter: OBQueryingClassSortFilter new.
	
     smartGroup := OBMetaNode named: 'SmartGroup'.
     smartGroup
		displaySelector: #indentedName.
	(package edges reject: [:edge | edge selector = #classCategories]) do: [:edge | smartGroup addEdge: edge].
	smartGroup addFilter: OBModalFilter new;
		          addFilter: OBClassSortFilter new.
	
     env
		childAt: #packages
			labeled: 'pkgs'
			put: package;
		childAt: #classesHierarchically
			labeled: 'hier.'
			put: classes;				
		childAt: #smartGroups
			labeled: 'smart groups'
			put: smartGroup;
		addFilter: OBModalFilter new.
     ^env! !

!CSBrowser class methodsFor: 'as yet unclassified' stamp: 'bs 4/30/2009 23:12'!
paneCount
	^ 5! !

!CSBrowser class methodsFor: 'as yet unclassified' stamp: 'rkrk 10/6/2009 13:35'!
title
	^ 'Browser'! !

!CSBrowser methodsFor: 'autotdd - test runner dependence' stamp: 'rkrk 9/22/2009 03:23'!
addTestRunnerDependence

	self testRunner addDependent: self.! !

!CSBrowser methodsFor: 'commands' stamp: 'rkrk 9/22/2009 02:15'!
cmdShowTestResult
	^ CSCmdShowTestResult! !

!CSBrowser methodsFor: 'accessing' stamp: 'rkrk 9/30/2009 02:44'!
isRunning

	^ isRunning ifNil: [isRunning := false].! !

!CSBrowser methodsFor: 'opening' stamp: 'rkrk 9/22/2009 03:23'!
open

	super open.
	self addTestRunnerDependence.! !

!CSBrowser methodsFor: 'as yet unclassified' stamp: 'bs 5/4/2009 21:43'!
selectionChanged: announcement
	
	| node testMethod testClass testSelector testSuite |	
	node := announcement node.
	(node isNil not and: [ node isKindOf: OBCoveringTestCaseNode]) ifTrue: [
		testMethod := node testMethod.
		testClass := testMethod methodClass.
		testSelector := testMethod selector.
		testSuite := testClass suiteWithSelectorsMatching: testSelector.
		[
		BreakpointManager installSelfRemovingBreakpointInClass: (node coveredMethod methodClass) selector: (node coveredMethod selector).
		[ testSuite run ] ensure: [ 
			BreakpointManager unInstallInClass: (node coveredMethod methodClass) selector: (node coveredMethod selector). ].
		] forkAt: Processor activePriority.].! !

!CSBrowser methodsFor: 'as yet unclassified' stamp: 'bs 5/4/2009 20:48'!
subscribe
	super subscribe.
	self announcer
		observe: OBSelectionChanged send: #selectionChanged: to: self.! !

!CSBrowser methodsFor: 'autotdd - test runner dependence' stamp: 'rkrk 9/22/2009 03:21'!
testRunner

	^ CSTestRunner currentAutoTestRunner! !

!CSBrowser methodsFor: 'autotdd - test runner dependence' stamp: 'rkrk 9/30/2009 05:07'!
update: anAspect

	super update: anAspect.
	anAspect = #runnerClosed ifTrue: [
		"old runner is closed by user. Create a new one and register"
		self testRunner addDependent: self.
		^self].
	anAspect = #startTestRun ifTrue: [isRunning := true].
	anAspect = #endTestRun ifTrue: [isRunning := false].
	(#(newTestResult startTestRun endTestRun) includes: anAspect) ifFalse: [^ self].
	self announce: OBNodeChanged
	
	! !

OBClassAwareNode subclass: #OBCoveringTestCaseBaseNode
	instanceVariableNames: 'coveredMethodNode definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!OBCoveringTestCaseBaseNode methodsFor: 'accessing' stamp: 'test 9/22/2009 16:38'!
coveredMethod
	^ coveredMethodNode compiledMethod! !

!OBCoveringTestCaseBaseNode methodsFor: 'accessing' stamp: 'test 9/22/2009 16:38'!
coveredMethodNode
	^ coveredMethodNode ! !

!OBCoveringTestCaseBaseNode methodsFor: 'accessing' stamp: 'test 9/22/2009 16:37'!
coveredMethodNode: anObject
	coveredMethodNode := anObject.
	definition := coveredMethodNode definition.! !

!OBCoveringTestCaseBaseNode methodsFor: 'accessing' stamp: 'test 9/22/2009 16:38'!
definition
	^ definition! !

OBCoveringTestCaseBaseNode subclass: #OBCoveringTestCaseNode
	instanceVariableNames: 'testMethod statistics'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!OBCoveringTestCaseNode class methodsFor: 'as yet unclassified' stamp: 'bs 5/4/2009 21:50'!
on: testMethod statistic: statistics coveredMethodNode: aCoveredMethodNode
	^ self new 
		testMethod: testMethod;
		statistics: statistics;
		coveredMethodNode: aCoveredMethodNode.! !

!OBCoveringTestCaseNode methodsFor: 'as yet unclassified' stamp: 'test 9/22/2009 18:15'!
name
	^ self testMethod methodClass name, ' >> #',self testMethod selector! !

!OBCoveringTestCaseNode methodsFor: 'accessing' stamp: 'bs 5/4/2009 21:21'!
statistics
	^ statistics! !

!OBCoveringTestCaseNode methodsFor: 'accessing' stamp: 'bs 5/4/2009 21:21'!
statistics: anObject
	statistics := anObject! !

!OBCoveringTestCaseNode methodsFor: 'accessing' stamp: 'bs 5/4/2009 20:53'!
testMethod
	^ testMethod! !

!OBCoveringTestCaseNode methodsFor: 'accessing' stamp: 'bs 5/4/2009 20:53'!
testMethod: anObject
	testMethod := anObject! !

OBCoveringTestCaseBaseNode subclass: #OBCoveringTestCaseSeparatorNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!OBCoveringTestCaseSeparatorNode class methodsFor: 'as yet unclassified' stamp: 'test 9/22/2009 16:36'!
coveredMethodNode: aCoveredMethodNode
	^ self new 
		coveredMethodNode: aCoveredMethodNode.! !

!OBCoveringTestCaseSeparatorNode methodsFor: 'as yet unclassified' stamp: 'test 9/22/2009 21:21'!
name
	^ ('---------', String crlf, '---') asText! !

OBTextMorphEditorWithShout subclass: #CSEditor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!CSEditor methodsFor: 'initialize-release' stamp: 'bs 12/21/2009 14:46'!
changeParagraph: aParagraph
	super changeParagraph: aParagraph! !

SwaGeneralScrollPane subclass: #CSActivityScroller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSActivityScroller methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:46'!
hScrollbarNeeded
	"Never show the horizontal scrollbar."
	
	^ false! !

!CSActivityScroller methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 21:15'!
initialize

	super initialize.
	self
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		scrollTarget: CSActivityMorphList new.
		
	self scrollTarget addDependent: self.! !

!CSActivityScroller methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:56'!
newVScrollbar
	"Answer a new vertical scrollbar."

	^CSActivityScrollBar new
		model: self;
		setValueSelector: #vScrollbarValue:! !

!CSActivityScroller methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 21:14'!
update: aSymbol

	aSymbol = #resized
		ifTrue: [self updateScrollbars]! !

!CSActivityScroller methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:46'!
vScrollbarNeeded
	"Always show the vertical scrollbar."

	^ true! !

TestCase subclass: #CSAsyncAutoRunTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSAsyncAutoRunTests methodsFor: 'as yet unclassified' stamp: 'bs 9/13/2009 14:55'!
testForOnlyOneTestRunnerInTddAutoRunMode
	| runner1 runner2 runner3 |
	runner1 := CSTestRunner new.
	runner2 := CSTestRunner new.
	runner3 := CSTestRunner new.
	
	[runner1 tddAutoRunStart.
	self should: [runner1 isInTddAutoRunMode].
	self shouldnt: [runner2 isInTddAutoRunMode].
	self shouldnt: [runner3 isInTddAutoRunMode].
	
	runner2 tddAutoRunStart.
	self shouldnt: [runner1 isInTddAutoRunMode].
	self should: [runner2 isInTddAutoRunMode].
	self shouldnt: [runner3 isInTddAutoRunMode].
	
	runner3 tddAutoRunStart.
	self shouldnt: [runner1 isInTddAutoRunMode].
	self shouldnt: [runner2 isInTddAutoRunMode].
	self should: [runner3 isInTddAutoRunMode].
	
	runner3 tddAutoRunStop.
	self shouldnt: [runner1 isInTddAutoRunMode].
	self should: [runner2 isInTddAutoRunMode].
	self shouldnt: [runner3 isInTddAutoRunMode].
	
	runner2 tddAutoRunStop.
	self should: [runner1 isInTddAutoRunMode].
	self shouldnt: [runner2 isInTddAutoRunMode].
	self shouldnt: [runner3 isInTddAutoRunMode]. 
	
	runner1 tddAutoRunStop.
	self shouldnt: [runner1 isInTddAutoRunMode].
	self shouldnt: [runner2 isInTddAutoRunMode].
	self shouldnt: [runner3 isInTddAutoRunMode].
	]
	ensure: [
		runner1 tddAutoRunStop.
		runner2 tddAutoRunStop.
		runner3 tddAutoRunStop.
		]! !

!CSAsyncAutoRunTests methodsFor: 'as yet unclassified' stamp: 'bs 10/7/2009 11:14'!
testRunInBackground
	| testRunner |
	CSAsyncExampleTest resetCollectedMessages.
	self should: [ CSAsyncExampleTest collectedMessagesAsString = '' ].
	
	testRunner := CSTestRunner new.
	testRunner runSuite: CSAsyncExampleTest exampleSuite.
	self should: [ CSAsyncExampleTest collectedMessagesAsString = 'Hello' ].
	
	CSAsyncExampleTest resetCollectedMessages.
	testRunner enableRunningTestsInBackground.
	testRunner runSuite: CSAsyncExampleTest backgroundExampleSuite.
	(Delay forMilliseconds: 100) wait.
	CSTestRunner sourceCodeAccessMonitor critical: [ 
		CSAsyncExampleTest addMessage: 'World'. ].
	self should: [ CSAsyncExampleTest collectedMessagesAsString = 'HelloWorld' ].
	testRunner disableRunningTestsInBackground.! !

!CSAsyncAutoRunTests methodsFor: 'as yet unclassified' stamp: 'bs 9/14/2009 10:39'!
testRunningExampleSuite
	| testRunner |
	CSAsyncExampleTest resetCollectedMessages.
	testRunner := CSTestRunner new.
	testRunner runSuite: (CSAsyncExampleTest exampleSuite).
	self should: [CSAsyncExampleTest collectedMessagesAsString = 'Hello']. ! !

TestCase subclass: #CSAsyncExampleTest
	instanceVariableNames: ''
	classVariableNames: 'CollectedMessages'
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSAsyncExampleTest class methodsFor: 'as yet unclassified' stamp: 'bs 9/13/2009 14:07'!
addMessage: aString
	self collectedMessages add: aString.! !

!CSAsyncExampleTest class methodsFor: 'as yet unclassified' stamp: 'bs 9/14/2009 10:43'!
backgroundExampleSuite
	| aSuite |
	aSuite := TestSuite new.
	aSuite addTest: (self selector: #testRunExampleInBackground).
	^ aSuite.! !

!CSAsyncExampleTest class methodsFor: 'as yet unclassified' stamp: 'bs 9/13/2009 14:02'!
collectedMessages
	^ CollectedMessages ifNil: [CollectedMessages := OrderedCollection new.]! !

!CSAsyncExampleTest class methodsFor: 'as yet unclassified' stamp: 'bs 9/13/2009 14:05'!
collectedMessagesAsString
	| stream |
	stream := WriteStream with: String new.
	self collectedMessages do: [: elem | stream nextPutAll: elem].
	^ stream contents! !

!CSAsyncExampleTest class methodsFor: 'as yet unclassified' stamp: 'bs 9/14/2009 10:39'!
exampleSuite
	| aSuite |
	aSuite := TestSuite new.
	aSuite addTest: (self selector: #testExample).
	^ aSuite.! !

!CSAsyncExampleTest class methodsFor: 'as yet unclassified' stamp: 'bs 9/13/2009 14:03'!
resetCollectedMessages
	CollectedMessages := nil.! !

!CSAsyncExampleTest methodsFor: 'as yet unclassified' stamp: 'bs 9/14/2009 10:36'!
testExample
	self class addMessage: 'Hello'! !

!CSAsyncExampleTest methodsFor: 'as yet unclassified' stamp: 'bs 9/14/2009 16:25'!
testRunExampleInBackground
	| testRunner |
	testRunner := CSTestRunner new.
	testRunner enableRunningTestsInBackground.
	testRunner runSuite: (CSAsyncExampleTest exampleSuite).! !

TestCase subclass: #CSBenchmarkTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSBenchmarkTest class methodsFor: 'as yet unclassified' stamp: 'bs 10/11/2009 22:12'!
wantsRecompilationProgressReported
	"turn off for this clas because it is modified during tests"

	^ false
! !

!CSBenchmarkTest methodsFor: 'as yet unclassified' stamp: 'bs 10/11/2009 22:12'!
testFirstnameAccessors
	
	| aPerson aName |
	aPerson := CSExamplePerson new.
	aName := 'Bastian'.
	aPerson firstname: aName.
	self assert: aPerson firstname = aName.! !

TestCase subclass: #CSExampleFooBarTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestExamples'!

!CSExampleFooBarTest methodsFor: 'as yet unclassified' stamp: 'bs 9/24/2009 08:11'!
testFooBar
	CSExampleFooBar new fooBar! !

TestCase subclass: #CSExampleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestExamples'!

!CSExampleTest class methodsFor: 'as yet unclassified' stamp: 'bs 9/23/2009 20:42'!
wantsRecompilationProgressReported
	"turn off for this clas because it is modified during tests"

	^ false
! !

!CSExampleTest methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 16:01'!
testFailing
	self assert: false.
	! !

!CSExampleTest methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 15:35'!
testFirstnameAccessors
	
	| aPerson aName |
	aPerson := CSExamplePerson new.
	aName := 'Bastian'.
	aPerson firstname: aName.
	self assert: aPerson firstname = aName.
! !

TestCase subclass: #CSFixedExampleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestExamples'!

!CSFixedExampleTest methodsFor: 'as yet unclassified' stamp: 'bs 12/4/2009 14:33'!
testFooState
	|example|
	example := CSFixedExample new.
	example foo.
	self assert: [example state = 'Foo']! !

!CSFixedExampleTest methodsFor: 'as yet unclassified' stamp: 'bs 12/4/2009 14:56'!
testHelloWorld
	|example|
	example := CSFixedExample new.
	example hello.
	self assert: [example world = 'HelloWorld']! !

TestCase subclass: #CSTddMonitorTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSTddMonitorTests methodsFor: 'as yet unclassified' stamp: 'bs 12/13/2009 20:26'!
testOpenTddMonitor
	| monitorWindow |
	monitorWindow := CSTddMonitor open.
	monitorWindow delete.! !

TestCase subclass: #CSTestBase
	instanceVariableNames: 'tddAutoRunTestRunner registeredClasses removedMethods installedMethods createdClasses undoInTearDown registeredPackages unregisteredPackages createdWrappers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

CSTestBase subclass: #CSAutoTddTestBase
	instanceVariableNames: 'testRunner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSAutoTddTestBase methodsFor: 'as yet unclassified' stamp: 'test 9/21/2009 08:52'!
createExamplePersonSubclass
	| aClass |
	aClass := CSExamplePerson subclass: #CSExamplePersonSubclass
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'CodeSight-Tests'.
	self undoInTearDown ifTrue: [
		self createdClasses add: aClass].
	^ aClass! !

!CSAutoTddTestBase methodsFor: 'as yet unclassified' stamp: 'test 9/21/2009 19:51'!
createSuperCallTestCode
	^ 'testSuperCall', String crlf, String tab, 'CSExamplePersonSubclass new asString'.! !

!CSAutoTddTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/7/2009 10:09'!
setUp
	super setUp.
	testRunner := CSTestRunner new.
	testRunner tddAutoRunStart.! !

!CSAutoTddTestBase methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 16:05'!
tearDown
	testRunner tddAutoRunStop.
	testRunner disableRunningTestsInBackground.
	testRunner := nil.
	
	CSExamplePerson methodsDo: [ :method |
		method 
			resetCoveringTestMethods;
			resetFixedTestMethods ].

	super tearDown.! !

!CSAutoTddTestBase methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 14:04'!
testRunner
	^ testRunner! !

CSAutoTddTestBase subclass: #CSFixedTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSFixedTests methodsFor: 'as yet unclassified' stamp: 'bs 12/9/2009 16:57'!
charIntervalIn: codeString fromLine: from toLine: to
	| start end |
	self assert: [ codeString lineCount >= to and: [ codeString lineCount >= from ] ].
	start := 1.
	(1 to: (from - 1)) do: [:lineNo | 
		start := start + (codeString lineNumber: lineNo) size ].
	
	end := start.
	(to to: codeString lineCount) do: [:lineNo |
		end := end + (codeString lineNumber: lineNo) size ].
	
	^ (start to: end)! !

!CSFixedTests methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 10:47'!
testEstablishingFixedRelationship
	| aSelector aMethod source |
	aSelector := #firstname:.
	aMethod := CSExamplePerson >> aSelector.
	source := aMethod getSource.
	aMethod resetFixedTestMethods.

	self undoInTearDownDo: [
		self removeMethod: aMethod. 
		self traceClass: CSExamplePerson].
	
	testRunner runSuite: CSExampleTest suite.
	
	CSExamplePerson compile: source.
	self should: [ (CSExamplePerson >> aSelector) hasCoveringTestMethods ].
	self should: [ (CSExamplePerson >> aSelector) hasFixedTestMethods ].! !

!CSFixedTests methodsFor: 'as yet unclassified' stamp: 'test 12/14/2009 10:32'!
testRefactoringVariations
	| code anExamplePerson variationWrapper selectionInterval |
	
	code := (CSExamplePerson sourceCodeAt: #firstname) asString.
	selectionInterval := self charIntervalIn: code fromLine: 2 toLine: 2.
	self assert: [ code lineCount = 2 ].
	
	anExamplePerson := CSExamplePerson new.
	anExamplePerson firstname: 'Paul'.
	self assert: [ anExamplePerson firstname = 'Paul' ].
	
	variationWrapper := CSRefactoringVariationWrapper 
							on: #firstname 
							inClass: CSExamplePerson 
							withCode: 'firstname'.
	
	variationWrapper installClosestToMethod.
	self deny: [ anExamplePerson firstname = 'Paul' ].
	
	variationWrapper uninstall.
	
	! !

CSAutoTddTestBase subclass: #CSFixturesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 11/27/2009 14:28'!
testAdditionOfOverridingMethod
"	| personsubclass selector method superCallTestSelector superCallTestMethod |
	selector := #asString.
	method := (CSExamplePerson >> selector) clientMethod.
	method 
		resetCoveringTestMethods;
		resetFixedTestMethods.
	
	self undoInTearDownDo: [
		personsubclass := self createExamplePersonSubclass.
		self traceClass: personsubclass.
		self traceClass: CSExamplePerson.
		
		superCallTestSelector := self installMethod: self createSuperCallTestCode 
												inClass: CSExampleTest ].
	
	superCallTestMethod := (CSExampleTest >> superCallTestSelector).
	superCallTestMethod called: method receiver: nil.
	method fixes: superCallTestMethod.
	self should: [ method hasFixedTestMethods ].
		
	personsubclass compile: (CSExamplePerson sourceCodeAt: selector).
	self shouldnt: [ testRunner result passed size = 0 ].
	self should: [ (personsubclass >> selector) hasCoveringTestMethods  ].
	self should: [ (personsubclass >> selector) hasFixedTestMethods  ]."! !

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 12/1/2009 20:32'!
testCleanUpOfRecordingsAfterTestRun
	self fail.! !

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 11/30/2009 21:44'!
testFixedTestMethods
"	| testMethod appMethod |
	appMethod := (CSExamplePerson >> #firstname) clientMethod.
	testMethod := (CSExampleTest >> #testFirstnameAccessors) clientMethod.
	
	appMethod fixes: testMethod. 
	self should: [ appMethod fixedTestMethods size = 1 ].
	appMethod resetFixedTestMethods.	
	self should: [ appMethod fixedTestMethods size = 0 ]."! !

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 11/30/2009 21:44'!
testFixesInDoForEachBlock
"	| testMethod appMethod |
	appMethod := (CSExamplePerson >> #firstname) clientMethod.
	testMethod := (CSExampleTest >> #testFirstnameAccessors) clientMethod.
	appMethod resetAllCoverageData.
	testMethod resetAllCoverageData.
	
	testMethod doForEachCoveredMethod: [:blockTestMethod :blockAppMethod |
		blockAppMethod fixes: blockTestMethod ].
	
	testMethod called: appMethod receiver: nil.
	self should: [ appMethod coveringTestMethods size = 1 ].
	self should: [ appMethod fixedTestMethods size = 1 ].
	
	appMethod resetAllCoverageData.
	testMethod resetAllCoverageData."! !

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 12/1/2009 20:32'!
testNonRecordingOfTestModifications
	self fail.
	self fail. 
	! !

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 11/30/2009 21:43'!
testRecompiling
"	| testMethod |
	testMethod := (CSExampleTest >> #testFirstnameAccessors) clientMethod.
	
	(CSExamplePerson >> #firstname) fixes: testMethod.
	self should: [ (CSExamplePerson >> #firstname) fixedTestMethods size = 1 ].
	CSExamplePerson compile: (CSExamplePerson sourceCodeAt: #firstname).
	self should: [ (CSExamplePerson >> #firstname) fixedTestMethods size = 1 ]."! !

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 11/27/2009 14:26'!
testRefactoring
"	| personsubclass selector method superCallTestSelector subMethod |
	selector := #asString.
	method := (CSExamplePerson >> selector) clientMethod.
	method 
		resetCoveringTestMethods;
		resetFixedTestMethods.
	
	self undoInTearDownDo: [
		personsubclass := self createExamplePersonSubclass.
		self traceClass: personsubclass.
		self traceClass: CSExamplePerson.
		
		superCallTestSelector := self installMethod: self createSuperCallTestCode 
												inClass: CSExampleTest ].
	
	testRunner reset.
	personsubclass compile: selector asString.
	self should: [ testRunner result passed size = 1 ].
	
	subMethod := (personsubclass >> selector) clientMethod.
	self should: [ subMethod hasCoveringTestMethods ].
	subMethod fixes: (CSExampleTest >> superCallTestSelector).
	self should: [ subMethod hasFixedTestMethods ].
	
	testRunner reset.
	personsubclass compile: (CSExamplePerson sourceCodeAt: selector).
	self should: [ testRunner result passed size = 1 ].
	self should: [ (CSExamplePerson >> selector) hasCoveringTestMethods ].
	self should: [ (CSExamplePerson >> selector) hasFixedTestMethods ].
	
	testRunner reset.
	personsubclass compile: selector asString.
	self should: [ testRunner result passed size = 1 ].
	self shouldnt: [ (CSExamplePerson >> selector) hasFixedTestMethods ]."! !

!CSFixturesTest methodsFor: 'as yet unclassified' stamp: 'bs 11/27/2009 14:27'!
testRemoveOfOverriddenMethod
"	| personsubclass selector method superCallTestSelector |
	selector := #asString.
	method := (CSExamplePerson >> selector) clientMethod.
	method 
		resetCoveringTestMethods;
		resetFixedTestMethods.
	
	self undoInTearDownDo: [
		personsubclass := self createExamplePersonSubclass.
		personsubclass compile: (CSExamplePerson sourceCodeAt: selector).
		self traceClass: personsubclass.
		self traceClass: CSExamplePerson.
		
		superCallTestSelector := self installMethod: self createSuperCallTestCode 
												inClass: CSExampleTest ].
	
	method fixes: (CSExampleTest >> superCallTestSelector).
		
	self undoInTearDownDo: [
		self removeMethod: method inClass: CSExamplePerson].	
	
	self should: [ (personsubclass >> selector) hasFixedTestMethods ]"! !

CSAutoTddTestBase subclass: #CSTddAutoRunTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSTddAutoRunTests methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 14:40'!
createFooBarTestClass
	| aClass |
	aClass := TestCase subclass: #CSFooBarTest
					instanceVariableNames: ''
					classVariableNames: ''
					poolDictionaries: ''
					category: 'CodeSight-Tests'.
	self undoInTearDown ifTrue: [
		self createdClasses add: aClass].
	^ aClass.! !

!CSTddAutoRunTests methodsFor: 'as yet unclassified' stamp: 'bs 9/11/2009 23:18'!
createSubclassTestCode
	^ 'testFirstnameAccessorsForPersonAndAllSubclasses
			| aPerson aName |
			CSExamplePerson withAllSubclassesDo: [:aPersonClass | 
			aPerson := aPersonClass new.
			aName := #Bastian asString.
			aPerson firstname: aName.
			self assert: aPerson firstname = aName. ]'! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 10/6/2009 18:44'!
testAddAndChangeOfNonTestMethodsInTestClasses
	| selector |
	[	selector := #setUp.
		CSExampleTest compile: selector, String crlf.
		self should: [testRunner result passed isEmpty].
		CSExampleTest compile: selector, String crlf.
		CSExampleTest compile: selector, String crlf.
		self should: [testRunner result passed isEmpty]]
	ensure: [
		(CSExampleTest includesSelector: selector) 
			ifTrue: [CSExampleTest removeSelector: selector]].
	! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/21/2009 18:54'!
testAdditionAndRemovalOfOverriddenMethod
	| methodSelector methodSource personsubclass superCallTestSelector |
	CSExamplePerson methodsDo: [ :method | 
		method 
			resetCoveringTestMethods;
			resetFixedTestMethods. ].
	
	methodSelector := #asString.
	methodSource := CSExamplePerson sourceCodeAt: methodSelector.
	
	self undoInTearDownDo: [
		personsubclass := self createExamplePersonSubclass.
		personsubclass compile: methodSource.
		superCallTestSelector := self installMethod: self createSuperCallTestCode inClass: CSExampleTest.
		
		self traceClass: CSExamplePerson.
		self traceClass: personsubclass ].
	
	testRunner reset; runSuite: CSExampleTest suite.
	self shouldnt: [ (CSExamplePerson >> methodSelector) coveringTestMethods isEmpty ].
	self shouldnt: [ (personsubclass >> methodSelector) coveringTestMethods isEmpty ].
		
	testRunner reset.
	self undoInTearDownDo: [
		self removeMethod: (CSExamplePerson >> methodSelector) ].
		
	self should: [ testRunner result passed size = 1 ].
	self shouldnt: [ (personsubclass >> methodSelector) coveringTestMethods isEmpty ].
		
	testRunner reset.
	CSExamplePerson compile: methodSource.
	self should: [ testRunner result passed size = 1].
	self shouldnt: [ (CSExamplePerson >> methodSelector) coveringTestMethods isEmpty ].
	self shouldnt: [ (personsubclass >> methodSelector) coveringTestMethods isEmpty ].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 11/26/2009 16:05'!
testAdditionAndRemovalOfSubclassesWithOverridingMethods
	| selector source subclass subclassTestSelector |
	CSExamplePerson methodsDo: [ :method |
		method 
			resetCoveringTestMethods;
			resetFixedTestMethods ].
	
	selector := #firstname.
	source := CSExamplePerson sourceCodeAt: selector.
	
	self undoInTearDownDo: [
		subclass := self createExamplePersonSubclass.
		self traceClass: CSExamplePerson.
		self traceClass: subclass.
		subclassTestSelector := self installMethod: self createSubclassTestCode 
											  inClass: CSExampleTest ].
	
	testRunner reset; runSuite: CSExampleTest suite.
	self should: [ testRunner result passed size = 2 ].
	self should: [ (CSExamplePerson >> #firstname) coveringTestMethods size = 2 ].
	
	testRunner reset.
	subclass compile: source.
	self should: [ testRunner result passed size = 2 ].
	
	testRunner reset.
	subclass removeSelector: selector.
	self should: [ testRunner result passed size = 1 ].
	
	subclass compile: source.
	testRunner reset.
	subclass removeFromSystem.
	self should: [ testRunner result passed size = 1 ].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 14:48'!
testCoverageAdaptationAfterTestClassOrMethodRemoved
	| selector testClass testMethodSelector testMethodSource |
	
	selector := #firstname. 
	testMethodSelector := #testFirstnameAccessors.
	testMethodSource := CSExampleTest sourceCodeAt: testMethodSelector.
	CSExamplePerson methodsDo: [:method | 
		method 
			resetCoveringTestMethods;
			resetFixedTestMethods ].
	
	self undoInTearDownDo: [ 
		testClass := self createFooBarTestClass.
		self traceClass: CSExamplePerson.
		 ].
	
	testClass compile: testMethodSource.
	
	self should: [testRunner result passed size = 1].
	self should: [(CSExamplePerson >> selector) coveringTestMethods size = 1].
		
	testClass removeSelector: testMethodSelector.
	self should: [(CSExamplePerson >> selector) coveringTestMethods size = 0].
	
	testClass compile: testMethodSource.
	testClass removeFromSystem.
	
	self should: [(CSExamplePerson >> selector) coveringTestMethods size = 0].
	! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 14:49'!
testDistinguishingOfTestsHavingEqualHashes
	| failingCode1 failingMethod1 failingCode2 failingMethod2 |

	"test adding"
	self undoInTearDownDo: [
		failingCode1 := 'testFailingTest1 self fail.'.
		failingCode2 := 'testFailingTest2 self fail.'.
		failingMethod1 := self installMethod: failingCode1 inClass: CSExampleTest.
		failingMethod2 := self installMethod: failingCode2 inClass: CSExampleTest. ].
	
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = failingMethod1 ] ] ].
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = failingMethod2 ] ] ].

	"test modification"
	self installMethod: failingCode1 inClass: CSExampleTest.
	self installMethod: failingCode2 inClass: CSExampleTest.
	
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = failingMethod1 ] ] ].
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = failingMethod2 ] ] ].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'test 9/21/2009 08:41'!
testDontResetResultIfThereIsNothingToRun
	|source|
	testRunner runSuite: CSExampleTest suite.
	self shouldnt: [testRunner result passed = 0].
	
	source := ('fooBar', String crlf).
	self undoInTearDownDo: [
		self installMethod: source inClass: CSExamplePerson.].
	self shouldnt: [testRunner result passed = 0].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 14:52'!
testExecutionOfAddedTestMethod
	| source runTestCase aMethod |
	aMethod := CSExampleTest >> #testFirstnameAccessors.
	source := aMethod getSource.
	
	self undoInTearDownDo: [
		self removeMethod: aMethod ].
	CSExampleTest compile: source.
	
	self should: [testRunner result passed isEmpty not].
	runTestCase := testRunner result passed first.
	self should: [runTestCase isKindOf: CSExampleTest].
	self should: [runTestCase selector = aMethod selector].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 14:55'!
testExecutionOfChangedTestMethod
	| selector runTestCase|
	selector := #testFirstnameAccessors. 
	CSExampleTest compile: (CSExampleTest sourceCodeAt: selector).
	self should: [testRunner result passed isEmpty not].
	runTestCase := testRunner result passed first.
	self should: [runTestCase isKindOf: CSExampleTest].
	self should: [runTestCase selector = selector].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 14:55'!
testExecutionOfCoveringTestsAfterMethodChanged

	self undoInTearDownDo: [
		self registerClass: CSExamplePerson ].
	testRunner reset; runSuite: CSExampleTest suite. 
	
	CSExamplePerson compile: (CSExamplePerson sourceCodeAt: #firstname:).
	self should: [testRunner result passed isEmpty not]. 
	self should: [testRunner result passed first isKindOf: CSExampleTest].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 14:55'!
testExecutionOfFailedTestsAfterMethodAddedOrChanged
	| methodSelector appMethodSource method | 
	methodSelector := #firstname.
	appMethodSource := CSExamplePerson sourceCodeAt: methodSelector.
	method := CSExamplePerson >> methodSelector.
	method 
		resetCoveringTestMethods;
		resetFixedTestMethods.
	
	self undoInTearDownDo: [
		self removeMethod: method].
	
	testRunner reset; runSuite: CSExampleTest suite.
	self deny: [testRunner result errors isEmpty].
	self should: [testRunner result errors anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = #testFirstnameAccessors]]].
	
	CSExamplePerson compile: appMethodSource.
	self should: [testRunner result errors isEmpty].
	self deny: [testRunner result passed isEmpty].
	self should: [testRunner result passed first isKindOf: CSExampleTest].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 11/30/2009 21:55'!
testExecutionOfFailedTestsAfterTestChanged
	| aSuite failingCode failingMethod2 |
	aSuite := TestSuite new.
	aSuite addTest: (CSExampleTest >> #testFailing) newTestCase.
	testRunner runSuite: aSuite.
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = #testFailing]]].

	self undoInTearDownDo: [
		failingCode := 'testFailing2 self fail'.
		failingMethod2 := self installMethod: failingCode inClass: CSExampleTest ].
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = #testFailing]]].

	CSExampleTest compile: failingCode.
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = #testFailing]]].
	self should: [testRunner result failures anySatisfy: [:test | 
		(test isKindOf: CSExampleTest) and: [test selector = failingMethod2]]].! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 10/6/2009 22:17'!
testForEndlessLoopUsingDoForEachCoveredMethod
	|runner|
	
	runner := CSTestRunner new.
	runner testCasesToRun add: (CSExampleTest selector: #testFirstnameAccessors).
	runner runOpenTestCasesInSuite.
	
	"original version that break a lot of stuff.
	 I assume that using 'self doForEachCoveredMethod' in another doForEach.. Block was not a good idea"
	"CSTestRunner >> #runOpenTestCasesInSuite
	| aSuite testMethod |
	
	aSuite := TestSuite new.
	
	self failedTestCases do: [ :aTestCase |
		aSuite addTest: aTestCase.
		testMethod := aTestCase class >> aTestCase selector.
		testMethod doForEachCoveredMethod: [:aTestMethod :anAppMethod | 
			anAppMethod fixes: aTestMethod.
			self doForEachCoveredMethod valueWithArguments: {aTestMethod. anAppMethod}] ].
	
	
	self testCasesToRun do: [ :aTestCase |
		aSuite addTest: aTestCase.
		testMethod := aTestCase class >> aTestCase selector.
		testMethod doForEachCoveredMethod: [:aTestMethod :anAppMethod | 
			self doForEachCoveredMethod valueWithArguments: {aTestMethod. anAppMethod}]].
	
	self resetTestCasesToRun.
	
	aSuite tests ifNotEmpty: [
		self reset; runSuite: aSuite].
	
	self doForEachCoveredMethod: nil."
	! !

!CSTddAutoRunTests methodsFor: 'test class events' stamp: 'bs 10/28/2009 14:15'!
testModificationOfClass
	|instVarName|
	instVarName := 'foobar'. 
	[	CSInstrumentor systemInstrumentor instrumentClass: CSExamplePerson.
		testRunner reset; runSuite: CSExampleTest suite.
		self should: [(CSExamplePerson >> #firstname) hasCoveringTestMethods].

		CSExamplePerson addInstVarName: instVarName.
		self should: [(CSExamplePerson >> #firstname) hasCoveringTestMethods]]
	ensure: [
		(CSExamplePerson instVarNames includes: instVarName)
			ifTrue: [CSExamplePerson removeInstVarName: instVarName].
		CSInstrumentor systemInstrumentor uninstrumentClass: CSExamplePerson]! !

!CSTddAutoRunTests methodsFor: 'test class events' stamp: 'bs 10/28/2009 14:15'!
testModificationTestOfClass
	|instVarName|
	instVarName := 'foobar'.
	[	CSInstrumentor systemInstrumentor instrumentClass: CSExamplePerson.
		CSExampleTest addInstVarName: instVarName.
		self should: [testRunner result passed size = 1].
		self should: [(CSExamplePerson >> #firstname) hasCoveringTestMethods]]
	ensure: [
		(CSExampleTest instVarNames includes: instVarName)
			ifTrue: [CSExampleTest removeInstVarName: instVarName].
		CSInstrumentor systemInstrumentor uninstrumentClass: CSExamplePerson]! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 15:25'!
testRemovalOfFailedTestCase
	| selector |
	[	selector := #testFooBar. 
		self class compile: selector, String crlf, String tab, 'self fail.'.
		self should: [testRunner result failures notEmpty].
		self should: [testRunner result failures anySatisfy: [:test | 
			(test isKindOf: self class) and: [test selector = selector]]].
		self class removeSelector: selector.
		self should: [testRunner result failures isEmpty].
		self should: [testRunner result errors isEmpty]] 
	ensure: [
		(self class includesSelector: selector) 
			ifTrue: [self class removeSelector: selector]]! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 12/22/2009 15:25'!
testRemovalOfFailedTestsAfterTestClassOrMethodRemoved
	| testClass fooBarSource testClassName testMethodSelector |
	
	[	testClass := self createFooBarTestClass.
		testClassName := testClass name. 
		fooBarSource := 'testFooBar', String crlf , String tab , 'self fail'.
		
		testMethodSelector := testClass compile: fooBarSource.
		self should: [ testRunner result failures size = 1 ].
		testClass removeSelector: testMethodSelector.
		self should: [ testRunner result failures isEmpty ].
		
		testMethodSelector := testClass compile: fooBarSource.
		self should: [ testRunner result failures size = 1 ].
		testClass removeFromSystem.
		self should: [ testRunner result failures isEmpty ] ] 
	ensure: [ 
		(Smalltalk classNamed: testClassName) 
			ifNotNil: [ testClass removeFromSystem ] ]! !

!CSTddAutoRunTests methodsFor: 'test class events' stamp: 'bs 12/22/2009 15:26'!
testRemovalOfTestClass
	|testClass testClassName|
	[	testClass := self createFooBarTestClass. 
		testClassName := testClass name.
		testClass compile: 'testFooBar', String crlf, String tab, 'self fail'.
		self should: [testRunner result failures size = 1].
		testClass removeFromSystem.
		self should: [testRunner result failures isEmpty]]
	ensure: [
		(Smalltalk classNamed: testClassName) 
			ifNotNil: [testClass removeFromSystem]]! !

!CSTddAutoRunTests methodsFor: 'test class events' stamp: 'bs 10/28/2009 14:16'!
testRenamingOfClass
	|class oldName|
	[	CSInstrumentor systemInstrumentor instrumentClass: CSExamplePerson.
		testRunner reset; runSuite: CSExampleTest suite. 
		self should: [(CSExamplePerson >> #firstname) hasCoveringTestMethods].
		
		class := CSExamplePerson.
		oldName := class name.
		CSExamplePerson rename: (oldName, 'new').
		self should: [(class >> #firstname) hasCoveringTestMethods].]
	ensure: [
		class rename: oldName.
		CSInstrumentor systemInstrumentor uninstrumentClass: CSExamplePerson]! !

!CSTddAutoRunTests methodsFor: 'test method events' stamp: 'bs 10/28/2009 14:12'!
testRepro
	| selector source aSuite runner |
	CSExamplePerson methodsDo: [ :method |
		method 
			resetCoveringTestMethods;
			resetFixedTestMethods ].
	
	selector := #firstname.
	source := CSExamplePerson sourceCodeAt: selector.
	
	CSInstrumentor systemInstrumentor instrumentClass: CSExamplePerson.
	
	[	aSuite := TestSuite new.
		aSuite addTest: (CSExampleTest selector: #testFirstnameAccessors).
		runner := CSTestRunner new.
		runner runSuite: aSuite.
		self should: [ runner result passed size = 1 ]]
	ensure: [
		CSInstrumentor systemInstrumentor uninstrumentClass: CSExamplePerson.
		].! !

CSAutoTddTestBase subclass: #CSTestRunnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSTestRunnerTest methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 13:40'!
testResetReporter
	! !

!CSTestRunnerTest methodsFor: 'as yet unclassified' stamp: 'bs 10/7/2009 08:25'!
testRunAndDebugSuite
	|aSuite|
	aSuite := TestSuite new.
	aSuite addTest: (CSExampleTest selector: #testFirstnameAccessors).
	
	testRunner 
		runSuite: aSuite;
		reset;
		debugSuite: aSuite.! !

CSTestBase subclass: #CSBasicTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSBasicTests methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 16:26'!
testConcreteClassCoverage
	| aTestRunner aSuite method wrapper |
	method := (CSExamplePerson >> #firstname) unwrappedMethod.
	method resetAllCoverageData.
	self deny: [method hasCoveringTestMethods].
	
	wrapper := CSCoverageTracingWrapper on: #firstname inClass: CSExamplePerson.
	wrapper install.
	
	aTestRunner := CSTestRunner new.
	aSuite := CSExampleTest suite.
	aTestRunner runSuite: (CSExampleTest suite).
	
	self assert: [method hasCoveringTestMethods].
	self deny: [ (method coveringTestMethodsWhenReceiverWasOfKind: CSExamplePerson) size = 0 ].
	wrapper uninstall.
	
	self deny: [(CSExamplePerson >> #firstname) = wrapper]! !

!CSBasicTests methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 15:32'!
testCoverageApi
	| testMethod method |
	method := (CSExamplePerson >> #firstname) clientMethod.
	testMethod := (CSExampleTest >> #testFirstnameAccessors) clientMethod.
	method resetAllCoverageData.
	testMethod resetAllCoverageData.
	testMethod called: method receiver: nil.
	self should: [method hasCoveringTestMethods].
	self should: [method coveringTestMethods size = 1].
	self should: [method fixedTestMethods size = 0].
	self should: [method coveringAndNotFixedTestMethods size = 1].
	self should: [testMethod coveredMethods includes: method].
	self should: [testMethod coveredMethodsStatistics isEmpty not].
	method resetAllCoverageData.
	testMethod resetAllCoverageData.! !

!CSBasicTests methodsFor: 'as yet unclassified' stamp: 'bs 10/8/2009 15:50'!
testRetrievingTestsForMethod
	| aSuite aTestRunner |
	aTestRunner := CSTestRunner new.
	self undoInTearDownDo: [
		self traceClass: CSExamplePerson].
	aSuite := CSExampleTest suite.
	aTestRunner runSuite: aSuite. 
	self assert: ((CSExamplePerson>>#firstname:) coveringTestMethods includes: CSExampleTest >> #testFirstnameAccessors).! !

!CSBasicTests methodsFor: 'as yet unclassified' stamp: 'test 9/21/2009 08:45'!
testRetrievingTestsForQuickReturnMethod
	| aTestRunner aSuite |
	aTestRunner := CSTestRunner new.
	self undoInTearDownDo: [
		self traceClass: CSExamplePerson].
	aSuite := CSExampleTest suite.
	aTestRunner runSuite: aSuite.
	self assert: ((CSExamplePerson>>#firstname) coveringTestMethods includes: CSExampleTest >> #testFirstnameAccessors).! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 14:15'!
createWrapper: aWrapperClass on: aSelector inClass: aClass
	| result |
	result := aWrapperClass on: aSelector inClass: aClass.
	self undoInTearDown ifTrue: [
		self createdWrappers add: result].
	^ result! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'test 9/18/2009 17:17'!
createdClasses
	^ createdClasses ifNil: [createdClasses := OrderedCollection new]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 14:13'!
createdWrappers
	^ createdWrappers ifNil: [createdWrappers := OrderedCollection new]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'test 9/21/2009 08:40'!
installMethod: source inClass: aClass
	|selector|
	selector := aClass compile: source.
	self undoInTearDown ifTrue: [
		self installedMethods add: {aClass. selector}].
	^ selector! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 9/17/2009 20:56'!
installedMethods
	^ installedMethods ifNil: [installedMethods := OrderedCollection new]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 15:58'!
registerClass: aClass
	(CSInstrumentor systemInstrumentor instrumentedClasses includes: aClass)
		ifFalse: [
			CSInstrumentor systemInstrumentor instrumentClass: aClass.
			self undoInTearDown ifTrue: [
				self registeredClasses add: aClass ] ].! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:16'!
registerPackage: aPackageInfo
	CSInstrumentor systemInstrumentor instrumentPackage: aPackageInfo.
	self undoInTearDown ifTrue: [
		self registeredPackages add: aPackageInfo].! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'test 9/17/2009 21:51'!
registeredClasses
	^ registeredClasses ifNil: [registeredClasses := OrderedCollection new]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/6/2009 11:02'!
registeredPackages
	^ registeredPackages ifNil: [registeredPackages := OrderedCollection new]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'test 9/17/2009 21:58'!
reinstallRemovedMethods
	|aClass selector sourceCode |
	self removedMethods do: [ :classSelectorSourceCodeTuple | 
		aClass := classSelectorSourceCodeTuple first.
		selector := classSelectorSourceCodeTuple second.
		sourceCode := classSelectorSourceCodeTuple third.
		(aClass includesSelector: selector)
			ifFalse: [ aClass compile: sourceCode ] ]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/6/2009 11:11'!
removeCreatedClasses
	self createdClasses copy do: [ :aClass | 
		aClass removeFromSystem.
		(Smalltalk classNamed: aClass name) 
			ifNotNil: [aClass removeFromSystem].
		self createdClasses remove: aClass. ]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 9/17/2009 20:59'!
removeInstalledMethods
	| aClass selector |
	self installedMethods do: [:classSelectorTuple |
		aClass := classSelectorTuple first.
		selector := classSelectorTuple second.
		(aClass includesSelector: selector) 
			ifTrue: [ aClass removeSelector: selector ] ]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 12/21/2009 18:53'!
removeMethod: aCompiledMethod
	|selector sourceCode aClass|
	aClass := aCompiledMethod methodClass.
	selector := aCompiledMethod selector.
	sourceCode := aClass sourceCodeAt: selector.
	self removedMethods add: {aClass. selector. sourceCode}.
	aClass removeSelector: selector. ! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 9/17/2009 20:28'!
removedMethods
	^ removedMethods ifNil: [removedMethods := OrderedCollection new]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:17'!
reregisterPackages
	self unregisteredPackages copy do: [:aPackageInfo |
		CSInstrumentor systemInstrumentor instrumentPackage: aPackageInfo.
		self unregisteredPackages remove: aPackageInfo ].! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 9/11/2009 21:26'!
setUp
	super setUp.
	tddAutoRunTestRunner := CSTestRunner 
		allSubInstances select: [:runner | runner isInTddAutoRunMode].
	tddAutoRunTestRunner do: [:runner | runner tddAutoRunStop].! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 14:14'!
tearDown
	self
		uninstallCreatedWrappers;
		unregisterClasses;
		unregisterPackages;
		reregisterPackages;
		removeCreatedClasses;
		reinstallRemovedMethods;
		removeInstalledMethods.
	tddAutoRunTestRunner do: [:runner | runner tddAutoRunStart].
	super tearDown.! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:18'!
traceClass: aClass
	(CSInstrumentor systemInstrumentor instrumentedClasses includes: aClass)
		ifFalse: [ self registerClass: aClass ]! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:17'!
tracePackage: aPackageInfo
	(CSInstrumentor systemInstrumentor instrumentedPackages includes: aPackageInfo)
		ifFalse: [ self registerPackage: aPackageInfo ]! !

!CSTestBase methodsFor: 'accessing' stamp: 'test 9/21/2009 08:37'!
undoInTearDown
	^ undoInTearDown ifNil: [undoInTearDown := false]! !

!CSTestBase methodsFor: 'accessing' stamp: 'test 9/21/2009 08:36'!
undoInTearDown: anObject
	undoInTearDown := anObject! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'test 9/21/2009 08:37'!
undoInTearDownDo: aBlock
	self undoInTearDown: true.
	aBlock value.
	self undoInTearDown: false.! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 14:15'!
uninstallCreatedWrappers
	self createdWrappers do: [ :wrapper | wrapper uninstall ].! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:17'!
unregisterClasses
	self registeredClasses do: [:aClass |
		CSInstrumentor systemInstrumentor uninstrumentClass: aClass ].! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:17'!
unregisterPackages
	self registeredPackages do: [:aPackageInfo |
		CSInstrumentor systemInstrumentor uninstrumentPackage: aPackageInfo ].! !

!CSTestBase methodsFor: 'as yet unclassified' stamp: 'bs 10/6/2009 11:20'!
unregisteredPackages
	^ unregisteredPackages ifNil: [unregisteredPackages := OrderedCollection new]! !

CSTestBase subclass: #CSTracingWrapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSTracingWrapperTest methodsFor: 'as yet unclassified' stamp: 'bs 9/23/2009 16:00'!
codeSightPackage
	
	^ PackageInfo named: 'CodeSight'.! !

!CSTracingWrapperTest methodsFor: 'as yet unclassified' stamp: 'bs 10/6/2009 11:06'!
createWrappingTestClass
	| aClass |
	aClass := Object subclass: #CSFooBarWrapping
					instanceVariableNames: ''
					classVariableNames: ''
					poolDictionaries: ''
					category: 'CodeSight-Tests'.
	self undoInTearDown ifTrue: [
		self createdClasses add: aClass].
	^ aClass! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 10/6/2009 10:56'!
testCompiledMethodAccess

	CSExamplePerson methodsDo: [:method | 
		method 
			resetCoveringTestMethods;
			resetFixedTestMethods ].
			
	self undoInTearDownDo: [
		self traceClass: CSExamplePerson ].
	
	self should: [(CSExamplePerson >> #firstname) isMwMethodWrapper].
	self shouldnt: [(CSExamplePerson >> #firstname) hasCoveringTestMethods] raise: Error.! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 10/28/2009 14:21'!
testDynamicRegistrationAndWrappingOfClassesAndMethods
	|aClass aClassName selector|
	
	selector := #firstname.
	self undoInTearDownDo: [
		self tracePackage: self codeSightPackage.
		aClass := self createWrappingTestClass ].
	
	aClassName := aClass name.
	self should: [CSInstrumentor systemInstrumentor instrumentedClasses includes: aClass].
	aClass compile: (CSExamplePerson sourceCodeAt: selector).
	self should: [(aClass >> selector) isMwMethodWrapper]. 
	self should: [(CSInstrumentor systemInstrumentor instrumentedMethodsAt: aClass) includes: selector].
	aClass removeSelector: selector.
	self shouldnt: [(CSInstrumentor systemInstrumentor instrumentedMethodsAt: aClass) includes: selector].
	aClass compile: (CSExamplePerson sourceCodeAt: selector).
	aClass removeFromSystem. 
	self shouldnt: [CSInstrumentor systemInstrumentor instrumentedClasses includes: aClass].
	self shouldnt: [(CSInstrumentor systemInstrumentor instrumentedMethodsAt: aClass) includes: selector]! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 10/28/2009 14:22'!
testReregistrationAfterClassModification
	"note, classes have idendity. modification does only change the same object.
	 registration is handled with classes' identity"
	|instVarName nRegisteredMethods|
	instVarName := 'foobar'. 
	self undoInTearDownDo: [
		self traceClass: CSExamplePerson ].
	
	[	nRegisteredMethods := CSInstrumentor systemInstrumentor instrumentedMethods size.
		CSExamplePerson addInstVarName: instVarName.
		self should: [(CSExamplePerson >> #firstname) isKindOf: CompiledMethod].
		self should: [nRegisteredMethods == CSInstrumentor systemInstrumentor instrumentedMethods size]]
	ensure: [
		(CSExamplePerson instVarNames includes: instVarName)
				ifTrue: [CSExamplePerson removeInstVarName: instVarName] ]! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 10/6/2009 11:17'!
testRewrapAfterChangeOfWrappedMethod
 
	self undoInTearDownDo: [
		self traceClass: CSExamplePerson ].
	
	CSExamplePerson compile: (CSExamplePerson sourceCodeAt: #firstname).
	self should: [(CSExamplePerson >> #firstname) isMwMethodWrapper].! !

!CSTracingWrapperTest methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:22'!
testStartStopSystemEventHandling
	| systemInstrumentor handlesSystemEvents | 
	systemInstrumentor := CSInstrumentor systemInstrumentor.
	handlesSystemEvents := systemInstrumentor handlesSystemEvents.
	handlesSystemEvents
		ifTrue: [
			systemInstrumentor stopHandlingSystemEvents.
			self assert: [systemInstrumentor handlesSystemEvents not].
			systemInstrumentor startHandlingSystemEvents ]
		ifFalse: [
			systemInstrumentor startHandlingSystemEvents.
			self assert: [systemInstrumentor handlesSystemEvents].
			systemInstrumentor stopHandlingSystemEvents ]! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 10/28/2009 14:21'!
testUninstall
	|wasRegistered|
	wasRegistered := CSInstrumentor systemInstrumentor isMethodInstrumented: #firstname inClass: CSExamplePerson.
	wasRegistered ifTrue: [CSInstrumentor systemInstrumentor uninstrumentMethod: #firstname inClass: CSExamplePerson ].
	
	CSInstrumentor systemInstrumentor instrumentMethod: #firstname inClass: CSExamplePerson.
	CSInstrumentor systemInstrumentor uninstrumentMethod: #firstname inClass: CSExamplePerson.
	
	self deny: [(CSExamplePerson >> #firstname) isMwMethodWrapper].
	
	wasRegistered ifTrue: [CSInstrumentor systemInstrumentor instrumentMethod: #firstname inClass: CSExamplePerson ].! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 10/28/2009 14:25'!
testUnregisterAllAndReset
	| packages classes methods |
	packages := CSInstrumentor systemInstrumentor instrumentedPackages copy.
	classes := CSInstrumentor systemInstrumentor instrumentedClasses copy.
	methods := CSInstrumentor systemInstrumentor instrumentedMethods copy.
	
	CSInstrumentor systemInstrumentor instrumentClass: CSExamplePerson.
	CSInstrumentor systemInstrumentor uninstrumentAllAndReset.
	self assert: [CSInstrumentor systemInstrumentor instrumentedPackages size = 0].
	self assert: [CSInstrumentor systemInstrumentor instrumentedClasses size = 0].
	self assert: [CSInstrumentor systemInstrumentor instrumentedMethods keys size = 0].
	self assert: [(CSExamplePerson >> #firstname) isMwMethodWrapper not].
	
	packages do: [:ea | CSInstrumentor systemInstrumentor instrumentPackage: ea].
	classes do: [:ea | CSInstrumentor systemInstrumentor instrumentClass: ea].
	methods keysAndValuesDo: [ :aClass :selectors | 
		selectors do: [:s | CSInstrumentor systemInstrumentor instrumentMethod: s inClass: aClass] ].! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 10/28/2009 14:23'!
testUnregisterPackage
	| aPackage |
	aPackage := self codeSightPackage.
	(CSInstrumentor systemInstrumentor instrumentedPackages includes: aPackage)
		ifTrue: [self unregisteredPackages add: aPackage]
		ifFalse: [self registeredPackages add: aPackage].
	
	
	CSInstrumentor systemInstrumentor instrumentPackage: aPackage.
	self should: [(CSExamplePerson >> #firstname) isMwMethodWrapper].
	CSInstrumentor systemInstrumentor uninstrumentPackage: aPackage.
	self shouldnt: [(CSExamplePerson >> #firstname) isMwMethodWrapper].
	! !

!CSTracingWrapperTest methodsFor: 'registration' stamp: 'bs 12/8/2009 17:14'!
testWrapperInstallation
	| aPackage |
	aPackage := self codeSightPackage.
	(CSInstrumentor systemInstrumentor instrumentedPackages includes: aPackage)
		ifTrue: [self unregisteredPackages add: aPackage]
		ifFalse: [self registeredPackages add: aPackage].
	
	CSInstrumentor systemInstrumentor instrumentPackage: aPackage.
	self should: [(CSExamplePerson >> #firstname) isMwMethodWrapper].
	self shouldnt: [(CSExampleTest >> #testFirstnameAccessors) isMwMethodWrapper].
	
	CSInstrumentor systemInstrumentor uninstrumentPackage: aPackage.
	self shouldnt: [(CSExamplePerson >> #firstname) isMwMethodWrapper].
	
	CSInstrumentor systemInstrumentor instrumentPackage: aPackage.
	CSInstrumentor systemInstrumentor instrumentPackage: aPackage.
	CSInstrumentor systemInstrumentor uninstrumentPackage: aPackage.
	self shouldnt: [(CSExamplePerson >> #firstname) isMwMethodWrapper].! !

CSTestBase subclass: #CSWrapperExtensionTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tests'!

!CSWrapperExtensionTests methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 14:31'!
testConflictDetection
	| wrapperOne wrapperTwo aClass aSelector |
	aClass := CSExamplePerson.
	aSelector := #firstname:.
	
	self undoInTearDownDo: [
		wrapperOne := self createWrapper: MwCalledMethodWrapper on: aSelector inClass: aClass.
		wrapperTwo := self createWrapper: MwCountMethodWrapper on: aSelector inClass: aClass.
		].
	
	wrapperOne installClosestToMethod.
	self should: [ wrapperTwo installClosestToMethod ] raise: AssertionFailure.
	
	wrapperTwo uninstall.
	wrapperOne uninstall.! !

!CSWrapperExtensionTests methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 14:16'!
testInstallClosestToMethod
	| current wrapperOne wrapperTwo aClass aSelector |
	aClass := CSExamplePerson.
	aSelector := #firstname:.
	
	current := aClass compiledMethodAt: aSelector.
	
	self undoInTearDownDo: [
		wrapperOne := self createWrapper: MwCalledMethodWrapper on: aSelector inClass: aClass.
		wrapperTwo := self createWrapper: MwCountMethodWrapper on: aSelector inClass: aClass.
		].
	
	wrapperOne install.
	wrapperTwo installClosestToMethod.
	
	self deny: [ wrapperTwo clientMethod isMwMethodWrapper ].
	
	current isMwMethodWrapper
		ifTrue: [ 
			self assert: [ current clientMethod == wrapperTwo ].
			self assert: [ wrapperOne clientMethod == current ] ]
		ifFalse: [
			self assert: [ wrapperTwo clientMethod == current ].
			self assert: [ wrapperOne clientMethod == wrapperTwo ] ].
	
	wrapperTwo uninstall.
	wrapperOne uninstall.! !

!TestCase methodsFor: '*CodeSight' stamp: 'bs 12/21/2009 17:45'!
currentTestCaseMethod
	^ self class >> self selector! !

!TestCase methodsFor: '*CodeSight' stamp: 'bs 12/21/2009 17:45'!
recordRunFailedWith: anException
	self currentTestCaseMethod testFailedWith: anException.
	
	! !

!TestCase methodsFor: '*CodeSight' stamp: 'bs 12/21/2009 17:46'!
recordRunWasSuccessful
	self currentTestCaseMethod testWasSuccessful! !

!Symbol methodsFor: '*CodeSight' stamp: 'bs 9/10/2009 08:54'!
isTestSelector
	^ (self last ~= $:) and: ['test*' match: self]! !

!OBMethodCategoryNode methodsFor: '*codesight' stamp: 'bs 10/28/2009 14:25'!
backgroundColor
	| methodSelectors methodsToBeChecked |
	
	(CSInstrumentor systemInstrumentor instrumentedClasses includes: theClass)
		ifTrue: [
			methodSelectors := (name = ClassOrganizer allCategory)
				ifTrue: [theClass organization allMethodSelectors]
				ifFalse: [theClass organization listAtCategoryNamed: name].
			
			methodsToBeChecked := methodSelectors select: [:aSelector | aSelector isTestSelector not ] 
														 	  thenCollect: [:aSelector | theClass compiledMethodAt: aSelector].
				
			(methodsToBeChecked allSatisfy: [:aMethod | aMethod hasCoveringTestMethods])
				ifFalse: [^ Color lightOrange]].
	
	^ super backgroundColor! !

TextAttribute subclass: #CSTextBackgroundColor
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Text'!

!CSTextBackgroundColor methodsFor: 'accessing' stamp: 'bs 12/21/2009 16:57'!
color
	^ color! !

!CSTextBackgroundColor methodsFor: 'accessing' stamp: 'bs 12/21/2009 16:57'!
color: anObject
	color := anObject! !

!CSTextBackgroundColor methodsFor: 'displaying' stamp: 'bs 12/21/2009 17:03'!
emphasizeBackground: bounds on: aCanvas
	aCanvas 
		fillRectangle: bounds
		color: self color.! !

!CSTextBackgroundColor methodsFor: 'as yet unclassified' stamp: 'bs 12/21/2009 17:34'!
shoutShouldPreserve
	^ true! !

!TextAttribute methodsFor: '*CodeSight' stamp: 'bs 12/21/2009 17:01'!
emphasizeBackground: bounds on: aCanvas
	"Subclasses may override"! !

OBTextMorphWithShout subclass: #CSTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!CSTextMorph methodsFor: 'private' stamp: 'bs 12/21/2009 14:39'!
editorClass
	^ CSEditor! !

!CSTextMorph methodsFor: 'private' stamp: 'bs 12/21/2009 14:39'!
paragraphClass
	^ CSParagraph! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/28/2009 15:29'!
called: aCompiledMethod

	| statistic |
	self flag: #rkrk. "for benchmarking"
	self pvtCodeSightMonitor critical: [
		
		aCompiledMethod privateAddCoveringTestMethod: self.
		
		statistic := self pvtCoveredMethodsStatistics 
			at: aCompiledMethod
			ifAbsentPut: [ CSTestStatistic new ].
			
		self doForEachCoveredMethod valueWithArguments: {  self. aCompiledMethod  } ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/28/2009 15:32'!
called: aCompiledMethod receiver: anObject 
	| statistic |
	self pvtCodeSightMonitor critical: [
		aCompiledMethod privateAddCoveringTestMethod: self.
		
		statistic := self pvtCoveredMethodsStatistics 
			at: aCompiledMethod
			ifAbsentPut: [ CSTestStatistic new ].
		statistic addReceiver: anObject.
			
		self pvtDoForEachCoveredMethod valueWithArguments: {  self. aCompiledMethod  } ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:04'!
coveredMethods
	| result |
	self pvtCodeSightMonitor critical: 
		[ result := self pvtCoveredMethodsStatistics keys ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:04'!
coveredMethodsStatistics
	| result |
	self pvtCodeSightMonitor critical: 
		[ result := self pvtCoveredMethodsStatistics copy ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:04'!
coveringAndNotFixedTestMethods
	| result |
	self pvtCodeSightMonitor critical: [
		result := self pvtCoveringTestMethods copy.
		"fixed link might be established and coverage not yet"
		result removeAllFoundIn: self pvtFixedTestMethods ].
	^ result ! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:04'!
coveringTestMethods
	| result |
	self pvtCodeSightMonitor critical: [ 
		result := self pvtCoveringTestMethods copy ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:07'!
coveringTestMethodsWhenReceiverWasOfKind: aClass 
	| result |
	self pvtCodeSightMonitor critical: [ 
		result := self pvtCoveringTestMethods 
			select: [ :testMethod | 
				(testMethod privateStatisticsForCoveredMethod: self) 
					receiverClasses includes: aClass ] ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 09:34'!
doForEachCoveredMethod
	| result |
	self pvtCodeSightMonitor critical: [ 
		result := self pvtDoForEachCoveredMethod ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 09:35'!
doForEachCoveredMethod: aBlock
	self pvtCodeSightMonitor critical: [
		self pvtDoForEachCoveredMethod: aBlock ].
	^ aBlock! !

!CompiledMethod methodsFor: '*CodeSight-FailedExpression' stamp: 'bs 12/21/2009 17:36'!
failedExpression
	^ self properties at: #codeSightFailedExpression! !

!CompiledMethod methodsFor: '*CodeSight-FailedExpression' stamp: 'bs 12/21/2009 12:47'!
failedExpression: aSelectionInterval
	self properties at: #codeSightFailedExpression put: aSelectionInterval.! !

!CompiledMethod methodsFor: '*CodeSight-FailedExpression' stamp: 'bs 12/21/2009 17:02'!
findTestMethodInStack: anException
	^ anException signalerContext contextStack 
		detect: [:aMethodContext | aMethodContext method = self].! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:06'!
fixedTestMethods
	| result |
	self pvtCodeSightMonitor critical: 
		[ result := self pvtFixedTestMethods copy ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 20:05'!
fixes: aCompiledTestMethod 
	self fixesAll: {aCompiledTestMethod}! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:06'!
fixesAll: compiledTestMethods 
	self pvtCodeSightMonitor critical: [
		self pvtFixedTestMethods addAll: compiledTestMethods ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:06'!
hasCoveringTestMethods
	| result |
	self pvtCodeSightMonitor critical: [ 
		result := self pvtCoveringTestMethods isEmpty not ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:06'!
hasFixedTestMethods
	| result |
	self pvtCodeSightMonitor critical: [
		result := self pvtFixedTestMethods isEmpty not ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight-FailedExpression' stamp: 'bs 12/21/2009 15:58'!
hasTestFailed
	^ self properties includesKey: #codeSightFailedExpression
	! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 11/27/2009 10:41'!
isTestMethod
	^ (self methodClass inheritsFrom: TestCase) and: [self selector isTestSelector]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'test 9/20/2009 14:24'!
newTestCase
	self assert: [self methodClass inheritsFrom: TestCase].
	self assert: [self selector isTestSelector].
	
	^ self methodClass selector: self selector.! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 11/27/2009 11:05'!
overriddenAndOverridingMethods
	^ self overriddenAndOverridingMethodsIn: self methodClass! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'test 9/20/2009 21:39'!
overriddenAndOverridingMethodsIn: aClass
	|result|
	result := (self overridingMethodsIn: aClass) copy.
	(self overriddenMethodIn: aClass) 
		ifNotNilDo: [:aMethod | result add: aMethod].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'test 9/20/2009 21:34'!
overriddenMethod
	^ self overriddenMethodIn: self methodClass! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'test 9/20/2009 21:49'!
overriddenMethodIn: aClass
	^ (aClass superclass whichClassIncludesSelector: self selector)
			ifNotNilDo: [ :superclass | (superclass >> self selector) clientMethod ]
			ifNil: [ nil ].! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'test 9/20/2009 21:34'!
overridingMethods
	^ self overridingMethodsIn: self methodClass! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 11/27/2009 10:54'!
overridingMethodsIn: aClass
	| overriders |
	overriders := Set new.
	self methodClass addOverridersOf: self selector to: overriders.
	
	^ overriders collect: [ :overrider | 
			(overrider >> self selector) clientMethod ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 20:38'!
privateAddCoveringTestMethod: aCompiledTestMethod
	^ self pvtCoveringTestMethods add: aCompiledTestMethod! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:09'!
privateRemoveCoveredMethod: aCompiledAppMethod
	self pvtCodeSightMonitor critical: [
		self pvtCoveredMethodsStatistics removeKey: aCompiledAppMethod ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:08'!
privateRemoveCoveringTestMethod: aTestCaseMethod
	self pvtCodeSightMonitor critical: [
		"the other direction is removed by the sender"
		self pvtCoveringTestMethods remove: aTestCaseMethod ifAbsent: [] ]
	! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:06'!
privateStatisticsForCoveredMethod: aCompiledMethod 
	^ self pvtCoveredMethodsStatistics at: aCompiledMethod ! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 18:53'!
pvtCodeSightMonitor
	^ self properties 
		at: #codeSightMonitor
		ifAbsentPut: [ Monitor new ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 18:13'!
pvtCoveredMethodsStatistics
	^ self properties 
			at: #codeSightStatistics
			ifAbsentPut: [ IdentityDictionary new ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 18:10'!
pvtCoveringTestMethods
	^ self properties at: #codeSightCovering ifAbsentPut: [ IdentitySet new ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 09:33'!
pvtDoForEachCoveredMethod
	| |
	self assert: [ self selector isTestSelector ].
	^ self properties 
			at: #codeSightDoForEachCoveredMethod
			ifAbsentPut: [ [ :aTestMethod :anAppMethod |  ] ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 09:34'!
pvtDoForEachCoveredMethod: aBlock
	self assert: [ self selector isTestSelector ].
	self pvtCodeSightMonitor critical: [
		self properties at: #codeSightDoForEachCoveredMethod put: aBlock ].
	^ aBlock! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 18:30'!
pvtFixedTestMethods
	^ self properties at: #codeSightFixed ifAbsentPut: [ IdentitySet new ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 18:35'!
pvtRemoveCoveredMethod: aCompiledAppMethod
	self pvtCoveredMethodsStatistics removeKey: aCompiledAppMethod! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 18:37'!
pvtRemoveCoveringTestMethod: aTestCaseMethod
	"the other direction is removed by the sender"
	self pvtCoveringTestMethods remove: aTestCaseMethod! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:10'!
removeCoveredMethod: aCompiledAppMethod
	self pvtCodeSightMonitor critical: [
		self privateRemoveCoveredMethod: aCompiledAppMethod ]! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:09'!
removeCoveringTestMethod: aTestCaseMethod
	self pvtCodeSightMonitor critical: [
		"the other direction is removed by the sender"
		self privateRemoveCoveringTestMethod: aTestCaseMethod ]
	! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:07'!
removeFixedTestMethodIfNotCoveringAnymore: aTestCaseMethod
	self pvtCodeSightMonitor critical: [
		(self pvtFixedTestMethods includes: aTestCaseMethod) ifFalse: [^ self].
		(self pvtCoveringTestMethods includes: aTestCaseMethod) ifTrue: [^ self].
		self pvtFixedTestMethods remove: aTestCaseMethod ]
	! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 21:17'!
resetAllCoverageData
	self 
		resetCoveringTestMethods;
		resetCoveredMethodsStatistics;
		resetFixedTestMethods;
		resetDoForEachCoveredMethod.! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:09'!
resetCoveredMethodsStatistics
	| result |
	self pvtCodeSightMonitor critical: [
		self pvtCoveredMethodsStatistics keysDo: [ :coveredMethod | 
				coveredMethod privateRemoveCoveringTestMethod: self. ].
	
		result := self properties 
			removeKey: #codeSightStatistics 
			ifAbsent: [ ] ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:10'!
resetCoveringTestMethods
	self pvtCodeSightMonitor critical: [
		self pvtCoveringTestMethods do: [:testMethod |
			testMethod privateRemoveCoveredMethod: self.
			self pvtRemoveCoveringTestMethod: testMethod. ] ].
	! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:08'!
resetDoForEachCoveredMethod
	self pvtCodeSightMonitor critical: [ 
		self properties removeKey: #codeSightDoForEachCoveredMethod ifAbsent: [] ].! !

!CompiledMethod methodsFor: '*CodeSight-FailedExpression' stamp: 'bs 12/21/2009 12:57'!
resetFailedExpression
	self properties removeKey: #codeSightFailedExpression ifAbsent: [].! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 9/24/2009 08:08'!
resetFixedTestMethods
	self pvtCodeSightMonitor critical: [
		self pvtFixedTestMethods removeAll: (self pvtFixedTestMethods) ].! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 10/16/2009 10:07'!
statisticsForCoveredMethod: aCompiledMethod 
	| result |
	self pvtCodeSightMonitor critical: [
		result := self privateStatisticsForCoveredMethod: aCompiledMethod  ].
	^ result! !

!CompiledMethod methodsFor: '*CodeSight-FailedExpression' stamp: 'bs 12/21/2009 15:50'!
testFailedWith: anException
	| testMethodContext pc pcRange | 
	testMethodContext := self findTestMethodInStack: anException.
	pc := testMethodContext pc.
	pcRange := testMethodContext debuggerMap rangeForPC: pc contextIsActiveContext: false.
	self failedExpression: pcRange.! !

!CompiledMethod methodsFor: '*CodeSight-FailedExpression' stamp: 'bs 12/21/2009 15:51'!
testWasSuccessful
	self resetFailedExpression! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 11/27/2009 14:45'!
testsAffectedByAddition
	^ ((self overriddenAndOverridingMethods)
			gather: [ :supOrSubMethod | supOrSubMethod coveringTestMethods ]) 
				asSet! !

!CompiledMethod methodsFor: '*CodeSight' stamp: 'bs 11/27/2009 14:44'!
testsAffectedByModification
	^ self coveringTestMethods! !

!Behavior methodsFor: '*CodeSight' stamp: 'bs 10/28/2009 10:03'!
addOverridersOf: aSelector to: aCollection
	self allSubclassesDo: [:subclass | 
		(subclass includesSelector: aSelector)
			ifTrue: [aCollection add: subclass]
			ifFalse: [subclass addOverridersOf: aSelector to: aCollection]]! !

SwaGeneralScrollBar subclass: #CSActivityScrollBar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-View'!

!CSActivityScrollBar methodsFor: 'as yet unclassified' stamp: 'mt 12/15/2009 20:55'!
initializeMenuButton
	"Ignore menu button."
	
	menuButton := nil.! !

!OBClassNode methodsFor: '*codesight' stamp: 'bs 10/28/2009 14:25'!
backgroundColor

	(CSInstrumentor systemInstrumentor instrumentedClasses includes: theClass)
		ifFalse: [^ super backgroundColor].
	theClass methodDict values do: [:method |
		(method hasCoveringTestMethods
			or: [method selector isTestSelector
				and: [theClass inheritsFrom: TestCase]])
					ifFalse: [^ Color lightOrange]].
	^ super backgroundColor.! !

!OBMorphicIcons methodsFor: '*codesight' stamp: 'mt 5/3/2009 20:44'!
csAccept

	^ MenuIcons csAcceptIcon! !

!OBEnhancementDefinitionPanel methodsFor: '*CodeSight-override' stamp: 'bs 9/24/2009 08:03'!
accept: aText notifying: aController
	| result |
	CSTestRunner sourceCodeAccessMonitor critical: [
		result := super accept: aText notifying: aController ].
	^ result.! !

!OBMethodNode methodsFor: '*codesight' stamp: 'bs 10/28/2009 14:25'!
backgroundColor

	((CSInstrumentor systemInstrumentor instrumentedClasses includes: theClass) not
		or: [selector isTestSelector]
		or: [(theClass includesSelector: selector) not]
		or: [(theClass compiledMethodAt: selector) hasCoveringTestMethods])
			ifTrue: [^ super backgroundColor].
	^ Color lightOrange.! !

!OBMethodNode methodsFor: '*codesight' stamp: 'bs 5/4/2009 21:53'!
compiledMethod
	^ theClass compiledMethodAt: self selector! !

!OBMethodNode methodsFor: '*codesight' stamp: 'bs 10/7/2009 10:34'!
coveringTestMethods

	| testMethodNodes theMethod statistics node createAndAddCoveringNode |
	testMethodNodes := OrderedCollection new.
	(theClass includesSelector: selector) ifTrue: [
		theMethod := self compiledMethod.
		createAndAddCoveringNode := [:testMethod | 
			statistics := testMethod statisticsForCoveredMethod: theMethod unwrappedMethod.
			node := OBCoveringTestCaseNode on: testMethod 
							statistic: statistics coveredMethodNode: self.
			testMethodNodes add: node ].
		
		"theMethod fixedTestMethods do: createAndAddCoveringNode.
		
		node := OBCoveringTestCaseSeparatorNode coveredMethodNode: self.
		testMethodNodes add: node."
		
		theMethod coveringAndNotFixedTestMethods do: createAndAddCoveringNode ].
	
	^ testMethodNodes.! !

!OBPluggableTextMorphWithShout methodsFor: '*CodeSight-override' stamp: 'bs 12/21/2009 14:46'!
textMorphClass
	^ CSTextMorph! !

OBCommand subclass: #CSCmdShowTestResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!CSCmdShowTestResult methodsFor: 'user interface' stamp: 'rkrk 12/8/2009 13:51'!
addItemToMenu: aMenu

	"don't add to menu'"! !

!CSCmdShowTestResult methodsFor: 'accessing' stamp: 'rkrk 9/30/2009 01:56'!
browser

	^ requestor browser! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 9/30/2009 05:28'!
buttonLabelWithSpaces

	| fillColor frame canvas |
	fillColor := self testsAreGreen
		ifTrue: [Color green]
		ifFalse: [Color red].
	frame := 0@0 corner: 60@25.
	canvas := self canvasFits: frame colored: fillColor.
	self browser isRunning ifTrue: [
		self
			drawHourglassOn: canvas
			colored: fillColor
			at: 10@5].
	self defectCount isZero ifFalse: [
		self
			drawString: self defectCount asString
			on: canvas
			fitting: (30@5 corner: 55@20)].
	^ canvas form asMorph.! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 9/30/2009 05:27'!
canvasFits: frame colored: fillColor

	| canvas rounding roundingLeft roundingRight |
	canvas :=(FormCanvas extent: frame extent depth: 16) asAlphaBlendingCanvas: 1.
	
	rounding := 20.
	roundingLeft := frame topLeft corner: (frame right-rounding)@frame bottom.
	canvas fillOval: roundingLeft color: fillColor.
	roundingRight := (frame left+rounding)@frame top corner: frame bottomRight.
	canvas fillOval: roundingRight color: fillColor.
	canvas
		fillRectangle: (roundingRight topLeft corner: roundingLeft bottomRight)
		color: fillColor.

	^ canvas.! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 9/30/2009 02:33'!
defectCount

	^ self testResult failureCount + self testResult errorCount! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 9/30/2009 05:28'!
drawHourglassOn: canvas colored: fillColor at: pt

	| hourglass |
	hourglass := MenuIcons csHourGlassIcon deepCopy
		mapColor: Color white
		to: fillColor.
	canvas translucentImage: hourglass at: pt.! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 9/30/2009 04:37'!
drawString: string on: canvas fitting: rect

	canvas drawString: string in: rect.! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 10/7/2009 11:15'!
dummyLabelGreen: isGreen running: isRunning defectCount: defects

	| fillColor frame canvas |
	fillColor := isGreen
		ifTrue: [Color green]
		ifFalse: [Color red].
	frame := 0@0 corner: 60@25.
	canvas := self canvasFits: frame colored: fillColor.
	isRunning ifTrue: [
		self
			drawHourglassOn: canvas
			colored: fillColor
			at: 10@5].
	defects isZero ifFalse: [
		self
			drawString: defects asString
			on: canvas
			fitting: (30@5 corner: 55@20)].
	^ canvas form asMorph.! !

!CSCmdShowTestResult methodsFor: 'execution' stamp: 'rkrk 9/30/2009 01:57'!
execute

	| runner runnerMorph |
	runner := self browser testRunner.
	runnerMorph := World submorphs
		detect: [:ea | (ea respondsTo: #model) and: [ea model = runner]]
		ifNone: [runner buildWith: MorphicToolBuilder new].
	runnerMorph openInWorld.! !

!CSCmdShowTestResult methodsFor: 'accessing' stamp: 'rkrk 9/22/2009 02:20'!
group
	^ #codesight! !

!CSCmdShowTestResult methodsFor: 'testing' stamp: 'rkrk 9/22/2009 02:19'!
isActive
	^ true! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 9/30/2009 02:28'!
isRunning

	^ self browser isRunning! !

!CSCmdShowTestResult methodsFor: 'accessing' stamp: 'rkrk 12/8/2009 13:52'!
label

	^ 'this should not appear anywhere'! !

!CSCmdShowTestResult methodsFor: 'accessing' stamp: 'rkrk 9/30/2009 02:31'!
testResult

	^ self browser testRunner result! !

!CSCmdShowTestResult methodsFor: 'icon assembly' stamp: 'rkrk 9/30/2009 02:31'!
testsAreGreen

	^ (self testResult hasErrors or: [self testResult hasFailures]) not! !

!CSCmdShowTestResult methodsFor: 'testing' stamp: 'rkrk 9/22/2009 02:19'!
wantsButton

	^ true
! !

MultiNewParagraph subclass: #CSParagraph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Browser'!

!CSParagraph methodsFor: 'fonts-display' stamp: 'bs 12/21/2009 16:48'!
displayOn: aCanvas using: displayScanner at: somePosition
	"Send all visible lines to the displayScanner for display"
	
	| visibleRectangle line |
	visibleRectangle := aCanvas clipRect.
	(self lineIndexForPoint: visibleRectangle topLeft)
		to: (self lineIndexForPoint: visibleRectangle bottomRight)
		do: [:i | line := lines at: i.
			self drawBackgroundEmphasisInLine: line on: aCanvas ].
	
	super displayOn: aCanvas using: displayScanner at: somePosition.
! !

!CSParagraph methodsFor: 'fonts-display' stamp: 'bs 12/21/2009 17:01'!
drawBackgroundEmphasisInLine: line on: aCanvas
	| textLine |
	textLine := self text
		copyFrom: line first
		to: line last.
	textLine runs withStartStopAndValueDo: [:start :stop :attributes | 
		attributes do: [:anAttr | | cb1 cb2 bounds |
			cb1 := self characterBlockForIndex: start + (line first - 1).
			cb2 := self characterBlockForIndex: stop + (line first - 1). 
			bounds := cb1 left @ line top corner: cb2 right @ line bottom.
			anAttr emphasizeBackground: bounds on: aCanvas ] ]! !

Object subclass: #ATddBenchmarker
	instanceVariableNames: 'packages methods coveredMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!ATddBenchmarker methodsFor: 'derived accessing' stamp: 'rkrk 9/21/2009 03:51'!
appMethods

	| class |
	^ self methods reject: [:ea |
		class := ea methodClass ifNil: [ea clientMethod searchForClass].
		class
			ifNil: [false]
			ifNotNil: [class inheritsFrom: TestCase]].! !

!ATddBenchmarker methodsFor: 'derived accessing' stamp: 'rkrk 9/20/2009 03:38'!
classesOfPackages

	^ (packages gather: [:p | p classes]) asSet! !

!ATddBenchmarker methodsFor: 'computing' stamp: 'rkrk 9/21/2009 04:14'!
computeTestClassesTimesPerMethodFrom: timeForEachTestClass

	| testClassesTimesPerMethod |
	testClassesTimesPerMethod := Dictionary new. "appMethod -> coveringTestClassesTime"
	(self methodsCovered: self methods) do: [:appMethod |
		testClassesTimesPerMethod
			at: appMethod
			put: (self coveringTestClassesTimeFor: appMethod from: timeForEachTestClass)].
	^ testClassesTimesPerMethod.! !

!ATddBenchmarker methodsFor: 'computing' stamp: 'rkrk 9/21/2009 04:14'!
computeTestTimesPerMethodFrom: timeForEachTest

	| testTimesPerMethod |
	testTimesPerMethod := Dictionary new. "appMethod -> coveringTestMethodsTime"
	(self methodsCovered: self methods) do: [:appMethod |
		testTimesPerMethod
			at: appMethod
			put: (self coveringTestMethodsTimeFor: appMethod from: timeForEachTest)].
	^ testTimesPerMethod.! !

!ATddBenchmarker methodsFor: 'computing' stamp: 'rkrk 9/20/2009 03:48'!
computeTimeForAllTestsFrom: timeForEachTest

	^ timeForEachTest values sum.! !

!ATddBenchmarker methodsFor: 'computing' stamp: 'rkrk 9/20/2009 18:06'!
computeTimeForEachTest

	| aTestRunner testClasses timeForEach |
	aTestRunner := CSTestRunner new.
	testClasses := self testClassesOfPackages.
	timeForEach := Dictionary new.
	testClasses
		do: [:testClass | 
			testClass selectors
				select: [:s | s isTestSelector]
				thenDo: [:s | | time |
					time := [aTestRunner runSuite: (testClass suiteWithSelectorsMatching: s)] timeToRun.
					timeForEach at: (testClass >> s) methodReference put: time]]
		displayingProgress: 'Running Tests (' , testClasses size asString, ' test classes)'.
	^ timeForEach! !

!ATddBenchmarker methodsFor: 'computing' stamp: 'rkrk 9/20/2009 18:11'!
computeTimeForEachTestClassFrom: timeForEachTest

	| timeForEach |
	timeForEach := Dictionary new.
	self testClassesOfPackages do: [:class | | methodsOfClass time |
		methodsOfClass := timeForEachTest keys select: [:m | m actualClass = class].
		time := methodsOfClass inject: 0 into: [:t :m | (timeForEachTest at: m) + t].
		timeForEach at: class put: time].
	^ timeForEach! !

!ATddBenchmarker methodsFor: 'derived accessing' stamp: 'rkrk 9/22/2009 14:43'!
coveringTestClassesTimeFor: appMethod from: timeForEachTestClass

	| testClasses |
	testClasses := (appMethod coveringTestMethods
		collect: [:ea | ea methodClass]
		thenSelect: [:ea | ea isObsolete not]) asSet.
	^ testClasses
		inject: 0
		into: [:sum :c | (timeForEachTestClass at: c) + sum].! !

!ATddBenchmarker methodsFor: 'derived accessing' stamp: 'rkrk 9/22/2009 14:43'!
coveringTestMethodsTimeFor: appMethod from: timeForEachTest

	^ appMethod coveringTestMethods
		inject: 0
		into: [:sum :m | (timeForEachTest at: m methodReference ifAbsent: [0"???"]) + sum].! !

!ATddBenchmarker methodsFor: 'running' stamp: 'rkrk 9/20/2009 18:17'!
measureCoveringTimesForPackages: packageString

	^ String streamContents: [:stream |
		self 
			measureCoveringTimesForPackages: packageString 
			printOn: stream].
	! !

!ATddBenchmarker methodsFor: 'running' stamp: 'bs 10/28/2009 14:11'!
measureCoveringTimesForPackages: packageString printOn: stream

	"^ #(coveringTestMethodsTime coveringTestClassesTime timeForRunningAllTestsAtEveryChange)"
	| timeForEachTest timeForEachTestClass timeForAllTests testClassesTimesPerMethod testTimesPerMethod coveredTestMethodsTimes coveredTestClassesTimes |
	packages := PackageOrganizer default packages select: [:p | p packageName startsWith: packageString].
	packages do: [:p | CSInstrumentor systemInstrumentor instrumentPackage: p].

	timeForEachTest := self computeTimeForEachTest.
	timeForEachTestClass := self computeTimeForEachTestClassFrom: timeForEachTest.
	timeForAllTests := self computeTimeForAllTestsFrom: timeForEachTest.
	testTimesPerMethod := self computeTestTimesPerMethodFrom: timeForEachTest.
	testClassesTimesPerMethod := self computeTestClassesTimesPerMethodFrom: timeForEachTestClass.

	"CSV style output
	appMethods,coveredMethods,coveredTestMethodsTimes average, coveredTestMethodsTimes median, coveredTestMethodsTimes min, coveredTestMethodsTimes max, coveredTestMethodsTimes sum, coveredTestClassesTimes avg, coveredTestClassesTimes median, coveredTestClassesTimes sum, timeForAllTests, timeForAllTests sum"
	coveredTestMethodsTimes := testTimesPerMethod values.
	coveredTestClassesTimes := testClassesTimesPerMethod values.
	stream
		print: self appMethods size; nextPut: $,;
		print: (self methodsCovered: self appMethods) size; nextPut: $,;
		print: (coveredTestMethodsTimes average roundTo: 1); nextPut: $,;
		print: (coveredTestMethodsTimes median roundTo: 1); nextPut: $,;
		print: coveredTestMethodsTimes min; nextPut: $,;
		print: coveredTestMethodsTimes max; nextPut: $,;
		print: coveredTestMethodsTimes sum; nextPut: $,;
		print: (coveredTestClassesTimes average roundTo: 1); nextPut: $,;
		print: (coveredTestClassesTimes median roundTo: 1); nextPut: $,;
		print: coveredTestClassesTimes sum; nextPut: $,;
		print: timeForAllTests; nextPut: $,;
		print: (self methodsCovered: self methods) size * timeForAllTests;
		cr.
		
	packages do: [:p |CSInstrumentor systemInstrumentor uninstrumentPackage: p].! !

!ATddBenchmarker methodsFor: 'running' stamp: 'bs 10/8/2009 16:10'!
measureTestExecutionTimeForPackages: packageString

	| aTestSuite testClasses nRuns |
	packages := PackageOrganizer default packages select: [:p | p packageName startsWith: packageString].
	"packages do: [:p | CSCoverageTracingWrapper registerPackage: p]."
	
	aTestSuite := TestSuite new.
	testClasses := self testClassesOfPackages.
	
	self testClassesOfPackages
		do: [:testClass | 
			testClass selectors
				select: [:s | s isTestSelector]
				thenDo: [:s | aTestSuite addTest: (testClass selector: s)]].
			
	nRuns := 2.
	^ ([nRuns timesRepeat: [CSTestRunner new runSuite: aTestSuite]] timeToRun / nRuns).
	! !

!ATddBenchmarker methodsFor: 'derived accessing' stamp: 'rkrk 9/22/2009 14:43'!
methods

	^ methods ifNil: [
		methods := self classesOfPackages gather: [:class |
			class methodDict values]].! !

!ATddBenchmarker methodsFor: 'derived accessing' stamp: 'rkrk 9/21/2009 04:16'!
methodsCovered: methodCol

	^ methodCol select: [:ea | ea hasCoveringTestMethods].! !

!ATddBenchmarker methodsFor: 'derived accessing' stamp: 'rkrk 9/22/2009 14:43'!
testClassesOfPackages

	^ self classesOfPackages select: [:class |
		class inheritsFrom: TestCase].! !

Object subclass: #ATddMCAnalyzer
	instanceVariableNames: 'inspector methodVersionResult doAfterGathering'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!ATddMCAnalyzer class methodsFor: 'service' stamp: 'bs 11/20/2009 15:42'!
printVersionDatesOfPackage: packageName

	| instance dates |
	instance := self new.
	instance
		gatherVersionsOf: packageName
		thenDo: [:versionInfos |
			self halt.
			dates := (versionInfos collect: [:ea | ea date]) asSet.
			dates := dates	asOrderedCollection sortBy: [:a :b | a < b].
			Transcript cr; show: packageName; show: ' -- active development'.
			dates do: [:ea | Transcript cr; show: ea asString]].
	^instance.! !

!ATddMCAnalyzer class methodsFor: 'service' stamp: 'rkrk 9/20/2009 00:57'!
printVersionDatesOfPackage: packageName inRepo: repo

	| instance dates |
	instance := self new.
	instance
		gatherVersionsOf: {packageName}
		in: repo
		thenDo: [:versionInfos |
			dates := (versionInfos collect: [:ea | ea date]) asSet.
			dates := dates	asOrderedCollection sortBy: [:a :b | a < b].
			Transcript cr; show: packageName; show: ' -- active development'.
			dates do: [:ea | Transcript cr; show: ea asString]].
	^instance.! !

!ATddMCAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 17:24'!
gatherVersionsOf: packageNames in: mcRepository thenDo: block 
	inspector ifNotNil: [ inspector stop ].
	inspector := CSMczInspectorOld 
		newForPackagesNamed: packageNames
		inRepo: mcRepository.
	methodVersionResult := OrderedCollection new.
	inspector addDependent: self.
	inspector startInspectVersions.
	doAfterGathering := block.
	^ methodVersionResult! !

!ATddMCAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 17:24'!
gatherVersionsOf: packageName thenDo: block 
	inspector ifNotNil: [ inspector stop ].
	inspector := CSMczInspectorOld newForPackageNamed: packageName.
	methodVersionResult := OrderedCollection new.
	inspector addDependent: self.
	inspector startInspectVersions.
	doAfterGathering := block.
	^ methodVersionResult! !

!ATddMCAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 17:24'!
inspect: selector of: class in: packageName 
	inspector ifNotNil: [ inspector stop ].
	inspector := CSMczInspectorOld newForPackageNamed: packageName.
	methodVersionResult := OrderedCollection new.
	inspector addDependent: self.
	inspector 
		startInspectVersionsFor: selector
		in: class.
	^ methodVersionResult! !

!ATddMCAnalyzer methodsFor: 'nil' stamp: 'rkrk 9/20/2009 00:58'!
methodVersionResult

	^ methodVersionResult! !

!ATddMCAnalyzer methodsFor: 'as yet unclassified' stamp: 'rkrk 9/20/2009 00:24'!
update: aSymbol

	super update: aSymbol.
	aSymbol = #done ifFalse: [^ self].
	Transcript cr show: 'done'.
	doAfterGathering value: methodVersionResult.! !

!ATddMCAnalyzer methodsFor: 'as yet unclassified' stamp: 'rkrk 9/19/2009 23:13'!
update: aSymbol with: anObject

	super update: aSymbol with: anObject.
	aSymbol = #versionFound ifFalse: [^self].
	Transcript cr; show: 'Found ' , anObject asString.
	methodVersionResult add: anObject.! !

Object subclass: #ATddReceiverAnalyzer
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!ATddReceiverAnalyzer methodsFor: 'analyzing' stamp: 'rkrk 10/7/2009 21:32'!
getClassOf: method

	^ method methodClass ifNil: [method searchForClass].! !

!ATddReceiverAnalyzer methodsFor: 'analyzing' stamp: 'rkrk 10/7/2009 21:38'!
init

	stream := WriteStream on: ''.! !

!ATddReceiverAnalyzer methodsFor: 'analyzing' stamp: 'rkrk 10/7/2009 21:29'!
isOverwritten: method

	"CompiledMethod>>overridingMethods does not work"
	^ (self overridingClasses: method) notEmpty! !

!ATddReceiverAnalyzer methodsFor: 'analyzing' stamp: 'rkrk 10/7/2009 21:35'!
methodsNotOverwrittenIn: package

	^ ((package coreMethods collect: [:ea | ea compiledMethod]
		thenSelect: [:method | method hasCoveringTestMethods])
		reject: [:method | self isOverwritten: method])
		reject: [:method | (self getClassOf: method) allSubclasses isEmpty].! !

!ATddReceiverAnalyzer methodsFor: 'analyzing' stamp: 'rkrk 10/7/2009 21:32'!
overridingClasses: method

	^ (self getClassOf: method) allSubclasses select: [:ea |
		ea includesSelector: method selector].! !

!ATddReceiverAnalyzer methodsFor: 'analyzing' stamp: 'rkrk 10/7/2009 22:29'!
printTestsForOverwrittenMethodsWithAndWithoutReceiverIn: package

	"self printTestsForOverwrittenMethodsWithAndWithoutReceiverIn:
		(PackageInfo named: 'AweSOM')"
	| methods
	appMethodClass pureTestCount testCountReceiver allNoRecv allRecv stat |
	self init.
	methods := self methodsNotOverwrittenIn: package.
	methods do: [:appMethod |
		appMethodClass := self getClassOf: appMethod.
		pureTestCount := appMethod coveringTestMethods size.
		allNoRecv := allRecv := 0.
		"<for each subclass>"
		appMethodClass allSubclasses do: [:subclass |
			testCountReceiver := 0.
			"<for each coveringTest>"
			"<if subclass or any of its subclasses are
			included in receiverclasses of coveringTest --> count+1>"
			testCountReceiver := appMethod coveringTestMethods count: [:test | 
				stat := test statisticsForCoveredMethod: appMethod clientMethod.
				(stat receiverClasses includesAnyOf: subclass withAllSubclasses)].
			"----"
			allNoRecv := allNoRecv + pureTestCount.
			allRecv := allRecv + testCountReceiver.
			"print it"
			stream
				cr; nextPutAll: 'If method '; nextPutAll: appMethod selector;
				nextPutAll: ' would be added to '; nextPutAll: subclass name; space;
				print: pureTestCount; nextPutAll: '/'; print: testCountReceiver;
				nextPutAll: ' should run.'.
		].
		"print it again"
		stream
			cr; print: allRecv; nextPut: $/; print: allNoRecv;
			nextPut: $=;
			print: (allRecv isZero
				ifTrue: [0]
				ifFalse: [allRecv / allNoRecv * 100 asFloat]).
		Transcript show: stream contents.
	].! !

Object subclass: #ATddVersionLoader
	instanceVariableNames: 'repo versionFileNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!ATddVersionLoader class methodsFor: 'as yet unclassified' stamp: 'rkrk 9/20/2009 02:42'!
loadVersionsNamed: versionNames from: mcRepository

	| fileNames |
	fileNames := versionNames collect: [:ea |
		(ea endsWith: '.mcz') ifTrue: [ea] ifFalse: [ea , '.mcz']].
	self new
		repo: mcRepository;
		versionFileNames: fileNames;
		loadAll;
		yourself.! !

!ATddVersionLoader methodsFor: 'as yet unclassified' stamp: 'rkrk 9/20/2009 02:46'!
loadAll

	| versions |
	versions := self versionFileNames collect: [:ea |
		repo loadVersionFromFileNamed: ea].
	versions do: [:ea | ea load].! !

!ATddVersionLoader methodsFor: 'as yet unclassified' stamp: 'rkrk 9/20/2009 02:42'!
repo: mcRepository

	repo := mcRepository! !

!ATddVersionLoader methodsFor: 'as yet unclassified' stamp: 'rkrk 9/20/2009 02:44'!
versionFileNames

	^ versionFileNames! !

!ATddVersionLoader methodsFor: 'as yet unclassified' stamp: 'rkrk 9/20/2009 02:43'!
versionFileNames: fileNames

	versionFileNames := fileNames! !

Object subclass: #CSExampleFooBar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestExamples'!

!CSExampleFooBar methodsFor: 'as yet unclassified' stamp: 'bs 9/24/2009 08:20'!
fooBar! !

Object subclass: #CSExamplePerson
	instanceVariableNames: 'firstname'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestExamples'!

!CSExamplePerson class methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 09:24'!
wantsRecompilationProgressReported
	"turn off for this clas because it is modified during tests"

	^ false
! !

!CSExamplePerson methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 15:35'!
asString
	^ super asString! !

!CSExamplePerson methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 15:35'!
firstname
	^ firstname! !

!CSExamplePerson methodsFor: 'as yet unclassified' stamp: 'bs 12/22/2009 15:35'!
firstname: anObject
	firstname := anObject! !

Object subclass: #CSFailedExpressionStyler
	instanceVariableNames: ''
	classVariableNames: 'RegisteredInstance'
	poolDictionaries: ''
	category: 'CodeSight-Text'!

!CSFailedExpressionStyler class methodsFor: 'initialize-release' stamp: 'bs 12/22/2009 10:30'!
initialize
	"self initialize"
	super initialize.
	self load.! !

!CSFailedExpressionStyler class methodsFor: 'initialize-release' stamp: 'bs 12/22/2009 10:33'!
load
	self registeredInstance: self new.
	SwaSourceTextStyler instance registerStyler: self registeredInstance.! !

!CSFailedExpressionStyler class methodsFor: 'initialize-release' stamp: 'bs 12/22/2009 10:29'!
registeredInstance
	^ RegisteredInstance! !

!CSFailedExpressionStyler class methodsFor: 'initialize-release' stamp: 'bs 12/22/2009 10:29'!
registeredInstance: anInstance
	^ RegisteredInstance := anInstance! !

!CSFailedExpressionStyler class methodsFor: 'initialize-release' stamp: 'bs 12/22/2009 10:30'!
unload
	SwaSourceTextStyler instance unregisterStyler: self registeredInstance.
	self registeredInstance: nil.! !

!CSFailedExpressionStyler methodsFor: 'styling' stamp: 'bs 12/22/2009 10:15'!
createFailedExpressionAttribute
	^ CSTextBackgroundColor new
		color: Color red darker;
		yourself! !

!CSFailedExpressionStyler methodsFor: 'styling' stamp: 'bs 12/22/2009 10:17'!
styleSource: aText of: aCompiledMethod
	| styledSourceText failedExpressionTextInterval |
	styledSourceText := aText.
	(aCompiledMethod isTestMethod and: [aCompiledMethod hasTestFailed])
		ifTrue: [
			failedExpressionTextInterval := aCompiledMethod failedExpression.
			styledSourceText 
				addAttribute: self createFailedExpressionAttribute
				from: failedExpressionTextInterval first
				to: failedExpressionTextInterval last ].
	^ styledSourceText.! !

Object subclass: #CSFixedAnalyzer
	instanceVariableNames: 'package codeRevisionsCoverage installer filenames methodModificationsOverTime methodAdditionsOverTime fixedRelationshipOverTime fixedCoverageDeltas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:18'!
addedMethodsAt: revision
	^ self methodAdditionsOverTime at: revision! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:18'!
addedMethodsAt: revision put: methodNames
	^ self methodAdditionsOverTime at: revision put: methodNames! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 08:28'!
analyzeCoverageForVersion: aVersionNumber	
	self 
		instrumentPackage;
		runAllTestsOfPackage;
		uninstrumentPackage;
		gatherCoverageDataAndStoreAsVersion: aVersionNumber
	! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:45'!
analyzeCoverageOverTime
	| alreadyRunThrough |
	alreadyRunThrough := 0.
	self filenames withIndexDo: [:filename :index |
		(index > alreadyRunThrough) ifTrue: [
			Transcript cr; show: index asString, ' started ... '.
			self installer uninstallPackage.
			Transcript show: 'uninstalled ...'.
			self installer installFromCache: filename.
			Transcript show: 'installed ...'.
			self analyzeCoverageForVersion: index.
			Transcript clear. ] ].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 11:32'!
analyzeFixedRelationshipsOverTime
	|  madeItRunInRevisions revision methodsAdded methodsModified madeItRun fixedTests previousRevision previousCoveringTestMethods coverageDifference previousMadeItRun |
	
	madeItRunInRevisions := fixedRelationshipOverTime := Dictionary new.
	filenames withIndexDo: [:filename :index |
		Transcript show: filename, ' started ... '.
		revision := self codeRevisionsCoverage at: index.
		methodsAdded := self addedMethodsAt: index.
		methodsModified := self modifiedMethodsAt: index.
	
		madeItRun := madeItRunInRevisions at: index put: Dictionary new.
	 
		revision keysAndValuesDo: [:appMethod :coveringTestMethods |
			fixedTests := madeItRun at: appMethod put: Set new.
			(methodsAdded includes: appMethod)
				ifTrue: [
					fixedTests addAll: coveringTestMethods ]
				ifFalse: [
					previousRevision := self codeRevisionsCoverage at: (index - 1).
					previousCoveringTestMethods := previousRevision at: appMethod.
					coverageDifference := coveringTestMethods difference: previousCoveringTestMethods.
				
					(methodsModified includes: appMethod)
						ifTrue: [ 
							fixedTests addAll: coverageDifference ]
						ifFalse: [
							previousCoveringTestMethods isEmpty
								ifTrue: [
									fixedTests addAll: coverageDifference ]
								ifFalse: [
									previousMadeItRun := (madeItRunInRevisions at: (index - 1)) at: appMethod.
									fixedTests addAll: (previousMadeItRun intersection: coveringTestMethods) ].
						 	].
				 	].
			]. 
		Transcript clear ].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 11:35'!
analyzeForMethodModifications
	| modifiedMethodNames previousRevision currentRevision patch modifications |
	previousRevision := nil.
	self filenames withIndexDo: [:filename :index |
		Transcript show: filename, ' started ... '.
		currentRevision := self installer readVersionFromCache: filename.
		modifiedMethodNames := self modifiedMethodsAt: index put: Set new.
		previousRevision ifNotNil: [
			patch := previousRevision package snapshot patchRelativeToBase: currentRevision snapshot.
			modifications := patch operations select: [:ea | ea isModification and: [ea obsoletion isMethodDefinition]].
			modifications do: [:ea | 
				modifiedMethodNames add: (ea obsoletion className, ' >> #', ea obsoletion selector) ] ].
		previousRevision := currentRevision.
		Transcript clear. ].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 13:02'!
analyzeForMethodsAdded
	| coverage previousCoverage addedMethodNames |
	self filenames withIndexDo: [:filename :index |
		Transcript show: filename, ' started ... '.
		coverage := self codeRevisionsCoverage at: index.
		(index = 1) 
			ifTrue: [ self addedMethodsAt: index put: coverage keys ]
			ifFalse: [
				previousCoverage := self codeRevisionsCoverage at: (index - 1).
				addedMethodNames := coverage keys reject: [:ea | previousCoverage includesKey: ea].
				self addedMethodsAt: index put: addedMethodNames ].
		Transcript clear. ].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:41'!
analyzeModificationsOverTime

	self 
		analyzeForMethodModifications;
		analyzeForMethodsAdded.! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:45'!
analyzePackage: aPackageInfo
	package := aPackageInfo..
	self 
		fetchVersionsToLocalCache;
		analyzeCoverageOverTime;
		analyzeModificationsOverTime;
		analyzeFixedRelationshipsOverTime.! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 13:12'!
analyzePackage: aPackageInfo asVersion: aVersionNumber
	
	package := aPackageInfo.
	
	self 
		instrumentPackage;
		runAllTestsOfPackage;
		uninstrumentPackage;
		gatherCoverageDataAndStoreAsVersion: aVersionNumber
	! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 10:35'!
classesAndMetaclasses
	^ package classes gather: [ :aClass | {aClass. aClass class} ]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 13:08'!
codeRevisionsCoverage
	^ codeRevisionsCoverage ifNil: [ codeRevisionsCoverage := Dictionary new. ]
	! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 11:14'!
computeFixedCoverageDeltas
	| deltasInRev coveringTests |
	self codeRevisionsCoverage keysAndValuesDo: [:revisionIndex :coverageInRev |
		deltasInRev := self fixedCoverageDeltas at: revisionIndex put: Dictionary new.
		(self fixedRelationshipAt: revisionIndex) keysAndValuesDo: [:appMethod :fixedTests | 
			coveringTests := coverageInRev at: appMethod.
			coveringTests ifNotEmpty: [
				deltasInRev at: appMethod put: (fixedTests size / coveringTests size) ].
	 	].
	].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 10:41'!
deltas
	^ deltas ifNil: [deltas := Dictionary new.]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:04'!
fetchVersionsToLocalCache
	
	self filenames: self installer fetchVersionsToLocalCache.! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:04'!
filenames
	^ filenames! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:04'!
filenames: aCollection
	filenames := aCollection! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 10:44'!
fixedCoverageDeltas
	^ fixedCoverageDeltas ifNil: [fixedCoverageDeltas := Dictionary new.]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 10:49'!
fixedCoverageDeltasAvg
	| deltasRevisionAvg |
	deltasRevisionAvg := Dictionary new.
	self fixedCoverageDeltas keysAndValuesDo: [:revisionIndex :deltasInRev |
		deltasRevisionAvg at: revisionIndex put: (deltasInRev values sum / deltasInRev values size) asFloat ].
	^ deltasRevisionAvg.! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:54'!
fixedRelationshipAt: revisionIndex
	^ self fixedRelationshipOverTime at: revisionIndex! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:55'!
fixedRelationshipAt: revisionIndex put: methodToFixedTestsDict
	^ self fixedRelationshipOverTime at: revisionIndex put: methodToFixedTestsDict.! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:53'!
fixedRelationshipOverTime
	^ fixedRelationshipOverTime ifNil: [ fixedRelationshipOverTime := Dictionary new. ]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 13:11'!
gatherCoverageDataAndStoreAsVersion: aVersionNumber
	| codeCoverage |
	codeCoverage := self codeRevisionsCoverage at: aVersionNumber put: Dictionary new.
	self nonTestMethods do: [ :aCompiledMethod |
		codeCoverage 
			at: (aCompiledMethod qualifiedName)
			put: (aCompiledMethod coveringTestMethods collect: [:ea | ea qualifiedName]) ].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 10:09'!
installer 
	^ installer ifNil: [ installer := CSMczInstaller newForPackageNamed: self package packageName ].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 13:13'!
instrumentPackage
	
	CSInstrumentor systemInstrumentor instrumentPackage: package.! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:12'!
methodAdditionsOverTime
	^ methodAdditionsOverTime ifNil: [ methodAdditionsOverTime := Dictionary new. ]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:11'!
methodModificationsOverTime
	^ methodModificationsOverTime ifNil: [ methodModificationsOverTime := Dictionary new. ]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:19'!
modifiedMethodsAt: revision
	^ self methodModificationsOverTime at: revision! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 09:19'!
modifiedMethodsAt: revision put: methodNames
	^ self methodModificationsOverTime at: revision put: methodNames! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 10:59'!
nonTestMethods
	^ self classesAndMetaclasses gather: [ :aClass | 
		aClass methodDict values reject: [:aCompiledMethod | 
			aCompiledMethod selector isTestSelector ] ]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 10:00'!
package
	^ package! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/26/2009 10:00'!
package: aPackageInfo
	package := aPackageInfo! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/23/2009 10:17'!
runAllTestsOfPackage	
	| aTestRunner aSuite |
	aTestRunner := CSTestRunner new.
	aSuite := TestSuite new.
	self testClasses do: [ :aTestClass | 
		aTestClass selectors 
			select: [:ea | ea isTestSelector ] 
			thenDo: [:ea | aSuite addTest: (aTestClass selector: ea) ].
		aTestRunner runSuite: aSuite ].! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 10:21'!
testClasses
	^ package classes select: [ :aClass | aClass inheritsFrom: TestCase ]! !

!CSFixedAnalyzer methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 13:13'!
uninstrumentPackage
	
	CSInstrumentor systemInstrumentor uninstrumentPackage: package.! !

Object subclass: #CSFixedExample
	instanceVariableNames: 'state world'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestExamples'!

!CSFixedExample methodsFor: 'as yet unclassified' stamp: 'bs 12/4/2009 14:44'!
foo
	state := 'Foo'.! !

!CSFixedExample methodsFor: 'as yet unclassified' stamp: 'bs 12/4/2009 15:27'!
hello
	world := 'Hello'.
	state := 'Foo'.
	"world := world, 'World'."! !

!CSFixedExample methodsFor: 'as yet unclassified' stamp: 'bs 12/4/2009 16:00'!
setStateToFoo
	state := 'Foo'.! !

!CSFixedExample methodsFor: 'as yet unclassified' stamp: 'bs 12/4/2009 14:31'!
state
	^ state! !

!CSFixedExample methodsFor: 'as yet unclassified' stamp: 'bs 12/4/2009 14:35'!
world
	^ world.! !

Object subclass: #CSInstrumentor
	instanceVariableNames: 'classes packages methods isActiveCounter isActive runningTestMethod handlesSystemEvents postProcessingHandlers'
	classVariableNames: 'SystemInstrumentor'
	poolDictionaries: ''
	category: 'CodeSight-TestTracer'!

!CSInstrumentor class methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 13:44'!
resetSystemInstrumentor
	^ SystemInstrumentor ifNotNil: [
		SystemInstrumentor stopHandlingSystemEvents.
		SystemInstrumentor := nil ].! !

!CSInstrumentor class methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 15:04'!
systemInstrumentor
	^ SystemInstrumentor ifNil: [
		SystemInstrumentor := CSInstrumentor new.
		SystemInstrumentor startHandlingSystemEvents ].! !

!CSInstrumentor methodsFor: 'tracing' stamp: 'bs 10/28/2009 14:54'!
activateTracing
	isActiveCounter ifNil: [isActiveCounter := 0].
	isActiveCounter := isActiveCounter + 1.
	isActive := isActiveCounter > 0.! !

!CSInstrumentor methodsFor: 'instrumenting' stamp: 'bs 10/28/2009 13:40'!
canInstrument: selector inClass: aClass
	^ CSCoverageTracingWrapper canWrap: selector inClass: aClass.! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:35'!
classAdded: anAddedEvent
	|aClass aClassPackage|
	aClass := anAddedEvent item.
	aClassPackage := PackageOrganizer default
		packageOfClass: aClass
		ifNone: [PackageInfo named: (Smalltalk organization categoryOfElement: aClass name)].
	
	(self instrumentedPackages includes: aClassPackage)
		ifTrue: [self instrumentClass: aClass].
		
	self systemEventPostProcessingHandlers do: [:handler |
		handler classAdded: anAddedEvent ]! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:36'!
classModified: aModifiedEvent
	|aClass|
	aClass := aModifiedEvent item.
	(self instrumentedClasses includes: aClass) 
		ifTrue: [
			self uninstrumentClass: aClass.
			self instrumentClass: aClass].
		
	self systemEventPostProcessingHandlers do: [:handler |
		handler classModified: aModifiedEvent ]! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:37'!
classRemoved: aRemovedEvent
	self uninstrumentClass: aRemovedEvent item.
	self systemEventPostProcessingHandlers do: [:handler |
		handler classRemoved: aRemovedEvent ]! !

!CSInstrumentor methodsFor: 'tracing' stamp: 'bs 10/28/2009 14:54'!
deactivateTracing
	isActiveCounter ifNil: [isActiveCounter := 0].
	isActiveCounter := isActiveCounter - 1.
	isActive := isActiveCounter > 0.! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 21:25'!
handlesSystemEvents
	^ handlesSystemEvents ifNil: [handlesSystemEvents := false].! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:37'!
handlesSystemEvents: aBoolean
	^ handlesSystemEvents := aBoolean! !

!CSInstrumentor methodsFor: 'instrumenting' stamp: 'bs 10/28/2009 14:59'!
installInstrumentation: selector inClass: aClass
	| wrapper |
	SystemChangeNotifier uniqueInstance doSilently: [
		wrapper := CSCoverageTracingWrapper on: selector inClass: aClass.
		wrapper install ].
	
	! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 13:29'!
instrumentClass: aClass
	|metaClass|
	metaClass := aClass class.
	(self instrumentedClasses includes: aClass) ifFalse: [
		{aClass. metaClass} do: [:c |
			self instrumentedClasses add: c.
			c selectors do: [:selector | self instrumentMethod: selector inClass: c ]]].! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 13:19'!
instrumentMethod: selector inClass: aClass
	| registeredMethods |
	((aClass inheritsFrom: TestCase) and: [selector isTestSelector])
		ifFalse: [
			registeredMethods := self instrumentedMethodsAt: aClass.
			(registeredMethods includes: selector) ifFalse: [
				(self canInstrument: selector inClass: aClass) ifTrue: [
					self installInstrumentation: selector inClass: aClass.
					registeredMethods add: selector.]]]! !

!CSInstrumentor methodsFor: 'instrumenting' stamp: 'bs 10/28/2009 13:31'!
instrumentPackage: aPackage
	(self instrumentedPackages includes: aPackage) ifFalse: [
		self instrumentedPackages add: aPackage.
		aPackage classes do: [:aClass | self instrumentClass: aClass]]! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 14:09'!
instrumentedClasses
	"classes := nil"
	^ classes ifNil: [classes := WeakSet new]! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 14:09'!
instrumentedMethods
	^ methods ifNil: [methods := WeakKeyDictionary new] ! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 13:17'!
instrumentedMethodsAt: aClass
	^ self instrumentedMethods at: aClass ifAbsentPut: [Set new]
	
	"(self registeredPackages includes: aPackage) ifFalse: [
		self registeredPackages add: aPackage.
		aPackage classes reject: [:aClass | aClass inheritsFrom: TestCase]
							  thenDo: [:class | class selectors do: [:selector | 
				(self on: selector inClass: class) ifNotNilDo: [:wrapper | wrapper install]]]]"! !

!CSInstrumentor methodsFor: 'instrumenting' stamp: 'bs 10/28/2009 14:09'!
instrumentedPackages
	"packages := nil"
	^ packages ifNil: [packages := WeakSet new]! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 13:21'!
isMethodInstrumented: selector inClass: aClass
	^ (self instrumentedMethodsAt: aClass) includes: selector
	! !

!CSInstrumentor methodsFor: 'tracing' stamp: 'bs 10/28/2009 16:47'!
isTracingActivated
	^ isActive ifNil: [isActive := false]! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:37'!
methodAdded: anAddedEvent
	| selector class |
	selector := anAddedEvent itemSelector.
	class := anAddedEvent itemClass.
	
	(self instrumentedClasses includes: class) ifTrue: [
		self instrumentMethod: selector inClass: class].
	
	self systemEventPostProcessingHandlers do: [:handler | 
			handler methodAdded: anAddedEvent ].! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:38'!
methodModified: aModifiedEvent
	| selector class |
	
	selector := aModifiedEvent itemSelector.
	class := aModifiedEvent itemClass.
	
	(self isMethodInstrumented: selector inClass: class) ifTrue: [
		self installInstrumentation: selector inClass: class ].
	
	self systemEventPostProcessingHandlers do: [:handler | 
		handler methodModified: aModifiedEvent ]! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:38'!
methodRemoved: aRemovedEvent
		| selector class |
	selector := aRemovedEvent itemSelector.
	class := aRemovedEvent itemClass.
	
	(self isMethodInstrumented: selector inClass: class) ifTrue: [
		self uninstrumentMethod: selector inClass: class].
	
	self systemEventPostProcessingHandlers do: [:handler | 
		handler methodRemoved: aRemovedEvent ]! !

!CSInstrumentor methodsFor: 'tracing' stamp: 'bs 10/28/2009 14:54'!
runningTestMethod
	^ runningTestMethod
	! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:35'!
startHandlingSystemEvents
	"CSCoverageTracingWrapper startHandlingSystemEvents"
	
	|notifier events eventItem handlerMethodSelector|
	events := {#method -> {#Added. #Modified. #Removed}. #class -> {#Added. #Modified. #Removed}}.
	self handlesSystemEvents ifFalse: [
		self handlesSystemEvents: true.
		notifier := SystemChangeNotifier uniqueInstance.
		events do: [:assoc | 
			eventItem := assoc key.
			assoc value do: [:changeKind | 
				handlerMethodSelector := (eventItem, changeKind, ':') asSymbol.
				notifier notify: self ofSystemChangesOfItem: eventItem 
					change: changeKind using: handlerMethodSelector]]]! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 13:35'!
stopHandlingSystemEvents
	"self stopHandlingSystemEvents" 
	
	self handlesSystemEvents ifTrue: [
		self handlesSystemEvents: false.
		SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self]! !

!CSInstrumentor methodsFor: 'system event handling' stamp: 'bs 10/28/2009 21:26'!
systemEventPostProcessingHandlers
	^ postProcessingHandlers ifNil: [postProcessingHandlers := Set new]! !

!CSInstrumentor methodsFor: 'tracing' stamp: 'bs 10/28/2009 15:34'!
testMethodGoingToRun: aCompiledMethod
	runningTestMethod := aCompiledMethod
	! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 13:12'!
uninstallInstrumentation: selector inClass: aClass
	SystemChangeNotifier uniqueInstance doSilently: [
		aClass methodDict at: selector ifPresent: [ :aMethod | 
			aMethod isMwMethodWrapper ifTrue: [ aMethod uninstall ] ] ]
	
	! !

!CSInstrumentor methodsFor: 'instrumenting' stamp: 'bs 10/28/2009 13:34'!
uninstrumentAllAndReset
	self instrumentedPackages do: [:ea | self uninstrumentPackage: ea].
	self instrumentedClasses do: [:ea | self uninstrumentClass: ea].
	self instrumentedMethods keysAndValuesDo: [ :aClass :selectors | 
		selectors do: [:s | self uninstrumentMethod: s inClass: aClass] ].
	self instrumentedMethods keysAndValuesRemove: [:k :v| true].! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 10/28/2009 13:16'!
uninstrumentClass: aClass
	| metaClass |
	metaClass := aClass class.
	(self instrumentedClasses includes: aClass)
		ifTrue: [
			self instrumentedClasses remove: aClass.
			self instrumentedClasses remove: metaClass ifAbsent:[]].
	
	{aClass. metaClass} do: [:c |
		c selectors do: [:selector | self uninstrumentMethod: selector inClass: c ]].! !

!CSInstrumentor methodsFor: 'registration' stamp: 'bs 11/26/2009 14:58'!
uninstrumentMethod: selector inClass: aClass
	| registeredMethods |
	registeredMethods := self instrumentedMethodsAt: aClass.
	(registeredMethods includes: selector) 
		ifTrue: [
			self uninstallInstrumentation: selector inClass: aClass.
			registeredMethods remove: selector.
			registeredMethods ifEmpty: [self instrumentedMethods removeKey: aClass] ].! !

!CSInstrumentor methodsFor: 'instrumenting' stamp: 'bs 10/28/2009 13:34'!
uninstrumentPackage: aPackage
	(self instrumentedPackages includes: aPackage) ifTrue: [
		aPackage classes do: [:aClass | self uninstrumentClass: aClass].
		self instrumentedPackages remove: aPackage]! !

Object subclass: #CSMczCache
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!CSMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 14:28'!
addMczToCache: aStream at: aMczFileName
	"Need to write a clean zip."
	
	^ ZipArchive new
		readFrom: aStream;
		writeToFileNamed: (self cacheDirectory fullNameFor: aMczFileName).! !

!CSMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 14:35'!
cacheAt: aFileName

	^ ZipArchive new
		readFrom: (self cacheDirectory readOnlyFileNamed: aFileName)! !

!CSMczCache class methodsFor: 'caching' stamp: 'mt 5/26/2009 18:48'!
cacheAt: aFileName ifNoneGetFrom: anHttpRepository
	"Return a zip archive for the mcz file."

	^ (self includes: aFileName)
		ifTrue: [self cacheAt: aFileName]
		ifFalse: [
			anHttpRepository
				readStreamForFileNamed: aFileName
				do: [:strm | self addMczToCache: strm at: aFileName]]! !

!CSMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 12:45'!
cacheDirectory

	^ (FileDirectory default directoryNamed: 'codetalk-cache')
		assureExistence;
		yourself! !

!CSMczCache class methodsFor: 'private' stamp: 'mt 5/15/2009 13:25'!
includes: aFileName

	^ self cacheDirectory includesKey: aFileName! !

Object subclass: #CSMczInspectorOld
	instanceVariableNames: 'isRunning process reader currentMethodDefinition followAncestors currentAncestors classFilter selectorFilter lastDefinition currentStep maxSteps repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

CSMczInspectorOld class
	instanceVariableNames: 'exampleCache'!

CSMczInspectorOld class
	instanceVariableNames: 'exampleCache'!

!CSMczInspectorOld class methodsFor: 'examples' stamp: 'bs 11/20/2009 16:39'!
exampleVersions
	| instance result repo |
	exampleCache ifNotNilDo: [ :c | ^ c ].
	result := OrderedCollection new.
	instance := self newForPackageNamed: 'Xpf-CodeTalk'.
	repo := instance repositories first key.
	(repo readableFileNames select: [ :filename | filename startsWith: 'Xpf-CodeTalk' ]) 
		do: 
			[ :filename | 
			instance zip: (CSMczCache 
					cacheAt: filename
					ifNoneGetFrom: repo).
			instance loadVersionInfo.
			(instance zip memberNamed: 'snapshot.bin') ifNotNilDo: 
				[ :bin | 
				
				[ (DataStream on: bin contentStream) next definitions 
					select: 
						[ :def | 
						def isMethodDefinition and: [ def className = 'XpfCodeTalk' and: [ def selector = #todo ] ] ]
					thenDo: 
						[ :def | 
						result add: (CSRemoteMethodVersion 
								repository: repo
								info: instance info
								definition: def) ] ] 
					on: Error
					do: 
						[ :fallThrough | 
						"Unknown content in snapshot.bin!!"
						 ] ] ]
		displayingProgress: 'Preparing...'.
	exampleCache := result.
	^ result! !

!CSMczInspectorOld methodsFor: 'adding repos' stamp: 'rkrk 9/19/2009 19:04'!
addRepository: anHttpRepository include: somePackageNames
	"Add the given repo and decide which packages
	to use when looking at the contents of the repo later."
	
	self repositories add: (anHttpRepository -> somePackageNames).! !

!CSMczInspectorOld methodsFor: 'accessing' stamp: 'rkrk 9/19/2009 19:03'!
currentAncestors

	^ currentAncestors ifNil: [currentAncestors := Set new].! !

!CSMczInspectorOld methodsFor: 'inspecting' stamp: 'bs 11/20/2009 17:15'!
filenames
	| packageName aHttpRepository | 
	
	packageName := self repositories first value.
	aHttpRepository := self repositories first key.
	^ self
		filterFileNames: aHttpRepository readableFileNames
		packages: packageName.! !

!CSMczInspectorOld methodsFor: 'inspecting' stamp: 'rkrk 9/19/2009 23:37'!
filterFileNames: someFileNames packages: somePackageNames

	| result fileNamesMatching |
	fileNamesMatching := someFileNames select: [:filename |
		somePackageNames anySatisfy: [:name |
			self packageName: name matchesFileName: filename]].
	result := fileNamesMatching sortBy: [:f1 :f2 |
		(f1 findTokens: $.) second asNumber > (f2 findTokens: $.) second asNumber].
	^ self followAncestors
		ifTrue: [result]
		ifFalse: [result reversed]! !

!CSMczInspectorOld methodsFor: 'accessing' stamp: 'rkrk 9/19/2009 19:02'!
followAncestors

	^ followAncestors ifNil: [followAncestors := false].! !

!CSMczInspectorOld methodsFor: 'accessing' stamp: 'mt 5/13/2009 14:13'!
followAncestors: anObject

	followAncestors := anObject.! !

!CSMczInspectorOld methodsFor: 'inspecting - methods' stamp: 'rkrk 9/19/2009 22:53'!
inspectDefinitions

	| definition updateObject |
	definition := self reader definitions
		detect: [:def | def isMethodDefinition
			and: [classFilter = def actualClass]
			and: [selectorFilter = def selector]]
		ifNone: [^ self].
	lastDefinition = definition ifTrue: [^ self].
	updateObject := Dictionary newFrom: {
		#repository -> self reader repository.
		#definition -> definition.
		#info -> self reader info}.
	self changed: #matchingVersionFound with: updateObject.
	lastDefinition := definition.! !

!CSMczInspectorOld methodsFor: 'inspecting' stamp: 'bs 11/20/2009 18:06'!
inspectRepository: anHttpRepository include: somePackageNames

	| filenames shouldInspectDefs |
	filenames := self
		filterFileNames: anHttpRepository readableFileNames
		packages: somePackageNames.
	self maxSteps: filenames size.		
	filenames withIndexDo: [:filename :i |	
		self currentStep: i.
		self reader readFile: filename from: anHttpRepository.
		self inspectVersion.
		shouldInspectDefs := self followAncestors not
			or: [self currentAncestors isEmpty]
			or: [self currentAncestors anySatisfy: [:anc | anc name = self reader info name]].
		shouldInspectDefs ifTrue: [self inspectDefinitions].
		self followAncestors ifTrue: [self updateCurrentAncestry]].! !

!CSMczInspectorOld methodsFor: 'inspecting - methods' stamp: 'rkrk 9/19/2009 19:29'!
inspectSourceText: aText

	| result |
	result := aText attributesOfKind: XpfComplexAttribute.

	self changed: #resultFound with: (Dictionary newFrom: {
		#repository -> self reader repository.
		#definition -> currentMethodDefinition.
		#info -> self reader info.
		#attributes -> result}).! !

!CSMczInspectorOld methodsFor: 'inspecting - versions' stamp: 'rkrk 9/19/2009 19:36'!
inspectVersion

	self changed: #versionFound with: self reader info.! !

!CSMczInspectorOld methodsFor: 'accessing' stamp: 'mt 5/9/2009 12:03'!
isRunning

	isRunning ifNil: [self isRunning: false].
	^ isRunning! !

!CSMczInspectorOld methodsFor: 'accessing' stamp: 'mt 5/9/2009 12:03'!
isRunning: anObject

	isRunning := anObject.
	self changed: #isRunning.! !

!CSMczInspectorOld methodsFor: 'testing' stamp: 'rkrk 9/19/2009 23:40'!
packageName: packageName matchesFileName: fileName

	"match only those packages which really include packageNames and not only start
	with that name.
	E.g. when somePackageNames = {Xpf-CodeTalk} don't match Xpf-CodeTalkExtension'"
	| packageNameDelimiter versionDelimiter followingChar |
	packageNameDelimiter := $-.
	versionDelimiter := $..
	(fileName beginsWith: packageName) ifFalse: [^ false].
	fileName size = packageName size ifTrue: [true].
	followingChar := fileName at: packageName size + 1.
	^ followingChar = packageNameDelimiter or: [followingChar = versionDelimiter].
! !

!CSMczInspectorOld methodsFor: 'debugging' stamp: 'mt 5/26/2009 13:48'!
print

	Transcript show: ('Next Result: {1}>>{2} ({3}) - {4}' format: {
		currentMethodDefinition className.
		currentMethodDefinition selector.
		currentMethodDefinition category.
		self reader info name}); cr.! !

!CSMczInspectorOld methodsFor: 'accessing' stamp: 'rkrk 9/19/2009 19:04'!
repositories

	^ repositories ifNil: [repositories := OrderedCollection new].! !

!CSMczInspectorOld methodsFor: 'controlling' stamp: 'rkrk 9/19/2009 23:16'!
start

	process ifNotNil: [self stop].
	process := [
		[self isRunning: true.
		self repositories do: [:repo |
			self
				inspectRepository: repo key
				include: repo value.
			currentAncestors := nil].
		] ensure: [
			self
				isRunning: false;
				changed: #done]
	] fork priority: 20.! !

!CSMczInspectorOld methodsFor: 'inspecting - versions' stamp: 'rkrk 9/19/2009 23:09'!
startInspectVersions

	self
		followAncestors: true;
		start.! !

!CSMczInspectorOld methodsFor: 'inspecting - versions' stamp: 'rkrk 9/19/2009 23:09'!
startInspectVersionsFor: aSelector in: aClass

	classFilter := aClass.
	selectorFilter := aSelector.
	self startInspectVersions.! !

!CSMczInspectorOld methodsFor: 'controlling' stamp: 'mt 5/9/2009 12:13'!
stop

	process ifNotNilDo: [:p | p terminate].
	process := nil.
	self isRunning: false.! !

!CSMczInspectorOld methodsFor: 'inspecting' stamp: 'mt 6/3/2009 11:05'!
updateCurrentAncestry

	self reader info ancestors ifNotNilDo: [:newAnc |
		self currentAncestors addAll: newAnc].! !

Object subclass: #CSMczInstaller
	instanceVariableNames: 'repository packageName currentStep maxSteps currentAncestors reader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!CSMczInstaller class methodsFor: 'instance creation' stamp: 'bs 11/26/2009 10:14'!
newForPackageNamed: aString

	| aPackage instance repos httpRepo |
	aPackage := MCPackage named: aString.
	aPackage hasWorkingCopy ifFalse: [^ nil].
	repos := aPackage workingCopy repositoryGroup repositories.
	httpRepo := repos detect: [:r | r isKindOf: MCHttpRepository ].
	
	instance := self new.
	instance 
		repository: httpRepo;
		packageName: aPackage name.
	^ instance ! !

!CSMczInstaller class methodsFor: 'instance creation' stamp: 'bs 11/20/2009 17:29'!
newForPackagesNamed: packageName inRepo: mcRepository

	| instance |
	instance := self new.
	instance
		repository: mcRepository;
		packageName: packageName.
	^ instance! !

!CSMczInstaller methodsFor: 'as yet unclassified' stamp: 'bs 11/21/2009 20:36'!
cacheAt: aFileName

	^ self cacheDirectory readOnlyFileNamed: aFileName! !

!CSMczInstaller methodsFor: 'as yet unclassified' stamp: 'bs 11/21/2009 20:35'!
cacheDirectory

	^ (FileDirectory default directoryNamed: 'codetalk-cache')
		assureExistence;
		yourself! !

!CSMczInstaller methodsFor: 'as yet unclassified' stamp: 'bs 11/21/2009 21:09'!
fetchVersionsToLocalCache
	| filenames |
	filenames := (self filterFileNames: self repository readableFileNames).	
	filenames do: [:filename |	
		self reader readFile: filename from: self repository ].
	^ filenames! !

!CSMczInstaller methodsFor: 'as yet unclassified' stamp: 'bs 11/21/2009 21:09'!
filterFileNames: someFileNames
	| result fileNamesMatching |
	fileNamesMatching := someFileNames select: [:filename | self packageMatchesFileName: filename].
	result := fileNamesMatching sortBy: [:f1 :f2 |
		(f1 findTokens: $.) second asNumber > (f2 findTokens: $.) second asNumber].
	^ result reversed! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/21/2009 20:46'!
installFromCache: aFileName
	MczInstaller install: aFileName stream: (self cacheAt: aFileName).! !

!CSMczInstaller methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 17:29'!
packageMatchesFileName: fileName

	"match only those packages which really include packageNames and not only start
	with that name.
	E.g. when somePackageNames = {Xpf-CodeTalk} don't match Xpf-CodeTalkExtension'"
	| packageNameDelimiter versionDelimiter followingChar |
	packageNameDelimiter := $-.
	versionDelimiter := $..
	(fileName beginsWith: self packageName) ifFalse: [^ false].
	fileName size = self packageName size ifTrue: [true].
	followingChar := fileName at: self packageName size + 1.
	^ followingChar = packageNameDelimiter or: [followingChar = versionDelimiter].
! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/20/2009 17:29'!
packageName

	^ packageName! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/20/2009 17:29'!
packageName: aName

	^ packageName := aName! !

!CSMczInstaller methodsFor: 'as yet unclassified' stamp: 'bs 11/20/2009 17:28'!
packageName: packageName matchesFileName: fileName

	"match only those packages which really include packageNames and not only start
	with that name.
	E.g. when somePackageNames = {Xpf-CodeTalk} don't match Xpf-CodeTalkExtension'"
	| packageNameDelimiter versionDelimiter followingChar |
	packageNameDelimiter := $-.
	versionDelimiter := $..
	(fileName beginsWith: packageName) ifFalse: [^ false].
	fileName size = packageName size ifTrue: [true].
	followingChar := fileName at: packageName size + 1.
	^ followingChar = packageNameDelimiter or: [followingChar = versionDelimiter].
! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/23/2009 21:36'!
readVersionFromCache: aFileName
	^ MCMczReader versionFromStream: (self cacheAt: aFileName)! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/21/2009 21:10'!
reader
	^ reader ifNil: [ reader := CSMczReader new ]! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/20/2009 17:25'!
repository

	^ repository! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/20/2009 17:25'!
repository: aHttpRepo
	repository := aHttpRepo.! !

!CSMczInstaller methodsFor: 'accessing' stamp: 'bs 11/21/2009 20:52'!
uninstallPackage
	
	(PackageInfo named: self packageName) classes do: [:aClass | aClass removeFromSystem ]! !

Object subclass: #CSMczReader
	instanceVariableNames: 'zip info definitions repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Analyzing'!

!CSMczReader commentStamp: 'mt 6/3/2009 11:04' prior: 0!
This is a simplified version of the McMczReader.!

!CSMczReader methodsFor: 'reading' stamp: 'mt 5/26/2009 13:11'!
associate: tokens

	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					value isString ifFalse: [value := value collect: [:ea | self associate: ea]].
					result at: key put: value].
	^ result! !

!CSMczReader methodsFor: 'accessing' stamp: 'bs 11/20/2009 16:59'!
definitions

	"if it does not work, look at loadDefinitions"
	^ definitions ifNil: [
		(self zip memberNamed: 'snapshot.bin') ifNotNilDo: [:bin |
			[definitions := (DataStream on: bin contentStream) next definitions]
				on: Error
				do: [:fallThrough | definitions := OrderedCollection new]]].! !

!CSMczReader methodsFor: 'reading' stamp: 'mt 5/26/2009 13:11'!
extractInfoFrom: dict

	^ MCVersionInfo
		name: (dict at: #name ifAbsent: [''])
		id: (UUID fromString: (dict at: #id))
		message: (dict at: #message ifAbsent: [''])
		date: ([Date fromString: (dict at: #date) ] on: Error do: [ :ex | ex return: nil ])
		time: ([ Time fromString:(dict at: #time)] on: Error do: [ :ex | ex return: nil ])
		author: (dict at: #author ifAbsent: [''])
		ancestors: ((dict at: #ancestors ifAbsent: [#()]) collect: [:ea | self extractInfoFrom: ea])
		stepChildren: ((dict at: #stepChildren ifAbsent: [#()]) collect: [:ea | self extractInfoFrom: ea])! !

!CSMczReader methodsFor: 'public' stamp: 'mt 5/26/2009 13:58'!
getDefinitionByClass: aClass selector: aSelector

	^ self definitions
		detect: [:def |
			def actualClass = aClass
				and: [def selector = aSelector]]
		ifNone: []! !

!CSMczReader methodsFor: 'accessing' stamp: 'bs 11/20/2009 17:00'!
info

	^ info ifNil: [info := self extractInfoFrom: (self parseMember: 'version')]! !

!CSMczReader methodsFor: 'accessing' stamp: 'mt 5/26/2009 13:09'!
info: anObject

	info := anObject.! !

!CSMczReader methodsFor: 'reading' stamp: 'mt 5/26/2009 13:17'!
loadDefinitions

	(self zip memberNamed: 'snapshot.bin') ifNotNilDo: [:bin |
		[self	definitions:
			(DataStream on: bin contentStream) next definitions]
		on: Error
		do: [:fallThrough |
			self definitions: OrderedCollection new.
			Transcript show: 'Unknown content in snapshot.bin!!'; cr]].! !

!CSMczReader methodsFor: 'reading' stamp: 'mt 5/26/2009 13:11'!
loadVersionInfo

	info := self extractInfoFrom: (self parseMember: 'version')! !

!CSMczReader methodsFor: 'reading' stamp: 'mt 5/26/2009 13:12'!
parseMember: fileName

	| tokens |
	tokens := (MCScanner scanTokens: (zip contentsOf: fileName)) first.
	^ self associate: tokens! !

!CSMczReader methodsFor: 'public' stamp: 'bs 11/20/2009 17:01'!
readFile: aFilename from: aRepository 
	self
		repository: aRepository;
		zip: (CSMczCache 
				cacheAt: aFilename
				ifNoneGetFrom: aRepository)! !

!CSMczReader methodsFor: 'accessing' stamp: 'mt 5/26/2009 13:32'!
repository

	^ repository! !

!CSMczReader methodsFor: 'accessing' stamp: 'mt 5/26/2009 13:32'!
repository: anObject

	repository := anObject.! !

!CSMczReader methodsFor: 'accessing' stamp: 'mt 5/26/2009 13:09'!
zip

	^ zip! !

!CSMczReader methodsFor: 'accessing' stamp: 'mt 5/26/2009 13:09'!
zip: anObject

	zip := anObject.! !

Object subclass: #CSRegisteredTrace
	instanceVariableNames: 'identifier timestamp startEntries returnValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tracer'!

!CSRegisteredTrace class methodsFor: 'as yet unclassified' stamp: 'mp 2/10/2009 10:57'!
with: anIdentifier

	^ self new
		identifier: anIdentifier! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/5/2009 11:02'!
identifier
	"Answer the value of identifier"

	^ identifier! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/5/2009 11:02'!
identifier: anObject
	"Set the value of identifier"

	identifier := anObject! !

!CSRegisteredTrace methodsFor: 'initialize-release' stamp: 'mp 2/10/2009 10:57'!
initialize
	
	self 
		timestamp: DateAndTime now;
		startEntries: OrderedCollection new.! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/7/2009 16:38'!
returnValue
	"Answer the value of returnValue"

	^ returnValue! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/7/2009 16:38'!
returnValue: anObject
	"Set the value of returnValue"

	returnValue := anObject! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/6/2009 15:16'!
startEntries
	"Answer the value of startEntries"

	^ startEntries! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/6/2009 15:16'!
startEntries: anObject
	"Set the value of startEntries"

	startEntries := anObject! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/5/2009 11:02'!
timestamp
	"Answer the value of timestamp"

	^ timestamp! !

!CSRegisteredTrace methodsFor: 'accessing' stamp: 'mp 2/5/2009 11:02'!
timestamp: anObject
	"Set the value of timestamp"

	timestamp := anObject! !

Object subclass: #CSStepper
	instanceVariableNames: 'blockToExecute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestTracer'!

!CSStepper methodsFor: 'as yet unclassified' stamp: 'bs 4/16/2009 12:01'!
executeInNextStep: aBlock
	blockToExecute := aBlock.
	ActiveWorld startStepping: self at: Time millisecondClockValue selector: #step arguments: nil stepTime: nil.! !

!CSStepper methodsFor: 'as yet unclassified' stamp: 'bs 4/16/2009 12:02'!
shouldGetStepsFrom: aWorld
	^ActiveWorld == aWorld! !

!CSStepper methodsFor: 'as yet unclassified' stamp: 'bs 4/16/2009 12:01'!
step
	blockToExecute value.
	ActiveWorld stopStepping: self selector: #step.! !

Object subclass: #CSTestStatistic
	instanceVariableNames: 'receiverClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestTracer'!

!CSTestStatistic methodsFor: 'initialize-release' stamp: 'bs 10/28/2009 15:19'!
addCallInDepth: aNumber
	self flag: #rkrk. "for benchmarking"! !

!CSTestStatistic methodsFor: 'public' stamp: 'bs 10/28/2009 15:26'!
addReceiver: anObject
	self receiverClasses add: anObject class.! !

!CSTestStatistic methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 15:18'!
printOn: stream

	stream
		nextPutAll: 'CSTestStatistic<Trace:';
		nextPutAll: 'Receivers:';
		print: self receiverClasses;
		nextPut: $>.! !

!CSTestStatistic methodsFor: 'accessing' stamp: 'bs 9/30/2009 16:17'!
receiverClasses
	^ receiverClasses ifNil: [receiverClasses := IdentitySet new]! !

Object subclass: #CSTraceChooser
	instanceVariableNames: 'list listSelections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestTracer'!

!CSTraceChooser class methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:20'!
allClasses
	| set |
	set := Set new.
	CSInstrumentor systemInstrumentor instrumentedPackages do: [ :aPackage |
		set addAll: aPackage classes ].
	^ set! !

!CSTraceChooser class methodsFor: 'as yet unclassified' stamp: 'mab 4/1/2009 19:11'!
build
	^ ToolBuilder build: self new! !

!CSTraceChooser class methodsFor: 'as yet unclassified' stamp: 'mab 4/1/2009 19:07'!
initialize
	"CSTraceChooser initialize"
	self registerInToolsFlap! !

!CSTraceChooser class methodsFor: 'as yet unclassified' stamp: 'mab 4/3/2009 15:31'!
open
	^ ToolBuilder open: self new! !

!CSTraceChooser class methodsFor: 'as yet unclassified' stamp: 'mab 4/2/2009 18:52'!
packages
	^ Packages ifNil: [ Packages := Set new ]! !

!CSTraceChooser class methodsFor: 'as yet unclassified' stamp: 'mab 4/1/2009 19:12'!
registerInToolsFlap
	self environment at: #Flaps ifPresent: [ :class |
		class
			registerQuad: #( CSTraceChooser build 'Test Traces' 'Choose your packages for test tracing.' ) forFlapNamed: 'Tools';
			replaceToolsFlap ].! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/4/2009 15:51'!
buildWith: builder
	| windowSpec listSpec buttonSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec 
		model: self;
		label: 'Test Trace Chooser';
		children: OrderedCollection new;
		closeAction: #close.
		
	listSpec := builder pluggableMultiSelectionListSpec new.
	listSpec 
		model: self;
		list: #list;
		getSelectionList: #listSelectionAt:;
		setSelectionList: #listSelectionAt:put:;
		frame: (0@0corner: 1@0.8).
	windowSpec children add: listSpec.
	
	buttonSpec := builder pluggableButtonSpec new.
	buttonSpec
		model: self;
		label: 'Refresh';
		action: #refresh;
		frame: (0@0.8 corner: 1@1).
	windowSpec children add: buttonSpec.
	
	^builder build: windowSpec! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/4/2009 15:48'!
close! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/24/2009 08:29'!
list
	^ list ifNil: [ list := self packageNamesSorted ]! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'mab 4/1/2009 18:07'!
listSelectionAt: index
	^ self listSelections at: index! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/4/2009 15:14'!
listSelectionAt: index put: isSelected
	
	isSelected 
		ifTrue: [self registerPackage: (self packageAt: index).]
		ifFalse: [self unregisterPackage: (self packageAt: index) ].
	
	^ self listSelections at: index put: isSelected! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/4/2009 15:14'!
listSelections
	listSelections ifNil: [
		self list ifNotNil: [
			listSelections := Array new: self list size withAll: false.
			self list doWithIndex: [ :each :index |
				(self registeredPackages includes: each) ifTrue: [
					listSelections at: index put: true ]]]].
	^ listSelections! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/4/2009 15:11'!
packageAt: index
	| packageName |
	packageName := self list at: index.
	^ self packageNamed: packageName! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/4/2009 15:10'!
packageNamed: aName
	^ PackageOrganizer default packageNamed: aName ifAbsent: [].! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/24/2009 08:36'!
packageNamesSorted
	| packageNames criticalPackages |
	packageNames := PackageOrganizer default packages 
			collect: [ :packageInfo | packageInfo packageName ].
	
	criticalPackages := {'Collections'. 'CollectionsTests'.
								'Kernel'. 'KernelTests'.
								'Graphics'. 'GraphicsTests'.
								'ST80'. 'SUnit'. 'System' }.
	^ (packageNames reject: [:package | 
			criticalPackages anySatisfy: [:critical | 
				critical = package] ]) sort! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 9/4/2009 15:51'!
refresh
	list := nil.
	listSelections := nil.! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:19'!
registerPackage: aPackage
	^ CSInstrumentor systemInstrumentor instrumentPackage: aPackage! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:19'!
registeredPackages
	^ CSInstrumentor systemInstrumentor instrumentedPackages! !

!CSTraceChooser methodsFor: 'as yet unclassified' stamp: 'bs 10/28/2009 14:20'!
unregisterPackage: aPackage
	^ CSInstrumentor systemInstrumentor uninstrumentPackage: aPackage! !

Object subclass: #CSTraceEntry
	instanceVariableNames: 'methodClass methodSelector depth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tracer'!

CSTraceEntry subclass: #CSTraceCallEntry
	instanceVariableNames: 'receiver arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tracer'!

!CSTraceCallEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:53'!
arguments
	"Answer the value of arguments"

	^ arguments! !

!CSTraceCallEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:53'!
arguments: anObject
	"Set the value of arguments"

	arguments := anObject! !

!CSTraceCallEntry methodsFor: 'iterating' stamp: 'mp 2/8/2009 12:18'!
getNextMethodPropertiesFor: anIdentifier

	^ (self methodClass methodDict at: self methodSelector) 
		properties at: anIdentifier! !

!CSTraceCallEntry methodsFor: 'testing' stamp: 'mp 2/4/2009 17:56'!
isCall

	^ true! !

!CSTraceCallEntry methodsFor: 'iterating' stamp: 'mp 2/9/2009 15:55'!
iterateNextMethodTrace: anIdentifier	withNextCallBlock: aCallBlock andReturnBlock: aReturnBlock
	
	(self getNextMethodPropertiesFor: anIdentifier)	
		do: [:each | 
			each
				predecessor: self
				ifNextCall: aCallBlock
				ifReturn: aReturnBlock]! !

!CSTraceCallEntry methodsFor: 'testing' stamp: 'mp 2/10/2009 10:48'!
predecessor: aCallEntry ifNextCall: aCallBlock ifReturn: aReturnBlock
	
	(self depth size = (aCallEntry depth size + 1) and: [self depth beginsWith: aCallEntry depth])
		ifTrue: [aCallBlock value: self].! !

!CSTraceCallEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:53'!
receiver
	"Answer the value of receiver"

	^ receiver! !

!CSTraceCallEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:53'!
receiver: anObject
	"Set the value of receiver"

	receiver := anObject! !

!CSTraceEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 18:21'!
depth
	"Answer the value of depth"

	^ depth! !

!CSTraceEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 18:21'!
depth: anObject
	"Set the value of depth"

	depth := anObject! !

!CSTraceEntry methodsFor: 'initialize-release' stamp: 'mp 2/11/2009 13:01'!
initialize

	self depth: OrderedCollection new.! !

!CSTraceEntry methodsFor: 'testing' stamp: 'mp 2/4/2009 17:56'!
isCall
	
	^ false! !

!CSTraceEntry methodsFor: 'testing' stamp: 'mp 2/4/2009 17:56'!
isReturn

	^ false! !

!CSTraceEntry methodsFor: 'accessing' stamp: 'mp 2/13/2009 13:24'!
methodClass
	"Answer the value of methodClass"

	^ methodClass! !

!CSTraceEntry methodsFor: 'accessing' stamp: 'mp 2/13/2009 13:24'!
methodClass: anObject
	"Set the value of methodClass"

	methodClass := anObject! !

!CSTraceEntry methodsFor: 'accessing' stamp: 'mp 2/13/2009 13:24'!
methodSelector
	"Answer the value of methodSelector"

	^ methodSelector! !

!CSTraceEntry methodsFor: 'accessing' stamp: 'mp 2/13/2009 13:24'!
methodSelector: anObject
	"Set the value of methodSelector"

	methodSelector := anObject! !

!CSTraceEntry methodsFor: 'testing' stamp: 'mp 2/8/2009 12:42'!
predecessor: aCallEntry ifNextCall: aCallBlock ifReturn: aReturnBlock

	self subclassResponsibility.! !

CSTraceEntry subclass: #CSTraceReturnEntry
	instanceVariableNames: 'returnValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tracer'!

!CSTraceReturnEntry methodsFor: 'testing' stamp: 'mp 2/4/2009 17:56'!
isReturn

	^ true! !

!CSTraceReturnEntry methodsFor: 'testing' stamp: 'mp 2/9/2009 15:55'!
predecessor: aCallEntry ifNextCall: aCallBlock ifReturn: aReturnBlock

	self depth = aCallEntry depth
		ifTrue: [aReturnBlock value: self].! !

!CSTraceReturnEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:54'!
returnValue
	"Answer the value of returnValue"

	^ returnValue! !

!CSTraceReturnEntry methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:54'!
returnValue: anObject
	"Set the value of returnValue"

	returnValue := anObject! !

Object subclass: #CSTraceRegistry
	instanceVariableNames: 'traces'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'CodeSight-Tracer'!

!CSTraceRegistry class methodsFor: 'queries' stamp: 'mp 2/13/2009 11:24'!
cleanAllAnnotations

	CSTraceRegistry current traces keys do: [:key | self cleanAnnotationsFor: key].
	CSTraceRegistry current initialize.
	Smalltalk garbageCollect.! !

!CSTraceRegistry class methodsFor: 'queries' stamp: 'mp 2/13/2009 11:25'!
cleanAnnotationsFor: anIdentifier

	Smalltalk do: [:each | 
		[each methodDict 
			do: [:method | [method properties removeKey: anIdentifier] ifError: []].
		each class methodDict 
			do: [:method | [method properties removeKey: anIdentifier] ifError: []]]
		ifError: []].
	[CSTraceRegistry current traces removeKey: anIdentifier] ifError: [].! !

!CSTraceRegistry class methodsFor: 'instance creation' stamp: 'mp 2/5/2009 10:14'!
current

	UniqueInstance ifNil: [UniqueInstance := self basicNew initialize].
	^ UniqueInstance! !

!CSTraceRegistry class methodsFor: 'instance creation' stamp: 'mp 2/5/2009 10:12'!
new

	self error: 'Use current to get an instance of Class ' , self name.! !

!CSTraceRegistry methodsFor: 'tracing' stamp: 'mp 2/10/2009 10:56'!
addNewTrace: anIdentifier

	self traces 
		at: anIdentifier ifPresent: [:each | self error: 'Trace key already exist.'];
		at: anIdentifier put: (CSRegisteredTrace with: anIdentifier).
! !

!CSTraceRegistry methodsFor: 'tracing' stamp: 'mp 2/7/2009 16:39'!
at: anIdentifier putReturnValue: anObject

	(self traces at: anIdentifier) returnValue: anObject.! !

!CSTraceRegistry methodsFor: 'tracing' stamp: 'mp 2/6/2009 15:18'!
at: anIdentifier putStartEntry: aTraceEntry

	(self traces at: anIdentifier) startEntries addLast: aTraceEntry.! !

!CSTraceRegistry methodsFor: 'initialize-release' stamp: 'mp 2/5/2009 10:14'!
initialize

	self traces: IdentityDictionary new.! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 13:05'!
printCall: aCallEntry forTrace: anIdentifier on: aStream

	self printCallEntry: aCallEntry on: aStream.	
	aCallEntry 
		iterateNextMethodTrace: anIdentifier
		withNextCallBlock: [:call | 
			self printCall: call forTrace: anIdentifier on: aStream] 
		andReturnBlock: [:return | 
			self printReturnEntry: return on: aStream].! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 11:56'!
printCallEntry: aCallEntry on: aStream
			
	aStream cr.
	aCallEntry depth size timesRepeat: [aStream space].
	aStream	
		nextPutAll: aCallEntry methodClass asString;
		nextPutAll: '>>#';
		nextPutAll: aCallEntry methodSelector asString.! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 13:05'!
printReturnEntry: aReturnEntry on: aStream

	aStream cr.
	aReturnEntry depth size timesRepeat: [aStream space].
	aStream
		nextPut: $^;	
		nextPutAll: aReturnEntry returnValue asString.

	! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 11:52'!
printTrace: anIdentifier
		
	self printTrace: anIdentifier on: Transcript.
	! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 12:01'!
printTrace: anIdentifier on: aStream
		
	self 
		printTraceIdentifier: anIdentifier on: aStream;
		printTraceStartEntries: anIdentifier on: aStream;
		printTraceResult: anIdentifier on: aStream.
	aStream flush.
	! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 11:53'!
printTraceIdentifier: anIdentifier on: aStream

	aStream 
		cr;
		nextPut: $#;
		nextPutAll: anIdentifier asString.! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 18:40'!
printTraceResult: anIdentifier on: aStream

	aStream 
		cr;
		nextPut: $^; 
		nextPutAll: (self traces at: anIdentifier) returnValue returnValue asString.! !

!CSTraceRegistry methodsFor: 'printing' stamp: 'mp 2/8/2009 11:55'!
printTraceStartEntries: anIdentifier on: aStream
	
	(self traces at: anIdentifier) startEntries do: [:each | 
		each isCall 
			ifTrue: [self printCall: each forTrace: anIdentifier on: aStream]
			ifFalse: [self error: 'Should not be possible. Trace is corrupted.']].! !

!CSTraceRegistry methodsFor: 'as yet unclassified' stamp: 'mp 2/8/2009 14:14'!
removeCall: aCallEntry forTrace: anIdentifier

	aCallEntry
		iterateNextMethodTrace: anIdentifier
		withNextCallBlock: [:call | 
			self removeCall: call forTrace: anIdentifier] 
		andReturnBlock: [:return | 
			((aCallEntry getNextMethodPropertiesFor: anIdentifier) last == return)
				ifTrue: [self removeData: aCallEntry fromTrace: anIdentifier]].	! !

!CSTraceRegistry methodsFor: 'as yet unclassified' stamp: 'mp 2/8/2009 14:16'!
removeData: aCallEntry fromTrace: anIdentifier

	(aCallEntry methodClass methodDict at: aCallEntry methodSelector) 
		properties removeKey: anIdentifier.! !

!CSTraceRegistry methodsFor: 'as yet unclassified' stamp: 'mp 2/8/2009 15:24'!
removeTrace: anIdentifier

	(self traces at: anIdentifier) startEntries do: [:each | 
		each isCall 
			ifTrue: [self removeCall: each forTrace: anIdentifier]
			ifFalse: [self error: 'Should not be possible. Trace is corrupted.']].
	self traces removeKey: anIdentifier.! !

!CSTraceRegistry methodsFor: 'accessing' stamp: 'mp 2/5/2009 10:14'!
traces
	"Answer the value of traces"

	^ traces! !

!CSTraceRegistry methodsFor: 'accessing' stamp: 'mp 2/5/2009 10:14'!
traces: anObject
	"Set the value of traces"

	traces := anObject! !

Object subclass: #CSTracer
	instanceVariableNames: 'identifier traceBlock depth currentDepth printFlag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Tracer'!

!CSTracer class methodsFor: 'initialization' stamp: 'mp 2/10/2009 12:40'!
identifier: aSymbol
	
	^ self new
		identifier: aSymbol! !

!CSTracer methodsFor: 'private-adding' stamp: 'mp 2/7/2009 16:32'!
addCallFrom: aCurrentContext to: aPrevContext

	| entry |
	entry := self getCallEntry: aCurrentContext.
	aPrevContext == self traceBlock
		ifTrue: [self registry 
					at: self identifier 
					putStartEntry: entry]
		ifFalse: [self addEntry: entry to: aPrevContext].	! !

!CSTracer methodsFor: 'private-adding' stamp: 'test 9/17/2009 23:18'!
addEntry: entry to: aPrevContext

	| entries |
	entries := aPrevContext method properties 
				at: self identifier 
				ifAbsent: [OrderedCollection new].
	aPrevContext method properties 
		at: self identifier 
		put: (entries addLast: entry; yourself).! !

!CSTracer methodsFor: 'private-adding' stamp: 'mp 2/13/2009 13:39'!
addReturnFrom: aCurrentContext to: aPrevContext

	| entry |
	entry := self getReturnEntry: aCurrentContext.
	self addEntry: entry to: aPrevContext.! !

!CSTracer methodsFor: 'private-tracing' stamp: 'mp 2/7/2009 17:41'!
categoriesFromReceiver: anObject

	| receiverClass |
	receiverClass := anObject class.
	receiverClass isMeta
		ifTrue: [receiverClass := anObject].
	^ receiverClass category! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/6/2009 14:07'!
currentDepth
	"Answer the value of currentDepth"

	^ currentDepth! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/6/2009 14:07'!
currentDepth: anObject
	"Set the value of currentDepth"

	currentDepth := anObject! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/6/2009 10:06'!
depth
	"Answer the value of depth"

	^ depth! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/6/2009 10:06'!
depth: anObject
	"Set the value of depth"

	depth := anObject! !

!CSTracer methodsFor: 'private-adding' stamp: 'mp 2/7/2009 16:13'!
getCallEntry: aContext
							
	^ CSTraceCallEntry new
		depth: self depth veryDeepCopy;
		methodClass: aContext methodClass;
		methodSelector: aContext selector;
		"receiver: aContext receiver printString;"
		yourself! !

!CSTracer methodsFor: 'private-adding' stamp: 'mp 2/13/2009 18:16'!
getReturnEntry: aContext

	^ CSTraceReturnEntry new
		depth: self depth veryDeepCopy;
		returnValue: aContext top veryDeepCopy;
		methodClass: aContext methodClass;
		methodSelector: aContext selector;
		yourself! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:58'!
identifier
	"Answer the value of identifier"

	^ identifier! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/4/2009 17:58'!
identifier: anObject
	"Set the value of identifier"

	identifier := anObject! !

!CSTracer methodsFor: 'private-depth' stamp: 'mp 2/6/2009 14:16'!
increaseDepth
		
	| entry |
	entry := self depth at: self currentDepth ifAbsent: nil.
	entry
		ifNil: [self depth at: self currentDepth ifAbsentPut: 1]
		ifNotNil: [self depth at: self currentDepth put: entry + 1].! !

!CSTracer methodsFor: 'initialize-release' stamp: 'mp 2/11/2009 14:43'!
initialize
	
	self 
		identifier: DateAndTime now asString asSymbol;
		depth: OrderedCollection new;
		printFlag: false.! !

!CSTracer methodsFor: 'private-printing' stamp: 'mp 2/11/2009 14:44'!
printCallFrom: aContext

	self printFlag 
		ifTrue:[
			Transcript cr.
			self currentDepth timesRepeat: [Transcript space].
			Transcript 
				nextPutAll: aContext receiver printString;
				space; 
				nextPutAll: aContext selector; 
				space; 
				nextPutAll: self depth asString; 
				flush].! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/11/2009 14:43'!
printFlag
	"Answer the value of printFlag"

	^ printFlag! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/11/2009 14:43'!
printFlag: anObject
	"Set the value of printFlag"

	printFlag := anObject! !

!CSTracer methodsFor: 'private-printing' stamp: 'mp 2/11/2009 14:45'!
printReturnFrom: aContext

	self printFlag 
		ifTrue:[
			Transcript cr.
			self currentDepth timesRepeat: [Transcript space].
			Transcript 
				space; 
				nextPut: $^;
				nextPutAll: aContext top printString;
				space; 
				nextPutAll: self depth asString;
				flush].! !

!CSTracer methodsFor: 'private-depth' stamp: 'mp 2/6/2009 14:15'!
reduceDepth

	self depth size = (self currentDepth + 2)
		ifTrue: [self depth removeLast.].! !

!CSTracer methodsFor: 'private-adding' stamp: 'mp 2/10/2009 10:56'!
registerTrace

	self registry addNewTrace: self identifier.! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/5/2009 11:09'!
registry
	
	^ CSTraceRegistry current! !

!CSTracer methodsFor: 'tracing' stamp: 'mp 2/11/2009 14:49'!
trace: aBlock

	| previousContext result |
	self 
		registerTrace;
		traceBlock: aBlock.
	previousContext := self traceBlock.
	result := thisContext sender
		runSimulated: self traceBlock
		contextAtEachStep:
			[:currentContext |
			"Sensor anyButtonPressed ifTrue: [^ nil]."
			currentContext == previousContext
				ifFalse:
					[previousContext := self traceWith: currentContext and: previousContext]].
		self registry 
			at: self identifier 
			putReturnValue: (CSTraceReturnEntry new
								returnValue: result).
		^ result! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/5/2009 11:28'!
traceBlock
	"Answer the value of traceBlock"

	^ traceBlock! !

!CSTracer methodsFor: 'accessing' stamp: 'mp 2/5/2009 11:28'!
traceBlock: anObject
	"Set the value of traceBlock"

	traceBlock := anObject! !

!CSTracer methodsFor: 'private-tracing' stamp: 'mp 2/7/2009 18:12'!
traceMethodCallWith: aCurrentContext and: aPreviousContext
	
	self 
		increaseDepth;
		addCallFrom: aCurrentContext to: aPreviousContext;
		printCallFrom: aCurrentContext.! !

!CSTracer methodsFor: 'private-tracing' stamp: 'mp 2/7/2009 18:12'!
traceReturnWith: aCurrentContext and: aPreviousContext
	
	self 
		reduceDepth;
		addReturnFrom: aCurrentContext to: aPreviousContext;
		printReturnFrom: aCurrentContext.! !

!CSTracer methodsFor: 'private-tracing' stamp: 'mp 2/10/2009 10:48'!
traceWith: aCurrentContext and: aPreviousContext

	self currentDepth: (aCurrentContext depthBelow: self traceBlock).
	aPreviousContext sender
		ifNil: [self traceReturnWith: aCurrentContext and: aPreviousContext]
		ifNotNil: [self traceMethodCallWith: aCurrentContext and: aPreviousContext].
	^ aCurrentContext! !

Object subclass: #CodeSightSarBuilder
	instanceVariableNames: 'zip'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-Install'!

!CodeSightSarBuilder class methodsFor: 'service' stamp: 'rkrk 11/27/2009 15:14'!
create

	"self create"
	^ self new create! !

!CodeSightSarBuilder methodsFor: 'sar creation' stamp: 'rkrk 11/27/2009 15:17'!
addFiles

	{self downloadLatestCodeSightVersion.
	self downloadMethodWrapperMcz} do: [:path | 
		zip addFile: path as: (path subStrings: '/\') last].! !

!CodeSightSarBuilder methodsFor: 'postscript' stamp: 'rkrk 11/27/2009 13:28'!
addPostScript

	zip addString: self postscript as: 'install/postscript'.! !

!CodeSightSarBuilder methodsFor: 'preamble' stamp: 'rkrk 11/27/2009 13:28'!
addPreamble

	zip addString: self preamble as: 'install/preamble'.! !

!CodeSightSarBuilder methodsFor: 'sar creation' stamp: 'rkrk 11/27/2009 15:12'!
create

	self
		createZip;
		addPreamble;
		addPostScript;
		addFiles;
		writeAndCloseZip.! !

!CodeSightSarBuilder methodsFor: 'sar creation' stamp: 'rkrk 11/27/2009 13:30'!
createZip

	zip := ZipArchive new.! !

!CodeSightSarBuilder methodsFor: 'download' stamp: 'rkrk 11/27/2009 15:49'!
downloadLatestCodeSightVersion

	^ self fromCodeSightRepoDownload: 'CodeSight'.! !

!CodeSightSarBuilder methodsFor: 'download' stamp: 'rkrk 11/27/2009 15:45'!
downloadMethodWrapperFromSqueakMap

	| squeakMap package |
	squeakMap := SMSqueakMap default.
	squeakMap objects ifNil: [
		squeakMap inspect.
		self error: 'There is a problem with the SqueakMap instance, inst var objects should be a dictionary but is nil. Please fix it and restart the build process!!'].
	package := squeakMap packageWithNameBeginning: 'MethodWrappers'.
	^ (SMInstaller forPackageRelease: package lastRelease)
		download;
		fullFileName.! !

!CodeSightSarBuilder methodsFor: 'download' stamp: 'rkrk 11/27/2009 15:59'!
downloadMethodWrapperMcz

	^ self fromCodeSightRepoDownload: 'MethodWrappers'.! !

!CodeSightSarBuilder methodsFor: 'default' stamp: 'rkrk 11/27/2009 15:14'!
filename

	^ 'CodeSight-' , Date today yyyymmdd , '.sar'! !

!CodeSightSarBuilder methodsFor: 'download' stamp: 'rkrk 11/27/2009 15:49'!
fromCodeSightRepoDownload: packageNamed

	| repo versionName filename mimeDoc |
	repo := MCHttpRepository
		location: 'http://www.hpi.uni-potsdam.de/hirschfeld/squeaksource/CodeSight'
		user: ''
		password: ''.
	versionName := repo allVersionNames detect: [:ea | ea startsWith: packageNamed]. "assume latest first. Seems to work..."
	filename := versionName , '.mcz'.
	mimeDoc := (Url absoluteFromText:  (repo urlForFileNamed: filename)) retrieveContents.
	^ FileStream
		forceNewFileNamed: filename
		do: [:s | s binary. s nextPutAll: mimeDoc content. s fullName].
	"version := repo versionFromFileNamed: filename. 
	version load."! !

!CodeSightSarBuilder methodsFor: 'postscript' stamp: 'rkrk 11/27/2009 15:10'!
postscript

	^ 'CSCoverageTracingWrapper initialize'! !

!CodeSightSarBuilder methodsFor: 'preamble' stamp: 'rkrk 11/27/2009 15:01'!
preamble

	^ String streamContents: [:s |
		s
			nextPutAll: self preambleMethodWrapperInstall; cr;
			nextPutAll: self preambleCodeSightInstall; cr].! !

!CodeSightSarBuilder methodsFor: 'preamble' stamp: 'rkrk 11/27/2009 15:04'!
preambleCodeSightInstall

	^ '(self membersMatching: ''CodeSight-*.mcz'') do: [:ea | self installMember: ea].'! !

!CodeSightSarBuilder methodsFor: 'preamble' stamp: 'rkrk 11/27/2009 15:59'!
preambleMethodWrapperInstall

	^ '(self membersMatching: ''MethodWrappers-*.mcz'') do: [:ea | self installMember: ea].'! !

!CodeSightSarBuilder methodsFor: 'sar creation' stamp: 'rkrk 11/27/2009 15:15'!
writeAndCloseZip

	zip
		writeToFileNamed: self filename;
		close.! !

!Object methodsFor: '*CodeSight' stamp: 'bs 4/28/2009 18:20'!
debugAndUninstallBreakpointBefore
	(Debugger context: thisContext)
		debug: 'Debugger' executeBefore: [:debugger | 
			debugger down.
			BreakpointManager unInstall: debugger selectedContext method.
			debugger restartAfterMethodChanged.].
	Processor activeProcess suspend.! !

TestRunner subclass: #CSTestRunner
	instanceVariableNames: 'isInTddAutoRunMode testCasesToRun runsTestsInBackground doForEachCoveredMethod methodsCoveredBeforeRun modifiedMethods'
	classVariableNames: 'CurrentAutoTestRunner RunInBackgroundMonitor WaitingTestRunners'
	poolDictionaries: ''
	category: 'CodeSight-TestTracer'!

CSTestRunner class
	instanceVariableNames: 'classesToBeTraced'!

CSTestRunner class
	instanceVariableNames: 'classesToBeTraced'!

!CSTestRunner class methodsFor: 'accessing' stamp: 'rkrk 9/22/2009 05:35'!
currentAutoTestRunner

	^ CurrentAutoTestRunner ifNil: [
		self new
			tddAutoRunStart;
			yourself].! !

!CSTestRunner class methodsFor: 'as yet unclassified' stamp: 'bs 4/16/2009 15:28'!
initialize
	"CSTestRunner initialize"
	self registerInWorldMenu; registerInToolsFlap.! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'bs 10/28/2009 14:19'!
measureExecutionTimesForPackagesWith: aString
	"CSTestRunner measureExecutionTimes"
	|packages classes testCase time testClasses testCaseTimes result coveredMethods coveringTestMethods coveredMethodsTestTimeDeltas classTestTime aTestRunner aSuite |
	packages := PackageOrganizer default packages select: [:p | p packageName startsWith: aString].
	packages do: [:p | CSInstrumentor systemInstrumentor instrumentPackage: p].
	classes := (packages gather: [:p | p classes]) asSet.

	aTestRunner := CSTestRunner new.
	testCase := nil.
	time := nil.

	testClasses := classes select: [:class | class inheritsFrom: TestCase].
	testCaseTimes := Dictionary new.
	testClasses do: [:testClass | 
		testClass selectors select: [:s | s isTestSelector] thenDo: [:s | 
			aSuite := TestSuite new.
			aSuite addTest: (testClass selector: s).
			time := [aTestRunner runSuite: aSuite] timeToRun.
			testCaseTimes at: (testClass >> s) put: time]].

	result := Dictionary new.	
	coveredMethods := nil.
	coveredMethodsTestTimeDeltas := nil.
	classes select: [:c | c methodDict values contains: [:m | m hasCoveringTestMethods]]
	 		  thenDo: [:aClass |
		coveredMethods := aClass methodDict values select: [:m | m hasCoveringTestMethods ].
		coveringTestMethods := (coveredMethods gather: [:m | m coveringTestMethods]) asSet.
		classTestTime := (coveringTestMethods collect: [:m | testCaseTimes at: m ifAbsent: [0]]) sum.
		
		coveredMethodsTestTimeDeltas := classTestTime - (coveredMethods collect: [:cm | 
			(cm coveringTestMethods collect: [:tm | testCaseTimes at: tm ifAbsent: [0]]) sum]).
		
		coveredMethodsTestTimeDeltas ifNotEmpty: [
			result at: aClass put: {coveredMethods size. coveredMethodsTestTimeDeltas max. coveredMethodsTestTimeDeltas average}]].
	
	packages do: [:p | CSInstrumentor systemInstrumentor uninstrumentPackage: p].
	^ result
! !

!CSTestRunner class methodsFor: 'as yet unclassified' stamp: 'bs 4/22/2009 15:14'!
registerInToolsFlap
	self environment at: #Flaps ifPresent: [ :class |
		class
			registerQuad: #( CSTestRunner build 'CodeSight Test Runner' 'A production scale test-runner for XP Forums.' ) forFlapNamed: 'Tools';
			replaceToolsFlap ].! !

!CSTestRunner class methodsFor: 'as yet unclassified' stamp: 'mab 4/2/2009 20:33'!
registerInWorldMenu
	self environment at: #TheWorldMenu ifPresent: [ :class |
		class registerOpenCommand: (Array 
			with: 'CodeSight Test Runner' 
			with: (Array
				with: self
				with: #open)) ].! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'test 9/17/2009 23:32'!
resetSourceCodeAccessMonitor
	^ RunInBackgroundMonitor := nil! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'bs 10/28/2009 09:48'!
resetWaitingAndCurrentTestRunners
	"self resetWaitingAndCurrentTestRunners"
	self waitingTestRunners: nil.
	CurrentAutoTestRunner := nil.! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'bs 9/14/2009 10:50'!
sourceCodeAccessMonitor
	^ RunInBackgroundMonitor ifNil: [ RunInBackgroundMonitor := Monitor new ]! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'bs 9/13/2009 14:53'!
tddAutoRunStartFor: aCSTestRunner
	CurrentAutoTestRunner
		ifNotNil: [
			CurrentAutoTestRunner disableTddAutoRunMode.
			self waitingTestRunners add: CurrentAutoTestRunner].
		
	CurrentAutoTestRunner := aCSTestRunner enableTddAutoRunMode.! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'bs 9/13/2009 14:55'!
tddAutoRunStopFor: aCSTestRunner
	aCSTestRunner disableTddAutoRunMode.
	(CurrentAutoTestRunner == aCSTestRunner)
		ifTrue: [
			CurrentAutoTestRunner := nil.
			self waitingTestRunners ifNotEmpty: [
				CurrentAutoTestRunner := self waitingTestRunners removeLast.
				CurrentAutoTestRunner enableTddAutoRunMode].
			]
		ifFalse: [
			self waitingTestRunners remove: aCSTestRunner ifAbsent: []].! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'bs 10/28/2009 09:48'!
waitingTestRunners
	^ WaitingTestRunners ifNil: [WaitingTestRunners := OrderedCollection new.]! !

!CSTestRunner class methodsFor: 'synchronizing' stamp: 'bs 9/23/2009 17:32'!
waitingTestRunners: anObject
	^ WaitingTestRunners := anObject! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 11/27/2009 14:29'!
affectedTestMethodsForAddedMethod: aMethod
	^ ((aMethod overriddenAndOverridingMethods)
			gather: [ :supOrSubMethod | supOrSubMethod coveringTestMethods ]) 
				asSet collect: [:aTestMethod | aTestMethod newTestCase]! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 10/28/2009 16:48'!
basicSetUpSuite: aTestSuite 
	| testMethod |
	super basicSetUpSuite: aTestSuite.
	
	aTestSuite tests do: [ :aTestCase | 
		testMethod := aTestCase class >> aTestCase selector.
		self methodsCoveredBeforeRun at: testMethod put: testMethod coveredMethods.
		testMethod resetCoveredMethodsStatistics ].

	CSInstrumentor systemInstrumentor activateTracing.! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 10/28/2009 16:48'!
basicTearDownSuite: aTestSuite
	CSInstrumentor systemInstrumentor deactivateTracing.
	
	self methodsCoveredBeforeRun keysAndValuesDo: [:testMethod :coveredMethods |
		coveredMethods do: [:coveredMethod |
			coveredMethod removeFixedTestMethodIfNotCoveringAnymore: testMethod ]]. 
	
	self resetMethodsCoveredBeforeRun.
	super basicTearDownSuite: aTestSuite.
	! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 9/15/2009 12:02'!
buildControlPanelWith: aBuilder
	^ aBuilder pluggablePanelSpec new
		layout: #vertical;
		children: {
			self buildButtonsWith: aBuilder.
			self buildTddControlButtonsWith: aBuilder
			};
		yourself.! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 9/15/2009 12:18'!
buildTddControlButtonsWith: aBuilder
	^ aBuilder pluggablePanelSpec new
		model: self;
		layout: #horizontal;
		children: (self tddFlags collect: [ :each |
			aBuilder pluggableActionButtonSpec new
				model: self; 
				label: each first;
				action: each second;
				yourself ]);
		yourself.! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 12/18/2009 16:26'!
buildWith: aBuilder
	| window |
	window := aBuilder pluggableWindowSpec new
		model: self; label: self label; extent: self extent;
		children: (OrderedCollection new 
			add: ((self buildCategoriesWith: aBuilder)
				frame: (0.00 @ 0.20 corner: 0.25 @ 0.84 );
				yourself);				
			add: ((self buildClassesWith: aBuilder)
				frame: (0.25 @ 0.00 corner: 0.50 @ 0.84 );
				yourself);
			add: ((self buildStatusWith: aBuilder)
				frame: (0.50 @ 0.00 corner: 1.00 @ 0.08);
				yourself);
			add: ((self buildFailureListWith: aBuilder)
				frame: (0.50 @ 0.08 corner: 1.00 @ 0.50);
				yourself);
			add: ((self buildErrorListWith: aBuilder)
				frame: (0.50 @ 0.50 corner: 1.00 @ 0.84);
				yourself);
			add: ((self buildControlPanelWith: aBuilder)
				frame: (0.00 @ 0.84 corner: 1.00 @ 1.00);
				yourself);
			yourself);
		closeAction: #close;
		yourself.
	^ aBuilder build: window.! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 9/11/2009 18:04'!
classAdded: anAddedEvent
	! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 12/11/2009 11:05'!
classModified: aModifiedEvent
	| modifiedClass collectingSelector |
	modifiedClass := aModifiedEvent oldItem.
	collectingSelector := (modifiedClass inheritsFrom: TestCase) 
		ifTrue: [#collectTestCasesOfTestClass: ]
		ifFalse: [#collectTestCasesCoveringClass:].
	
	modifiedClass withAllSubclassesDo: [:subclass | 
		self testCasesToRun addAll: 
			(self perform: collectingSelector with: subclass)].
			
	self runOpenTestCasesInSuite.! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 12/22/2009 15:35'!
classRemoved: aRemovedEvent
	| aClass |
	aClass := aRemovedEvent item. 
	self handleRemovedMethods: (aClass methodDict values).
	self runOpenTestCasesInSuite.! !

!CSTestRunner methodsFor: 'building' stamp: 'rkrk 9/22/2009 05:39'!
close

	self
		disableRunningTestsInBackground;
		tddAutoRunStop;
		changed: #runnerClosed.! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 9/11/2009 11:18'!
collectTestCasesCoveringClass: aClass 
	^ (aClass methodDict values gather: [ :method | self collectTestCasesCoveringMethod: method ]) asSet! !

!CSTestRunner methodsFor: 'event handling' stamp: 'test 9/20/2009 14:25'!
collectTestCasesCoveringMethod: aMethod 
	^ aMethod coveringTestMethods collect: [ :aTestMethod | aTestMethod newTestCase ]! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 9/15/2009 08:58'!
collectTestCasesOfTestClass: aTestClass 
	^ aTestClass selectors 
			select: [ :aSelector | aSelector isTestSelector ]
			thenCollect: [ :aSelector | aTestClass selector: aSelector ].! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 4/30/2009 12:09'!
debugSuite: aTestSuite
	self basicRunSuite: aTestSuite do: [ :each | self debugTest: each ].! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 10/28/2009 16:05'!
debugTest: aTestCase
	| outerTest | 
	outerTest := CSInstrumentor systemInstrumentor runningTestMethod.
	CSInstrumentor systemInstrumentor testMethodGoingToRun: (aTestCase class >> aTestCase selector) unwrappedMethod.
	aTestCase debug.
	CSInstrumentor systemInstrumentor testMethodGoingToRun: outerTest.! !

!CSTestRunner methodsFor: 'background processing' stamp: 'bs 9/14/2009 16:16'!
disableRunningTestsInBackground
	^ runsTestsInBackground := false! !

!CSTestRunner methodsFor: 'auto run mode' stamp: 'bs 10/28/2009 14:27'!
disableTddAutoRunMode
	self isInTddAutoRunMode: false.
	CSInstrumentor systemInstrumentor systemEventPostProcessingHandlers remove: self ifAbsent: [].! !

!CSTestRunner methodsFor: 'running' stamp: 'test 9/18/2009 21:01'!
doForEachCoveredMethod
	^ doForEachCoveredMethod ifNil: [ doForEachCoveredMethod := [ :aTestMethod :anAppMethod | ] ]! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 10/7/2009 09:38'!
doForEachCoveredMethod: aBlock
	^ doForEachCoveredMethod := aBlock.! !

!CSTestRunner methodsFor: 'background processing' stamp: 'bs 9/14/2009 16:16'!
enableRunningTestsInBackground
	^ runsTestsInBackground := true! !

!CSTestRunner methodsFor: 'auto run mode' stamp: 'bs 10/28/2009 14:27'!
enableTddAutoRunMode
	self isInTddAutoRunMode: true.
	CSInstrumentor systemInstrumentor systemEventPostProcessingHandlers add: self! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 12/22/2009 13:35'!
failedTestCases
	^ self result failures copy addAll: self result errors! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 11/30/2009 21:37'!
failedTests.
	^ (self result failures copy addAll: self result errors; yourself)
			collect: [ :aTestCase | aTestCase class >> aTestCase selector ]! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 12/22/2009 15:32'!
handleRemovedMethods: methods
	| subOrSupMethods fixedTestMethods removedMethod |

	subOrSupMethods := IdentityDictionary new.
	fixedTestMethods := IdentityDictionary new.

	methods collect: [:m | m clientMethod] thenDo: [ :method | 
		(method overriddenAndOverridingMethods) do: [:subOrSupMethod | 
			subOrSupMethods at: subOrSupMethod put: method].
		fixedTestMethods at: method put: method fixedTestMethods.
		
		self testCasesToRun addAll: (self collectTestCasesCoveringMethod: method).
		method isTestMethod
			ifTrue: [
				method resetCoveredMethodsStatistics.
				self result removeIfPresent: method]
			ifFalse: [
				method 
					resetCoveringTestMethods;
					resetFixedTestMethods ] ].
			
	self doForEachCoveredMethod: [:aTestMethod :anAppMethod |
		(subOrSupMethods includesKey: anAppMethod) 
			ifTrue: [
				removedMethod := subOrSupMethods at: anAppMethod.
				anAppMethod fixesAll: (fixedTestMethods at: removedMethod)]].! !

!CSTestRunner methodsFor: 'auto run mode' stamp: 'bs 9/11/2009 14:52'!
isInTddAutoRunMode
	^ isInTddAutoRunMode ifNil: [isInTddAutoRunMode := false].! !

!CSTestRunner methodsFor: 'auto run mode' stamp: 'bs 9/11/2009 14:51'!
isInTddAutoRunMode: aBoolean
	^ isInTddAutoRunMode := aBoolean.! !

!CSTestRunner methodsFor: 'building' stamp: 'mab 4/2/2009 20:33'!
label
	^ 'CodeSight Test Runner'! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 12/21/2009 18:49'!
methodAdded: anAddedEvent 
	| method affectedTests |
	method := anAddedEvent item.
	
	affectedTests := (method isTestMethod
		ifTrue: [ { method. } ]
		ifFalse: [ method testsAffectedByAddition ]) asIdentitySet.

	self runTestsInSuite: (affectedTests union: self failedTests)! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 12/21/2009 18:50'!
methodModified: aModifiedEvent 
	| method oldMethod affectedTests |
	method := aModifiedEvent item.
	oldMethod := aModifiedEvent oldItem.
	method fixesAll: (aModifiedEvent oldItem fixedTestMethods).
	
	affectedTests := (method isTestMethod
		ifTrue: [ { method. } ]
		ifFalse: [ oldMethod testsAffectedByModification ]) asIdentitySet.

	self runTestsInSuite: (affectedTests union: self failedTests)! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 12/22/2009 15:34'!
methodRemoved: aRemovedEvent
	| method |
	method := aRemovedEvent item.
	self handleRemovedMethods: {method}.
	self runOpenTestCasesInSuite.! !

!CSTestRunner methodsFor: 'running' stamp: 'test 9/21/2009 10:45'!
methodsCoveredBeforeRun
	^ methodsCoveredBeforeRun ifNil: [methodsCoveredBeforeRun := IdentityDictionary new]! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 11/27/2009 14:06'!
nonTestMethodAdded: aMethod
	self testCasesToRun addAll:
		(((aMethod overriddenAndOverridingMethods)
			gather: [ :supOrSubMethod | supOrSubMethod coveringTestMethods ]) 
				asSet collect: [:aTestMethod | aTestMethod newTestCase]).
			
	self doForEachCoveredMethod: [:aTestMethod :anAppMethod |
		(anAppMethod = aMethod) ifTrue: [
			anAppMethod fixesAll: 
				((anAppMethod overriddenAndOverridingMethods) 
					gather: [:supOrSubMethod | supOrSubMethod fixedTestMethods])]].! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 11/27/2009 14:17'!
nonTestMethodModified: aMethod previousVersion: oldMethod
	| coveredMethods |
	self testCasesToRun addAll: 
		(oldMethod coveringTestMethods 
			collect: [:testMethod | testMethod newTestCase]).
			
	coveredMethods := IdentityDictionary new.
	((oldMethod) coveringTestMethods) do: [:testMethod |
		coveredMethods at: testMethod put: testMethod coveredMethods ].
			
	self doForEachCoveredMethod: [:aTestMethod :anAppMethod | 
		((coveredMethods at: aTestMethod) includes: anAppMethod) ifFalse: [
			anAppMethod fixesAll: aMethod fixedTestMethods ] ]! !

!CSTestRunner methodsFor: 'running' stamp: 'test 9/21/2009 10:43'!
resetMethodsCoveredBeforeRun
	methodsCoveredBeforeRun := nil.! !

!CSTestRunner methodsFor: 'accessing' stamp: 'bs 9/10/2009 13:58'!
resetTestCasesToRun
	^ testCasesToRun := nil.! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 12/3/2009 22:38'!
runFailedTests
	self runTestMethods: self failedTests.! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 10/7/2009 09:44'!
runOpenTestCasesInSuite
	| aSuite testMethod doForEachCoveredMethodBlock testMethodsToBeReverted |
	
	aSuite := TestSuite new.
	testMethodsToBeReverted := Set new.
	
	doForEachCoveredMethodBlock := self doForEachCoveredMethod.
	self doForEachCoveredMethod: nil.
	
	self failedTestCases do: [ :aTestCase |
		aSuite addTest: aTestCase.
		testMethod := aTestCase class >> aTestCase selector.
		testMethodsToBeReverted add: testMethod.
		testMethod doForEachCoveredMethod: [:aTestMethod :anAppMethod | 
			anAppMethod fixes: aTestMethod.
			doForEachCoveredMethodBlock valueWithArguments: {aTestMethod. anAppMethod}]].
	
	
	self testCasesToRun do: [ :aTestCase |
		aSuite addTest: aTestCase.
		testMethod := aTestCase class >> aTestCase selector.
		testMethodsToBeReverted add: testMethod.
		testMethod doForEachCoveredMethod: [:aTestMethod :anAppMethod | 
			doForEachCoveredMethodBlock valueWithArguments: {aTestMethod. anAppMethod}]].
	
	self resetTestCasesToRun.
	
	aSuite tests ifNotEmpty: [
		self reset; runSuite: aSuite].
	
	testMethodsToBeReverted do: [:method | method resetDoForEachCoveredMethod ].! !

!CSTestRunner methodsFor: 'running' stamp: 'rkrk 9/30/2009 05:05'!
runSuite: aSuite 
	| runSuiteSynchronized |
	runSuiteSynchronized := [	
		CSTestRunner sourceCodeAccessMonitor critical: [
			self changed: #startTestRun.
			super runSuite: aSuite.
			self changed: #endTestRun]].

	(self runsTestsInBackground and: [
			ProcessBrowser isUIProcess: Processor activeProcess])
		ifTrue: [ runSuiteSynchronized forkAt: Processor userBackgroundPriority ]
		ifFalse: [ runSuiteSynchronized value ]! !

!CSTestRunner methodsFor: 'running' stamp: 'test 10/28/2009 22:04'!
runTest: aTestCase
	| outerTest | 
	outerTest := CSInstrumentor systemInstrumentor runningTestMethod.
	CSInstrumentor systemInstrumentor testMethodGoingToRun: (aTestCase class >> aTestCase selector) unwrappedMethod.
	super runTest: aTestCase.
	CSInstrumentor systemInstrumentor testMethodGoingToRun: outerTest.
	
	self changed: #newTestResult with: result.! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 12/3/2009 22:56'!
runTestMethods: testMethods
	| aSuite |
	
	testMethods ifNotEmpty: [
		aSuite := TestSuite new.
		aSuite addTests: (testMethods collect: [:ea | ea newTestCase]).
		self runSuite: aSuite ].! !

!CSTestRunner methodsFor: 'running' stamp: 'bs 12/3/2009 22:32'!
runTestsInSuite: testMethods
	| aSuite |
	
	testMethods ifNotEmpty: [
		aSuite := TestSuite new.
		aSuite addTests: (testMethods collect: [:ea | ea newTestCase]).
		self reset; runSuite: aSuite ].! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 9/15/2009 12:11'!
runningTestsInBackgroundLabel
	^ self runsTestsInBackground
		ifTrue: [ 'DisableRunningTestsInBackground' ]
		ifFalse: [ 'EnableRunningTestsInBackground' ]! !

!CSTestRunner methodsFor: 'background processing' stamp: 'bs 9/14/2009 16:17'!
runsTestsInBackground
	^ runsTestsInBackground ifNil: [runsTestsInBackground := false]! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 9/15/2009 13:06'!
tddAutoRunModeLabel
	^ self isInTddAutoRunMode 
		ifTrue: [ 'Stop Tdd Auto Run Mode' ]
		ifFalse: [ 'Start Tdd Auto Run Mode' ]! !

!CSTestRunner methodsFor: 'auto run mode' stamp: 'bs 9/13/2009 14:47'!
tddAutoRunStart
	(self isInTddAutoRunMode)
		ifFalse: [
			self class tddAutoRunStartFor: self]! !

!CSTestRunner methodsFor: 'auto run mode' stamp: 'bs 9/13/2009 14:47'!
tddAutoRunStop
	self class tddAutoRunStopFor: self! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 9/15/2009 12:03'!
tddFlags
	^ #(( #tddAutoRunModeLabel #toggleTddAutoRunMode )
		( #runningTestsInBackgroundLabel #toggleRunningTestsInBackground )).! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 12/22/2009 13:25'!
tddMonitor
	^ CSTddMonitor instance! !

!CSTestRunner methodsFor: 'accessing' stamp: 'bs 9/10/2009 13:56'!
testCasesToRun
	^ testCasesToRun ifNil: [testCasesToRun := OrderedCollection new].! !

!CSTestRunner methodsFor: 'event handling' stamp: 'bs 11/27/2009 14:03'!
testMethodAddedOrModified: aMethod
	| aTestClass |
	aTestClass := aMethod methodClass.
	self testCasesToRun add: (aTestClass selector: aMethod selector)! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 9/15/2009 12:21'!
toggleRunningTestsInBackground
	self runsTestsInBackground
		ifTrue: [ self disableRunningTestsInBackground. self changed: #runningTestsInBackgroundLabel. ]
		ifFalse: [ self enableRunningTestsInBackground. self changed: #runningTestsInBackgroundLabel. ]! !

!CSTestRunner methodsFor: 'building' stamp: 'bs 9/15/2009 13:07'!
toggleTddAutoRunMode
	self isInTddAutoRunMode 
		ifTrue: [ self tddAutoRunStop. self changed: #tddAutoRunModeLabel. ]
		ifFalse: [ self tddAutoRunStart. self changed: #tddAutoRunModeLabel. ]! !

MwMethodWrapper variableByteSubclass: #CSCoverageTracingWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestTracer'!

!CSCoverageTracingWrapper class methodsFor: 'wrapping' stamp: 'bs 10/28/2009 16:46'!
canWrap: aSelector inClass: aClass
	
	^ ((self canWrapClass: aClass) and: [self canWrapMethod: aSelector inClass: aClass])
			ifFalse: [false]
			ifTrue: [super canWrap: aSelector inClass: aClass]! !

!CSCoverageTracingWrapper class methodsFor: 'wrapping' stamp: 'bs 11/26/2009 15:54'!
canWrapClass: aClass
	| classesNotToBeWrapped |
	classesNotToBeWrapped := {CSTestStatistic. CSInstrumentor. CSInstrumentor class.}.
	^ (classesNotToBeWrapped includes: aClass) not.! !

!CSCoverageTracingWrapper class methodsFor: 'wrapping' stamp: 'bs 10/6/2009 09:53'!
canWrapMethod: aSelector inClass: aClass
	^ (self methodsNotToBeWrapped includesKey: aClass)
			ifFalse: [true]
			ifTrue: [
				((self methodsNotToBeWrapped at: aClass) includes: aSelector) not]! !

!CSCoverageTracingWrapper class methodsFor: 'as yet unclassified' stamp: 'bs 11/27/2009 16:50'!
initialize
	super initialize.! !

!CSCoverageTracingWrapper class methodsFor: 'wrapping' stamp: 'bs 10/28/2009 16:49'!
methodsNotToBeWrapped
	| result |
	result := Dictionary new.
	result 
		at: (CSInstrumentor) 
		put: (#(#isTracingActivated #runningTestMethod)).
	result 
		at: (CSInstrumentor class) 
		put: (#(#systemInstrumentor)).
		
	^ result! !

!CSCoverageTracingWrapper methodsFor: 'evaluating' stamp: 'bs 10/28/2009 16:05'!
beforeMethod

	self flag: #rkrk. "for benchmarking"
	self ifActive: [
		CSInstrumentor systemInstrumentor runningTestMethod called: self clientMethod ]! !

!CSCoverageTracingWrapper methodsFor: 'evaluating' stamp: 'bs 11/26/2009 14:35'!
beforeMethodWithReceiver: anObject.
	self ifActive: [
		CSInstrumentor systemInstrumentor runningTestMethod 
			ifNotNilDo: [:testMethod | 
				testMethod called: self clientMethod
						receiver: anObject ] ].! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 10/28/2009 15:30'!
called: aCompiledMethod
	self flag: #rkrk. "for benchmarking"
	^ self clientMethod called: aCompiledMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 10/28/2009 15:32'!
called: aCompiledMethod receiver: anObject
	^ self clientMethod called: aCompiledMethod receiver: anObject! !

!CSCoverageTracingWrapper methodsFor: 'evaluating' stamp: 'bs 9/9/2009 15:22'!
compiledMethod
	^ self clientMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/9/2009 16:49'!
coveredMethods
	^ self clientMethod coveredMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/9/2009 16:48'!
coveredMethodsStatistics
	^ self clientMethod coveredMethodsStatistics! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/22/2009 16:50'!
coveringAndNotFixedTestMethods
	^ self clientMethod coveringAndNotFixedTestMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/9/2009 16:28'!
coveringTestMethods
	^ self clientMethod coveringTestMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/21/2009 16:20'!
coveringTestMethodsWhenReceiverWasOfKind: aClass
	^ self clientMethod coveringTestMethodsWhenReceiverWasOfKind: aClass! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/18/2009 20:28'!
doForEachCoveredMethod
	^ self clientMethod doForEachCoveredMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/18/2009 20:29'!
doForEachCoveredMethod: aBlock
	^ self clientMethod doForEachCoveredMethod: aBlock! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/16/2009 21:18'!
fixedTestMethods
	^ self clientMethod fixedTestMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 10/6/2009 12:09'!
fixes: aTestMethod
	^ self clientMethod fixes: aTestMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 10/6/2009 12:09'!
fixesAll: testMethods
	^ self clientMethod fixesAll: testMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/4/2009 19:46'!
hasCoveringTestMethods
	^ self clientMethod hasCoveringTestMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/18/2009 08:35'!
hasFixedTestMethods
	^ self clientMethod hasFixedTestMethods! !

!CSCoverageTracingWrapper methodsFor: 'evaluating' stamp: 'bs 9/8/2009 18:36'!
ifActive: aBlock
	self isActive ifTrue: [aBlock value]! !

!CSCoverageTracingWrapper methodsFor: 'evaluating' stamp: 'bs 10/28/2009 16:25'!
isActive
	^ CSInstrumentor systemInstrumentor isTracingActivated
	"^ CSCoverageTracingWrapper isTracingActivated "! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/20/2009 14:24'!
newTestCase
	^ self clientMethod newTestCase! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/20/2009 14:44'!
overriddenAndOverridingMethods
	^ self clientMethod overriddenAndOverridingMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/20/2009 21:28'!
overriddenAndOverridingMethodsIn: aClass
	^ self clientMethod overriddenAndOverridingMethodsIn: aClass! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/20/2009 14:44'!
overriddenMethod
	^ self clientMethod overriddenMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/20/2009 21:28'!
overriddenMethodIn: aClass
	^ self clientMethod overriddenMethodIn: aClass! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/20/2009 14:44'!
overridingMethods
	^ self clientMethod overridingMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/20/2009 21:29'!
overridingMethodsIn: aClass
	^ self clientMethod overridingMethodsIn: aClass! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/11/2009 22:41'!
removeCoveredMethod: aCompiledAppMethod
	^ self clientMethod removeCoveredMethod: aCompiledAppMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/23/2009 18:43'!
removeCoveringTestMethod: aTestCaseMethod
	^ self clientMethod removeCoveringTestMethod: aTestCaseMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/21/2009 10:54'!
removeFixedTestMethodIfNotCoveringAnymore: aTestCaseMethod
	^ self clientMethod removeFixedTestMethodIfNotCoveringAnymore: aTestCaseMethod.! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/23/2009 16:06'!
resetAllCoverageData
	^ self clientMethod resetAllCoverageData! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/9/2009 16:45'!
resetCoveredMethodsStatistics
	^ self clientMethod resetCoveredMethodsStatistics! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/11/2009 22:42'!
resetCoveringTestMethods
	^ self clientMethod resetCoveringTestMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/23/2009 21:17'!
resetDoForEachCoveredMethod
	^ self clientMethod resetDoForEachCoveredMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'test 9/21/2009 09:28'!
resetFixedTestMethods
	^ self clientMethod resetFixedTestMethods! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 9/9/2009 16:45'!
statisticsForCoveredMethod: aCompiledMethod
	^ self clientMethod statisticsForCoveredMethod: aCompiledMethod! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 11/27/2009 14:43'!
testsAffectedByAddition
	^ self clientMethod testsAffectedByAddition! !

!CSCoverageTracingWrapper methodsFor: 'client method forwarding' stamp: 'bs 11/27/2009 14:45'!
testsAffectedByModification
	^ self clientMethod testsAffectedByModification! !

!CSCoverageTracingWrapper methodsFor: 'evaluating' stamp: 'bs 10/8/2009 15:51'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	self beforeMethodWithReceiver: anObject.
	^ [
		self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
	] ensure: [self afterMethod].! !

MwMethodWrapper variableByteSubclass: #CSRefactoringVariationWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeSight-TestTracer'!

!CSRefactoringVariationWrapper class methodsFor: 'as yet unclassified' stamp: 'bs 12/8/2009 22:59'!
initialize
	"self initialize"
	super initialize! !

!CSRefactoringVariationWrapper class methodsFor: 'as yet unclassified' stamp: 'bs 12/8/2009 21:58'!
on: aSelectorSymbol inClass: aClass withCode: sourceCodeString
	| wrapper variationAst |
	wrapper := super on: aSelectorSymbol inClass: aClass.
	variationAst := Compiler new 
						compile: sourceCodeString in: aClass
						notifying: nil
						ifFail: [].
	wrapper variationMethod: (variationAst generateWith: #(0 0 0 0) using: aClass).
	^ wrapper! !

!CSRefactoringVariationWrapper methodsFor: 'as yet unclassified' stamp: 'bs 12/8/2009 23:00'!
valueWithReceiver: anObject arguments: anArrayOfObjects
	^ self variationMethod valueWithReceiver: anObject arguments: anArrayOfObjects.! !

!CSRefactoringVariationWrapper methodsFor: 'as yet unclassified' stamp: 'bs 12/8/2009 21:57'!
variationMethod
	^ self properties at: #codeSightVariationMethod ifAbsent: []! !

!CSRefactoringVariationWrapper methodsFor: 'as yet unclassified' stamp: 'bs 12/8/2009 21:58'!
variationMethod: aCompiledMethod
	self properties at: #codeSightVariationMethod put: aCompiledMethod! !

!MwMethodWrapper methodsFor: '*CodeSight' stamp: 'bs 9/23/2009 16:10'!
compiledMethod
	^ self clientMethod! !

!MwMethodWrapper methodsFor: '*CodeSight' stamp: 'bs 12/22/2009 14:29'!
installAsLastInChain: aMethodWrapper

	self clientMethod isMwMethodWrapper
		ifTrue: [ self clientMethod installAsLastInChain: aMethodWrapper]
		ifFalse: [ 
			self assert: [self wasInstalledClosestToMethod not] 
				description: 'there is already a wrapper installed closest to the method'.
			
			self installBeforeClientMethod: aMethodWrapper.
			aMethodWrapper markAsInstalledClosestToMethod ].! !

!MwMethodWrapper methodsFor: '*CodeSight' stamp: 'bs 12/8/2009 20:39'!
installBeforeClientMethod: aMethodWrapper
	aMethodWrapper 
		clientMethod: self clientMethod;
		setSourcePointer: self sourcePointer;
		definingClass: self definingClass;
		wrappingWrapper: self.
			
	self clientMethod: aMethodWrapper! !

!MwMethodWrapper methodsFor: '*CodeSight' stamp: 'bs 12/22/2009 14:30'!
installClosestToMethod
	| holdingClass clientMethod |
	self wasActive ifTrue: [self error: 'Einwegwrapper!!'].
	holdingClass := self wrappedClass whichClassIncludesSelector: self clientSelector.
	holdingClass isNil ifTrue: [^ self].
	clientMethod := holdingClass compiledMethodAt: self clientSelector.
	clientMethod == self ifTrue: [^ self].
	
	clientMethod isMwMethodWrapper
		ifTrue: [
			clientMethod installAsLastInChain: self ]
		ifFalse: [
			self 
				definingClass: holdingClass;
				clientMethod: clientMethod;
				setSourcePointer: clientMethod sourcePointer.
		
			self wrappedClass addSelectorSilently: self clientSelector withMethod: self.
			self markAsInstalledClosestToMethod ].! !

!MwMethodWrapper methodsFor: '*CodeSight' stamp: 'bs 12/8/2009 20:46'!
markAsInstalledClosestToMethod
	self properties at: #mwInstalledClosestToMethod put: true.! !

!MwMethodWrapper methodsFor: '*CodeSight' stamp: 'bs 12/8/2009 20:41'!
wasInstalledClosestToMethod
	^ self properties at: #mwInstalledClosestToMethod ifAbsent: [false]! !
CSFailedExpressionStyler initialize!
CSTraceChooser initialize!
CSTestRunner initialize!
CSCoverageTracingWrapper initialize!
CSRefactoringVariationWrapper initialize!
